@@ -91,7 +91,7 @@ static void initRoleObject (roleObject *robj, roleDefinition *rdef, freeRoleDefF
 #ifdef DEBUG
 	size_t len = strlen (rdef->name);
 	for (int i = 0; i < len; i++)
-		Assert (isalnum (rdef->name [i]));
+		Assert (isalnum ((unsigned char) rdef->name [i]));
 #endif
 	robj->def = rdef;
 	robj->free = freefunc;
@@ -137,6 +137,11 @@ extern struct kindControlBlock* allocKindControlBlock (parserDefinition *parser)
 	{
 		kindObject *kind = kcb->kind + i;
 		kind->def = parser->kindTable + i;
+
+		Assert (kind->def->letter != KIND_FILE_DEFAULT_LETTER);
+		Assert (kind->def->name == NULL /* SELF (RUNTIME) TEST NEEDS THIS. */
+				|| strcmp(kind->def->name, KIND_FILE_DEFAULT_NAME));
+
 		kind->free = NULL;
 		kind->def->id = i;
 		kind->rcb = allocRoleControlBlock (kind);
