@@ -552,7 +552,7 @@ extern int ctags_cli_main (int argc CTAGS_ATTR_UNUSED, char **argv)
 {
 	cookedArgs *args;
 
-#if defined(WIN32) && defined(HAVE_MKSTEMP)
+#if defined(_WIN32) && defined(HAVE_MKSTEMP)
 	/* MinGW-w64's mkstemp() uses rand() for generating temporary files. */
 	srand ((unsigned int) clock ());
 #endif
