@@ -26,6 +26,7 @@
 #include "routines_p.h"
 #include "options_p.h"
 #include "parse_p.h"
+#include "promise.h"
 #include "promise_p.h"
 #include "stats_p.h"
 #include "trace.h"
@@ -68,6 +69,7 @@ typedef struct sComputPos {
 	long    offset;
 	bool open;
 	int crAdjustment;
+	size_t posInAllLines;
 } compoundPos;
 
 typedef struct sInputLineFposMap {
@@ -148,7 +150,24 @@ extern unsigned long getInputLineNumber (void)
 extern int getInputLineOffset (void)
 {
 	unsigned char *base = (unsigned char *) vStringValue (File.line);
-	int ret = File.currentLine - base - File.ungetchIdx;
+	int ret;
+
+	if (File.currentLine)
+		ret = File.currentLine - base - File.ungetchIdx;
+	else if (File.input.lineNumber)
+	{
+		/* When EOF is saw, currentLine is set to NULL.
+		 * So the way to calculate the offset at the end of file is tricky.
+		 */
+		ret = (mio_tell (File.mio) - (File.bomFound? 3: 0))
+			- getInputFileOffsetForLine(File.input.lineNumber);
+	}
+	else
+	{
+		/* At the first line of file. */
+		ret = mio_tell (File.mio) - (File.bomFound? 3: 0);
+	}
+
 	return ret >= 0 ? ret : 0;
 }
 
@@ -191,7 +210,9 @@ extern MIOPos getInputFilePositionForLine (unsigned int line)
 extern long getInputFileOffsetForLine (unsigned int line)
 {
 	compoundPos *cpos = getInputFileCompoundPosForLine (line);
-	return cpos->offset - (File.bomFound? 3: 0);
+	long r = cpos->offset - (File.bomFound? 3: 0) - cpos->crAdjustment;
+	Assert (r >= 0);
+	return r;
 }
 
 extern langType getInputLanguage (void)
@@ -245,9 +266,9 @@ extern bool doesInputLanguageAllowNullTag (void)
 	return doesLanguageAllowNullTag (getInputLanguage ());
 }
 
-extern bool doesInputLanguageRequestAutomaticFQTag (void)
+extern bool doesInputLanguageRequestAutomaticFQTag (const tagEntryInfo *e)
 {
-	return doesLanguageRequestAutomaticFQTag (getInputLanguage ());
+	return doesLanguageRequestAutomaticFQTag (e->langType);
 }
 
 extern const char *getSourceFileTagPath (void)
@@ -316,8 +337,14 @@ static void allocLineFposMap (inputLineFposMap *lineFposMap)
 	lineFposMap->count = 0;
 }
 
+static void resetLineFposMap (inputLineFposMap *lineFposMap)
+{
+	memset(lineFposMap->pos, 0, sizeof(compoundPos) * lineFposMap->size);
+	lineFposMap->count = 0;
+}
+
 static void appendLineFposMap (inputLineFposMap *lineFposMap, compoundPos *pos,
-							   bool crAdjustment)
+							   bool crAdjustment, size_t posInAllLines)
 {
 	int lastCrAdjustment = 0;
 
@@ -339,6 +366,7 @@ static void appendLineFposMap (inputLineFposMap *lineFposMap, compoundPos *pos,
 	lineFposMap->pos [lineFposMap->count].open = true;
 	lineFposMap->pos [lineFposMap->count].crAdjustment
 		= lastCrAdjustment + ((crAdjustment)? 1: 0);
+	lineFposMap->pos [lineFposMap->count].posInAllLines = posInAllLines;
 	lineFposMap->count++;
 }
 
@@ -436,7 +464,7 @@ static void resetLangOnStack (inputLangInfo *langInfo, langType lang)
 	langStackPush (& (langInfo->stack), lang);
 }
 
-extern langType baseLangOnStack (inputLangInfo *langInfo)
+static langType baseLangOnStack (inputLangInfo *langInfo)
 {
 	Assert (langInfo->stack.count > 0);
 	return langStackBotom (& (langInfo->stack));
@@ -509,7 +537,7 @@ static unsigned long readLineNumber (char **str)
 
 	skipWhite (str);
 	s = *str;
-	while (*s != '\0' && isdigit (*s))
+	while (*s != '\0' && isdigit ((unsigned char) *s))
 	{
 		lNum = (lNum * 10) + (*s - '0');
 		s++;
@@ -558,7 +586,7 @@ static bool parseLineDirective (char *s)
 	skipWhite (&s);
 	DebugStatement ( const char* lineStr = ""; )
 
-	if (isdigit (*s))
+	if (isdigit ((unsigned char) *s))
 		result = true;
 	else if (strncmp (s, "line", 4) == 0)
 	{
@@ -739,8 +767,8 @@ extern bool openInputFile (const char *const fileName, const langType language,
 		File.filePosition.offset = StartOfLine.offset = mio_tell (File.mio);
 		File.currentLine  = NULL;
 
-		if (File.line != NULL)
-			vStringClear (File.line);
+		File.line = vStringNewOrClear (File.line);
+		File.ungetchIdx = 0;
 
 		setInputFileParameters  (vStringNewInit (fileName), language);
 		File.input.lineNumberOrigin = 0L;
@@ -767,7 +795,7 @@ extern time_t getInputFileMtime (void)
 	return File.mtime;
 }
 
-extern void resetInputFile (const langType language)
+extern void resetInputFile (const langType language, bool resetLineFposMap_)
 {
 	Assert (File.mio);
 
@@ -777,11 +805,17 @@ extern void resetInputFile (const langType language)
 	File.filePosition.offset = StartOfLine.offset = mio_tell (File.mio);
 	File.currentLine  = NULL;
 
-	if (File.line != NULL)
-		vStringClear (File.line);
-	if (hasLanguageMultilineRegexPatterns (language))
+	Assert (File.line);
+	vStringClear (File.line);
+	File.ungetchIdx = 0;
+
+	if (hasLanguageMultilineRegexPatterns (language)
+		|| hasLanguagePostRunRegexPatterns (language))
 		File.allLines = vStringNew ();
 
+	if (resetLineFposMap_)
+		resetLineFposMap(&File.lineFposMap);
+
 	resetLangOnStack (& inputLang, language);
 	File.input.lineNumber = File.input.lineNumberOrigin;
 	sourceLang = language;
@@ -815,13 +849,13 @@ extern void *getInputFileUserData(void)
 
 /*  Action to take for each encountered input newline.
  */
-static void fileNewline (bool crAdjustment)
+static void fileNewline (bool crAdjustment, size_t posInAllLines)
 {
 	File.filePosition = StartOfLine;
 
 	if (BackupFile.mio == NULL)
 		appendLineFposMap (&File.lineFposMap, &File.filePosition,
-						   crAdjustment);
+						   crAdjustment, posInAllLines);
 
 	File.input.lineNumber++;
 	File.source.lineNumber++;
@@ -899,15 +933,13 @@ static vString *iFileGetLine (bool chop_newline)
 	eolType eol;
 	langType lang = getInputLanguage();
 
-	if (File.line == NULL)
-		File.line = vStringNew ();
-
+	Assert (File.line);
 	eol = readLine (File.line, File.mio);
 
 	if (vStringLength (File.line) > 0)
 	{
 		/* Use StartOfLine from previous iFileGetLine() call */
-		fileNewline (eol == eol_cr_nl);
+		fileNewline (eol == eol_cr_nl, File.allLines? vStringLength(File.allLines): 0);
 		/* Store StartOfLine for the next iFileGetLine() call */
 		mio_getpos (File.mio, &StartOfLine.pos);
 		StartOfLine.offset = mio_tell (File.mio);
@@ -920,7 +952,7 @@ static vString *iFileGetLine (bool chop_newline)
 
 		bool chopped = vStringStripNewline (File.line);
 
-		matchLanguageRegex (lang, File.line);
+		matchLanguageRegex (lang, File.line, false);
 
 		if (chopped && !chop_newline)
 			vStringPutNewlinAgainUnsafe (File.line);
@@ -934,6 +966,36 @@ static vString *iFileGetLine (bool chop_newline)
 			matchLanguageMultilineRegex (lang, File.allLines);
 			matchLanguageMultitableRegex (lang, File.allLines);
 
+			if (hasLanguagePostRunRegexPatterns (lang))
+			{
+
+				unsigned input_ln = File.input.lineNumber;
+				unsigned source_ln = File.source.lineNumber;
+				MIOPos pos = File.filePosition.pos;
+
+				vString *line = vStringNew();
+				for (size_t i = 0; i < File.lineFposMap.count; i++)
+				{
+					File.input.lineNumber = i + 1;
+					File.source.lineNumber = File.input.lineNumber;
+					File.filePosition.pos = File.lineFposMap.pos[i].pos;
+
+					vStringNCopySUnsafe(line,
+										vStringValue(File.allLines) +
+										File.lineFposMap.pos[i].posInAllLines,
+										(((i + 1) < File.lineFposMap.count)
+										 ? File.lineFposMap.pos[i+1].posInAllLines
+										 : vStringLength (File.allLines))
+										- File.lineFposMap.pos[i].posInAllLines);
+					matchLanguageRegex (lang, line, true);
+				}
+				vStringDelete(line);
+
+				File.filePosition.pos = pos;
+				File.input.lineNumber = input_ln;
+				File.source.lineNumber = source_ln;
+			}
+
 			/* To limit the execution of multiline/multitable parser(s) only
 			   ONCE, clear File.allLines field. */
 			vStringDelete (File.allLines);
@@ -1022,18 +1084,25 @@ extern int skipToCharacterInInputFile2 (int c0, int c1)
  *  the terminating newline. A NULL return value means that all lines in the
  *  file have been read and we are at the end of file.
  */
-extern const unsigned char *readLineFromInputFile (void)
+extern const unsigned char *readLineFromInputFileWithLength (size_t *length)
 {
 	vString* const line = iFileGetLine (true);
 	const unsigned char* result = NULL;
 	if (line != NULL)
 	{
 		result = (const unsigned char*) vStringValue (line);
+		*length = vStringLength (line);
 		DebugStatement ( debugPrintf (DEBUG_READ, "%s\n", result); )
 	}
 	return result;
 }
 
+extern const unsigned char *readLineFromInputFile (void)
+{
+	size_t dummy;
+	return readLineFromInputFileWithLength(&dummy);
+}
+
 /*
  *   Raw file line reading with automatic buffer sizing
  */
@@ -1075,7 +1144,8 @@ extern char *readLineFromBypass (
 	return result;
 }
 
-extern void   pushNarrowedInputStream (
+extern bool   pushNarrowedInputStream (
+				       bool useMemoryStreamInput,
 				       unsigned long startLine, long startCharOffset,
 				       unsigned long endLine, long endCharOffset,
 				       unsigned long sourceLineOffset,
@@ -1090,26 +1160,57 @@ extern void   pushNarrowedInputStream (
 						  endLine, endCharOffset,
 						  sourceLineOffset))
 	{
-		File.thinDepth++;
-		verbose ("push thin stream (%d)\n", File.thinDepth);
-		return;
+		if ((!useMemoryStreamInput
+			 || mio_memory_get_data (File.mio, NULL)))
+		{
+			File.thinDepth++;
+			verbose ("push thin stream (%d)\n", File.thinDepth);
+			return true;
+		}
+		error(WARNING, "INTERNAL ERROR: though pushing thin MEMORY stream, "
+			  "underlying input stream is a FILE stream: %s@%s",
+			  vStringValue (File.input.name), vStringValue (File.input.tagPath));
+		AssertNotReached ();
 	}
 	Assert (File.thinDepth == 0);
 
 	original = getInputFilePosition ();
 
 	tmp = getInputFilePositionForLine (startLine);
-	mio_setpos (File.mio, &tmp);
-	mio_seek (File.mio, startCharOffset, SEEK_CUR);
+	if (mio_setpos (File.mio, &tmp) != 0)
+		goto fail;
+	if (mio_seek (File.mio, startCharOffset, SEEK_CUR) != 0)
+		goto fail;
 	p = mio_tell (File.mio);
 
 	tmp = getInputFilePositionForLine (endLine);
-	mio_setpos (File.mio, &tmp);
-	mio_seek (File.mio, endCharOffset, SEEK_CUR);
+	if (mio_setpos (File.mio, &tmp) != 0)
+		goto fail;
+	if (endCharOffset == EOL_CHAR_OFFSET)
+	{
+		long line_start = mio_tell (File.mio);
+		vString *tmpstr = vStringNew ();
+		readLine (tmpstr, File.mio);
+		endCharOffset = mio_tell (File.mio) - line_start;
+		vStringDelete (tmpstr);
+		Assert (endCharOffset >= 0);
+	}
+	else
+	{
+		if (mio_seek (File.mio, endCharOffset, SEEK_CUR) != 0)
+			goto fail;
+	}
 	q = mio_tell (File.mio);
 
+	if (q < p || p == -1 || q == -1)
+		goto fail;
+
 	mio_setpos (File.mio, &original);
 
+	/* nothing to do */
+	if (q == p)
+		return false;
+
 	invalidatePatternCache();
 
 	size_t size = q - p;
@@ -1134,6 +1235,15 @@ extern void   pushNarrowedInputStream (
 
 	File.input.lineNumberOrigin = ((startLine == 0)? 0: startLine - 1);
 	File.source.lineNumberOrigin = ((sourceLineOffset == 0)? 0: sourceLineOffset - 1);
+
+	return true;
+
+fail:
+	error (WARNING, "Failed to push narrowed input stream when parsing %s: IO error or invalid offsets: "
+	       "start(line: %lu, offset: %ld, srcline: %lu), end(line: %lu, offset: %ld)",
+	       getInputFileName (), startLine, startCharOffset, sourceLineOffset, endLine, endCharOffset);
+	mio_setpos (File.mio, &original);
+	return false;
 }
 
 extern bool doesParserRunAsGuest (void)
@@ -1155,10 +1265,10 @@ extern unsigned int getNestedInputBoundaryInfo (unsigned long lineNumber)
 	info = 0;
 	if (File.nestedInputStreamInfo.startLine == lineNumber
 	    && File.nestedInputStreamInfo.startCharOffset != 0)
-		info |= BOUNDARY_START;
+		info |= INPUT_BOUNDARY_START;
 	if (File.nestedInputStreamInfo.endLine == lineNumber
 	    && File.nestedInputStreamInfo.endCharOffset != 0)
-		info |= BOUNDARY_END;
+		info |= INPUT_BOUNDARY_END;
 
 	return info;
 }
