@@ -23,8 +23,8 @@
 */
 
 enum nestedInputBoundaryFlag {
-	BOUNDARY_START = 1UL << 0,
-	BOUNDARY_END   = 1UL << 1,
+	INPUT_BOUNDARY_START = 1UL << 0,
+	INPUT_BOUNDARY_END   = 1UL << 1,
 };
 
 /*
@@ -40,7 +40,7 @@ extern unsigned int countInputLanguageKinds (void);
 extern unsigned int countInputLanguageRoles (int kindIndex);
 
 extern bool doesInputLanguageAllowNullTag (void);
-extern bool doesInputLanguageRequestAutomaticFQTag (void);
+extern bool doesInputLanguageRequestAutomaticFQTag (const tagEntryInfo *e);
 extern bool doesParserRunAsGuest (void);
 extern bool doesSubparserRun (void);
 extern langType getLanguageForBaseParser (void);
@@ -55,7 +55,7 @@ extern void freeInputFileResources (void);
 extern bool openInputFile (const char *const fileName, const langType language, MIO *mio, time_t mtime);
 extern MIO *getMio (const char *const fileName, const char *const openMode,
 				    bool memStreamRequired);
-extern void resetInputFile (const langType language);
+extern void resetInputFile (const langType language, bool resetLineFposMap_);
 extern void closeInputFile (void);
 extern void *getInputFileUserData(void);
 
@@ -68,7 +68,8 @@ extern time_t getInputFileMtime (void);
 
 /* Bypass: reading from fp in inputFile WITHOUT updating fields in input fields */
 extern char *readLineFromBypass (vString *const vLine, MIOPos location, long *const pSeekValue);
-extern void   pushNarrowedInputStream (
+extern bool   pushNarrowedInputStream (
+				       bool useMemoryStreamInput,
 				       unsigned long startLine, long startCharOffset,
 				       unsigned long endLine, long endCharOffset,
 				       unsigned long sourceLineOffset,
