@@ -15,19 +15,37 @@ typedef int langType;
 struct sTagEntryInfo;
 typedef struct sTagEntryInfo tagEntryInfo;
 
-struct sFieldDesc;
-typedef struct sFieldDesc fieldDesc;
-
 struct sPtagDesc;
 typedef struct sPtagDesc ptagDesc;
 
+struct sRoleDefinition;
+typedef struct sRoleDefinition roleDefinition;
+
 struct sKindDefinition;
 typedef struct sKindDefinition kindDefinition;
 
 struct sParserDefinition;
 typedef struct sParserDefinition parserDefinition;
 
-struct sPtrArray;
-typedef struct sPtrArray ptrArray;
+struct _MIO;
+typedef const char * (*selectLanguage) (struct _MIO *, langType *, unsigned int);
+
+struct sSlaveParser;
+typedef struct sSlaveParser slaveParser;
+
+struct sSubparser;
+typedef struct sSubparser subparser;
+
+struct sParserDependency;
+typedef struct sParserDependency parserDependency;
+
+struct sFieldDefinition;
+typedef struct sFieldDefinition fieldDefinition;
+
+struct sXtagDefinition;
+typedef struct sXtagDefinition xtagDefinition;
+
+struct sParamDefinition;
+typedef struct sParamDefinition paramDefinition;
 
 #endif	/* CTAGS_MAIN_TYPES_H */
