@@ -37,13 +37,13 @@ extern bool isXtagFixed (xtagType type);
 extern bool isCommonXtag (xtagType type);
 
 /* Return LANG_IGNORE for common fields. */
-extern langType getXtagOwner (xtagType type);
+extern langType getXtagLanguage (xtagType type);
 
 extern const char* getXtagName (xtagType type);
 extern const char* getXtagDescription (xtagType type);
 
 extern void initXtagObjects (void);
-extern int countXtags (void);
+extern unsigned int countXtags (void);
 
 extern int defineXtag (xtagDefinition *def, langType language);
 extern xtagType nextSiblingXtag (xtagType type);
