@@ -180,7 +180,7 @@ static void cblppAppendLine (vString *buffer,
 			if (*indicator == '-')
 			{
 				vStringStripTrailing (buffer);
-				while (isspace (*lineStart))
+				while (isspace ((unsigned char) *lineStart))
 					lineStart++;
 			}
 
@@ -242,13 +242,12 @@ static void initCOBOLRefTagEntry (tagEntryInfo *e, const char *name,
 								  const cobolKind kind, const int role)
 {
 	initRefTagEntry (e, name, kind, role);
-	e->lineNumber = CblInputState.lineNumber;
-	e->filePosition = CblInputState.filePosition;
+	updateTagLine (e, CblInputState.lineNumber, CblInputState.filePosition);
 }
 
 static void initCOBOLTagEntry (tagEntryInfo *e, const char *name, const cobolKind kind)
 {
-	initCOBOLRefTagEntry (e, name, kind, ROLE_INDEX_DEFINITION);
+	initCOBOLRefTagEntry (e, name, kind, ROLE_DEFINITION_INDEX);
 }
 
 static int makeCOBOLRefTag (const char *name, const cobolKind kind, const int role)
@@ -267,7 +266,7 @@ static int makeCOBOLRefTag (const char *name, const cobolKind kind, const int ro
 
 static int makeCOBOLTag (const char *name, const cobolKind kind)
 {
-	return makeCOBOLRefTag (name, kind, ROLE_INDEX_DEFINITION);
+	return makeCOBOLRefTag (name, kind, ROLE_DEFINITION_INDEX);
 }
 
 #define CBL_NL(nl) (*((unsigned int *) (nestingLevelGetUserData (nl))))
@@ -328,14 +327,14 @@ static void findCOBOLTags (const CobolFormat format)
 	do { \
 		const char READ_LITERAL__q = isQuote (*line) ? *line++ : 0; \
 		READ_WHILE (word, (READ_LITERAL__q && READ_LITERAL__q != *line) || \
-		                   isIdentifierChar (*line)); \
+		                   isIdentifierChar ((unsigned char) *line)); \
 		if (READ_LITERAL__q && READ_LITERAL__q == *line) \
 			line++; \
 		keyword = lookupCaseKeyword (word, Lang_cobol); \
 	} while (0)
 #define READ_WORD(word, keyword) \
 	do { \
-		READ_WHILE (word, isIdentifierChar (*line)); \
+		READ_WHILE (word, isIdentifierChar ((unsigned char) *line)); \
 		keyword = lookupCaseKeyword (word, Lang_cobol); \
 	} while (0)
 #define READ_KEYWORD(keyword) \
@@ -343,7 +342,8 @@ static void findCOBOLTags (const CobolFormat format)
 		char READ_KEYWORD__word[64]; \
 		READ_WORD (READ_KEYWORD__word, keyword); \
 	} while (0)
-#define SKIP_SPACES() do { while (isspace (*line)) line++; } while (0)
+#define SKIP_SPACES() \
+	do { while (isspace ((unsigned char) *line)) line++; } while (0)
 
 		SKIP_SPACES ();
 		READ_WORD (word, keyword);
@@ -498,7 +498,7 @@ static parserDefinition* commonCobolParserDefinition (const char *name,
 	def->kindCount = ARRAY_SIZE(CobolKinds);
 	def->keywordTable = cobolKeywordTable;
 	def->keywordCount = ARRAY_SIZE(cobolKeywordTable);
-	def->useCork = true;
+	def->useCork = CORK_QUEUE;
 	return def;
 }
 
