@@ -25,6 +25,7 @@
 #include "vstring.h"
 #include "param.h"
 #include "parse.h"
+#include "promise.h"
 #include "xtag.h"
 
 #include "cxx/cxx_debug.h"
@@ -38,6 +39,25 @@
 /*
 *   DATA DECLARATIONS
 */
+enum eCppCharacters {
+	/* white space characters */
+	SPACE         = ' ',
+	NEWLINE       = '\n',
+	CRETURN       = '\r',
+	FORMFEED      = '\f',
+	TAB           = '\t',
+	VTAB          = '\v',
+
+	/* some hard to read characters */
+	DOUBLE_QUOTE  = '"',
+	SINGLE_QUOTE  = '\'',
+	BACKSLASH     = '\\',
+
+	/* symbolic representations, above 0xFF not to conflict with any byte */
+	STRING_SYMBOL = CPP_STRING_SYMBOL,
+	CHAR_SYMBOL   = CPP_CHAR_SYMBOL
+};
+
 typedef enum { COMMENT_NONE, COMMENT_C, COMMENT_CPLUS, COMMENT_D } Comment;
 
 enum eCppLimits {
@@ -45,6 +65,21 @@ enum eCppLimits {
 	MaxDirectiveName = 10
 };
 
+/* For tracking __ASSEMBLER__ area. */
+enum eIfSubstate {
+	IF_IF,
+	IF_IFDEF,
+	IF_IFNDEF,
+	IF_ELSE,
+	IF_ELIF,
+	IF_ENDIF,
+};
+
+struct asmAreaInfo {
+	enum eIfSubstate ifSubstate;
+	unsigned long line;
+};
+
 /*  Defines the one nesting level of a preprocessor conditional.
  */
 typedef struct sConditionalInfo {
@@ -53,6 +88,9 @@ typedef struct sConditionalInfo {
 	bool branchChosen;       /* branch already selected */
 	bool ignoring;           /* current ignore state */
 	int enterExternalParserBlockNestLevel;          /* the parser state when entering this conditional: used only by cxx */
+
+	/* tracking __ASSEMBLER__ area */
+	struct asmAreaInfo asmArea;
 } conditionalInfo;
 
 enum eState {
@@ -60,6 +98,7 @@ enum eState {
 	DRCTV_DEFINE,  /* "#define" encountered */
 	DRCTV_HASH,    /* initial '#' read; determine directive */
 	DRCTV_IF,      /* "#if" or "#ifdef" encountered */
+	DRCTV_ELIF,    /* "#elif" encountered */
 	DRCTV_PRAGMA,  /* #pragma encountered */
 	DRCTV_UNDEF,   /* "#undef" encountered */
 	DRCTV_INCLUDE, /* "#include" encountered */
@@ -88,6 +127,7 @@ typedef struct sCppState {
 	bool useClientLangDefineMacroKindIndex;
 	int defineMacroKindIndex;
 	int macroUndefRoleIndex;
+	int macroConditionRoleIndex;
 
 	bool useClientLangMacroParamKindIndex;
 	int macroParamKindIndex;
@@ -101,6 +141,8 @@ typedef struct sCppState {
 
 	struct sDirective {
 		enum eState state;       /* current directive being processed */
+		enum eIfSubstate ifsubstate; /* For tracking __ASSEMBLER__.
+									  * assigned only when state == DICTV_IF */
 		bool	accept;          /* is a directive syntactically permitted? */
 		vString * name;          /* macro name */
 		unsigned int nestLevel;  /* level 0 is not used */
@@ -115,10 +157,12 @@ typedef struct sCppState {
 
 typedef enum {
 	CPREPRO_MACRO_KIND_UNDEF_ROLE,
+	CPREPRO_MACRO_KIND_CONDITION_ROLE,
 } cPreProMacroRole;
 
 static roleDefinition CPREPROMacroRoles [] = {
 	RoleTemplateUndef,
+	RoleTemplateCondition,
 };
 
 
@@ -201,6 +245,7 @@ static cppState Cpp = {
 	.useClientLangDefineMacroKindIndex = false,
 	.defineMacroKindIndex = CPREPRO_MACRO,
 	.macroUndefRoleIndex = CPREPRO_MACRO_KIND_UNDEF_ROLE,
+	.macroConditionRoleIndex = CPREPRO_MACRO_KIND_CONDITION_ROLE,
 	.useClientLangMacroParamKindIndex = false,
 	.macroParamKindIndex = CPREPRO_PARAM,
 	.useClientLangHeaderKindIndex = false,
@@ -251,6 +296,7 @@ static void cppInitCommon(langType clientLang,
 		     const bool hasSingleQuoteLiteralNumbers,
 		     int defineMacroKindIndex,
 		     int macroUndefRoleIndex,
+		     int macroConditionRoleIndex,
 		     int macroParamKindIndex,
 		     int headerKindIndex,
 		     int headerSystemRoleIndex, int headerLocalRoleIndex,
@@ -288,6 +334,7 @@ static void cppInitCommon(langType clientLang,
 		Cpp.useClientLangDefineMacroKindIndex = true;
 
 		Cpp.macroUndefRoleIndex = macroUndefRoleIndex;
+		Cpp.macroConditionRoleIndex = macroConditionRoleIndex;
 		Cpp.macrodefFieldIndex = macrodefFieldIndex;
 	}
 	else
@@ -296,6 +343,7 @@ static void cppInitCommon(langType clientLang,
 		Cpp.useClientLangDefineMacroKindIndex = false;
 
 		Cpp.macroUndefRoleIndex = CPREPRO_MACRO_KIND_UNDEF_ROLE;
+		Cpp.macroConditionRoleIndex = CPREPRO_MACRO_KIND_CONDITION_ROLE;
 		Cpp.macrodefFieldIndex = CPreProFields [F_MACRODEF].ftype;
 	}
 
@@ -355,6 +403,7 @@ extern void cppInit (const bool state, const bool hasAtLiteralStrings,
 		     const bool hasSingleQuoteLiteralNumbers,
 		     int defineMacroKindIndex,
 		     int macroUndefRoleIndex,
+		     int macroConditionRoleIndex,
 		     int macroParamKindIndex,
 		     int headerKindIndex,
 		     int headerSystemRoleIndex, int headerLocalRoleIndex,
@@ -364,7 +413,8 @@ extern void cppInit (const bool state, const bool hasAtLiteralStrings,
 
 	cppInitCommon (client, state, hasAtLiteralStrings,
 				   hasCxxRawLiteralStrings, hasSingleQuoteLiteralNumbers,
-				   defineMacroKindIndex, macroUndefRoleIndex, macroParamKindIndex,
+				   defineMacroKindIndex, macroUndefRoleIndex, macroConditionRoleIndex,
+				   macroParamKindIndex,
 				   headerKindIndex, headerSystemRoleIndex, headerLocalRoleIndex,
 				   macrodefFieldIndex);
 }
@@ -430,6 +480,20 @@ extern void cppEndStatement (void)
 /*  This puts a character back into the input queue for the input File. */
 extern void cppUngetc (const int c)
 {
+	if (c == STRING_SYMBOL || c == CHAR_SYMBOL)
+	{
+		Assert(Cpp.charOrStringContents != NULL);
+		cppUngetc(c == STRING_SYMBOL ? '"' : '\'');
+		cppUngetString(vStringValue(Cpp.charOrStringContents), vStringLength(Cpp.charOrStringContents));
+		cppUngetc(c == STRING_SYMBOL ? '"' : '\'');
+		vStringClear(Cpp.charOrStringContents);
+		return;
+	}
+	else if (c == EOF)
+	{
+		return;
+	}
+
 	if(!Cpp.ungetPointer)
 	{
 		// no unget data
@@ -467,7 +531,7 @@ extern void cppUngetc (const int c)
 	Cpp.ungetDataSize++;
 }
 
-int cppUngetBufferSize()
+int cppUngetBufferSize(void)
 {
 	return Cpp.ungetBufferSize;
 }
@@ -716,6 +780,7 @@ static bool pushConditional (const bool firstBranchChosen)
 				! firstBranchChosen  &&  ! BraceFormat  &&
 				(ifdef->singleBranch || !doesExaminCodeWithInIf0Branch)));
 		ifdef->enterExternalParserBlockNestLevel = externalParserBlockNestLevel;
+		ifdef->asmArea.line = 0;
 		ignoreBranch = ifdef->ignoring;
 	}
 	return ignoreBranch;
@@ -826,7 +891,7 @@ static void makeIncludeTag (const  char *const name, bool systemHeader)
 
 	if (isLanguageRoleEnabled(lang, Cpp.headerKindIndex, role_index))
 	{
-		if (doesCPreProRunAsStandaloneParser (CPREPRO_HEADER))
+		if (standing_alone)
 			pushLanguage (Cpp.lang);
 
 		initRefTagEntry (&e, name, Cpp.headerKindIndex, role_index);
@@ -834,39 +899,41 @@ static void makeIncludeTag (const  char *const name, bool systemHeader)
 		e.truncateLineAfterTag = true;
 		makeTagEntry (&e);
 
-		if (doesCPreProRunAsStandaloneParser (CPREPRO_HEADER))
+		if (standing_alone)
 			popLanguage ();
 	}
 }
 
-static void makeParamTag (vString *name, short nth, bool placeholder)
+static int makeParamTag (vString *name, short nth, bool placeholder)
 {
 	bool standing_alone = doesCPreProRunAsStandaloneParser(CPREPRO_MACRO);
-	langType lang = standing_alone ? Cpp.lang: Cpp.clientLang;
 
 	Assert (Cpp.macroParamKindIndex != KIND_GHOST_INDEX);
 
-	int r;
-	pushLanguage (lang);
-	r = makeSimpleTag (name, Cpp.macroParamKindIndex);
-	popLanguage ();
+	if (standing_alone)
+		pushLanguage (Cpp.lang);
+	int r = makeSimpleTag (name, Cpp.macroParamKindIndex);
+	if (standing_alone)
+		popLanguage ();
 
 	tagEntryInfo *e = getEntryInCorkQueue (r);
 	if (e)
 	{
 		e->extensionFields.nth = nth;
 		if (placeholder)
-			e->placeholder = 1;
+			markTagAsPlaceholder (e, placeholder);
 	}
+	return r;
 }
 
-static void regenreateSignatureFromParameters (vString * buffer, int from, int to)
+static void makeSignatureStringFromParameters (vString * buffer, intArray *parameters)
 {
 	vStringPut(buffer, '(');
-	for (int pindex = from; pindex < to; pindex++)
+	for (size_t i = 0; i < intArrayCount (parameters); i++)
 	{
+		int pindex = intArrayItem (parameters, i);
 		tagEntryInfo *e = getEntryInCorkQueue (pindex);
-		if (e && !isTagExtra (e))
+		if (e)
 		{
 			vStringCatS (buffer, e->name);
 			vStringPut (buffer, ',');
@@ -907,6 +974,7 @@ static int directiveDefine (const int c, bool undef)
 
 			if (p == '(')
 			{
+				intArray *params = intArrayNew ();
 				vString *param = vStringNew ();
 				int param_start = (int)countEntryInCorkQueue();
 				do {
@@ -921,7 +989,8 @@ static int directiveDefine (const int c, bool undef)
 
 					if (vStringLength (param) > 0)
 					{
-						makeParamTag (param, nth++, vStringChar(param, 0) == '.');
+						int r = makeParamTag (param, nth++, vStringChar(param, 0) == '.');
+						intArrayAdd (params, r);
 						vStringClear (param);
 					}
 					if (p == '\\')
@@ -933,18 +1002,18 @@ static int directiveDefine (const int c, bool undef)
 				if (p == ')')
 				{
 					vString *signature = vStringNew ();
-					regenreateSignatureFromParameters (signature, param_start, param_end);
+					makeSignatureStringFromParameters (signature, params);
 					r = makeDefineTag (vStringValue (Cpp.directive.name), vStringValue (signature), undef);
 					vStringDelete (signature);
 				}
 				else
 					r = makeDefineTag (vStringValue (Cpp.directive.name), NULL, undef);
+				intArrayDelete (params);
 
 				tagEntryInfo *e = getEntryInCorkQueue (r);
 				if (e)
 				{
-					e->lineNumber = lineNumber;
-					e->filePosition = filePosition;
+					updateTagLine (e, lineNumber, filePosition);
 					patchScopeFieldOfParameters (param_start, param_end, r);
 				}
 			}
@@ -996,10 +1065,62 @@ static void directivePragma (int c)
 	Cpp.directive.state = DRCTV_NONE;
 }
 
-static bool directiveIf (const int c)
+/*
+ * __ASSEMBLER__ ("3.7.1 Standard Predefined Macros" in GNU cpp info),
+ * __ASSEMBLY__	 (Used in Linux kernel)
+ */
+static bool isAssemblerBlock (int c)
 {
+	if (c != '_')
+		return false;
+
+	bool r = false;
+	vString *cond = vStringNew ();
+	readIdentifier (c, cond);
+	if (strcmp (vStringValue (cond), "__ASSEMBLER__") == 0
+		|| strcmp (vStringValue (cond), "__ASSEMBLY__") == 0)
+		r = true;
+
+	CXX_DEBUG_PRINT("ASSEMBLER[%s]: %s", r? "true": "false", vStringValue(cond));
+
+	size_t len = vStringLength (cond);
+	/* Pushing back to the stream.
+	 * The first character is not read in this function.
+	 * So don't touch the character here. */
+	for (size_t i = len; i > 1; i--)
+	{
+		c = vStringChar (cond, i - 1);
+		cppUngetc (c);
+	}
+
+	vStringDelete (cond);
+	return r;
+}
+
+static bool directiveIf (const int c, enum eIfSubstate if_substate)
+{
+	static langType asmLang = LANG_IGNORE;
+	if (asmLang == LANG_IGNORE)
+		asmLang = getNamedLanguage ("Asm", 0);
+
 	DebugStatement ( const bool ignore0 = isIgnore (); )
-	const bool ignore = pushConditional ((bool) (c != '0'));
+	bool firstBranchChosen = (bool) (c != '0');
+	bool assemblerBlock = false;
+	if (Cpp.clientLang != asmLang && firstBranchChosen)
+	{
+		assemblerBlock = isAssemblerBlock(c);
+		if (assemblerBlock && if_substate != IF_IFNDEF)
+			firstBranchChosen = false;
+	}
+
+	CXX_DEBUG_PRINT("firstBranchChosen: %d", firstBranchChosen);
+	const bool ignore = pushConditional (firstBranchChosen);
+	if (assemblerBlock)
+	{
+		conditionalInfo *ifdef = currentConditional ();
+		ifdef->asmArea.ifSubstate = if_substate;
+		ifdef->asmArea.line = getInputLineNumber();
+	}
 
 	Cpp.directive.state = DRCTV_NONE;
 	DebugStatement ( debugCppNest (true, Cpp.directive.nestLevel);
@@ -1008,6 +1129,10 @@ static bool directiveIf (const int c)
 	return ignore;
 }
 
+static void directiveElif (const int c)
+{
+	Cpp.directive.state = DRCTV_NONE;
+}
 
 static void directiveInclude (const int c)
 {
@@ -1021,6 +1146,36 @@ static void directiveInclude (const int c)
 	Cpp.directive.state = DRCTV_NONE;
 }
 
+static void promiseOrPrepareAsm (conditionalInfo *ifdef, enum eIfSubstate currentState)
+{
+	if (!ifdef->asmArea.line)
+		return;
+
+	if (((ifdef->asmArea.ifSubstate == IF_IF || ifdef->asmArea.ifSubstate == IF_IFDEF)
+		 && (currentState == IF_ELSE || currentState == IF_ELIF || currentState == IF_ENDIF))
+		|| ((ifdef->asmArea.ifSubstate == IF_ELSE)
+			&& (currentState == IF_ENDIF)))
+	{
+		unsigned long start = ifdef->asmArea.line + 1;
+		unsigned long end = getInputLineNumber ();
+
+		if (start < end)
+			makePromise ("Asm", start, 0, end, 0, start);
+
+		ifdef->asmArea.line = 0;
+	}
+	else if (ifdef->asmArea.ifSubstate == IF_IFNDEF)
+	{
+		if (currentState == IF_ELIF)
+			ifdef->asmArea.line = 0;
+		else if (currentState == IF_ELSE)
+		{
+			ifdef->asmArea.ifSubstate = IF_ELSE;
+			ifdef->asmArea.line = getInputLineNumber ();
+		}
+	}
+}
+
 static bool directiveHash (const int c)
 {
 	bool ignore = false;
@@ -1035,19 +1190,33 @@ static bool directiveHash (const int c)
 	else if (stringMatch (directive, "undef"))
 		Cpp.directive.state = DRCTV_UNDEF;
 	else if (strncmp (directive, "if", (size_t) 2) == 0)
+	{
 		Cpp.directive.state = DRCTV_IF;
+		Cpp.directive.ifsubstate = IF_IF;
+		if (directive[2] == 'd')
+			Cpp.directive.ifsubstate = IF_IFDEF;
+		else if (directive[2] == 'n')
+			Cpp.directive.ifsubstate = IF_IFNDEF;
+	}
 	else if (stringMatch (directive, "elif")  ||
 			stringMatch (directive, "else"))
 	{
+		enum eIfSubstate s = (directive[2] == 's')? IF_ELSE: IF_ELIF;
+		conditionalInfo *ifdef = currentConditional ();
+		promiseOrPrepareAsm (ifdef, s);
+
 		ignore = setIgnore (isIgnoreBranch ());
 		CXX_DEBUG_PRINT("Found #elif or #else: ignore is %d",ignore);
-		if (! ignore  &&  stringMatch (directive, "else"))
+		if (! ignore  &&  s == IF_ELSE)
 			chooseBranch ();
-		Cpp.directive.state = DRCTV_NONE;
+		Cpp.directive.state = (s == IF_ELIF)? DRCTV_ELIF: DRCTV_NONE;
 		DebugStatement ( if (ignore != ignore0) debugCppIgnore (ignore); )
 	}
 	else if (stringMatch (directive, "endif"))
 	{
+		conditionalInfo *ifdef = currentConditional ();
+		promiseOrPrepareAsm (ifdef, IF_ENDIF);
+
 		DebugStatement ( debugCppNest (false, Cpp.directive.nestLevel); )
 		ignore = popConditional ();
 		Cpp.directive.state = DRCTV_NONE;
@@ -1063,7 +1232,7 @@ static bool directiveHash (const int c)
 
 /*  Handles a pre-processor directive whose first character is given by "c".
  */
-static bool handleDirective (const int c, int *macroCorkIndex)
+static bool handleDirective (const int c, int *macroCorkIndex, bool *inspect_conidtion)
 {
 	bool ignore = isIgnore ();
 
@@ -1074,10 +1243,17 @@ static bool handleDirective (const int c, int *macroCorkIndex)
 			*macroCorkIndex = directiveDefine (c, false);
 			break;
 		case DRCTV_HASH:    ignore = directiveHash (c);  break;
-		case DRCTV_IF:      ignore = directiveIf (c);    break;
+		case DRCTV_IF:
+			ignore = directiveIf (c, Cpp.directive.ifsubstate);
+			*inspect_conidtion = true;
+			break;
+		case DRCTV_ELIF:
+			directiveElif (c);
+			*inspect_conidtion = true;
+			break;
 		case DRCTV_PRAGMA:  directivePragma (c);         break;
-		case DRCTV_UNDEF:   directiveUndef (c);         break;
-		case DRCTV_INCLUDE: directiveInclude (c);         break;
+		case DRCTV_UNDEF:   directiveUndef (c);          break;
+		case DRCTV_INCLUDE: directiveInclude (c);        break;
 	}
 	return ignore;
 }
@@ -1193,10 +1369,19 @@ static int skipToEndOfString (bool ignoreBackslash)
 	{
 		if (c == BACKSLASH && ! ignoreBackslash)
 		{
-			vStringPutWithLimit (Cpp.charOrStringContents, c, 1024);
-			c = cppGetcFromUngetBufferOrFile ();  /* throw away next character, too */
-			if (c != EOF)
-				vStringPutWithLimit (Cpp.charOrStringContents, c, 1024);
+			int c0 = cppGetcFromUngetBufferOrFile ();
+			if (c0 == '\n')
+				continue;
+			if (c0 == EOF)
+				break;
+
+			if (vStringPutWithLimit (Cpp.charOrStringContents, c, 1024))
+			{
+				if (vStringPutWithLimit (Cpp.charOrStringContents, c0, 1024))
+					continue;
+				/* delete the last back slash at the end of the vstring. */
+				vStringChop(Cpp.charOrStringContents);
+			}
 		}
 		else if (c == DOUBLE_QUOTE)
 			break;
@@ -1259,7 +1444,7 @@ static int skipToEndOfCxxRawLiteralString (void)
  *  special character to symbolically represent a generic character.
  *  Also detects Vera numbers that include a base specifier (ie. 'b1010).
  */
-static int skipToEndOfChar ()
+static int skipToEndOfChar (void)
 {
 	int c;
 	int count = 0, veraBase = '\0';
@@ -1271,10 +1456,19 @@ static int skipToEndOfChar ()
 	    ++count;
 		if (c == BACKSLASH)
 		{
-			vStringPutWithLimit (Cpp.charOrStringContents, c, 10);
-			c = cppGetcFromUngetBufferOrFile ();  /* throw away next character, too */
-			if (c != EOF)
-				vStringPutWithLimit (Cpp.charOrStringContents, c, 10);
+			int c0 = cppGetcFromUngetBufferOrFile ();
+			if (c0 == '\n')
+				continue;
+			if (c0 == EOF)
+				break;
+
+			if (vStringPutWithLimit (Cpp.charOrStringContents, c, 10))
+			{
+				if (vStringPutWithLimit (Cpp.charOrStringContents, c0, 10))
+					continue;
+				/* delete the last back slash at the end of the vstring.*/
+				vStringChop(Cpp.charOrStringContents);
+			}
 		}
 		else if (c == SINGLE_QUOTE)
 			break;
@@ -1307,14 +1501,79 @@ static int skipToEndOfChar ()
 static void attachFields (int macroCorkIndex, unsigned long endLine, const char *macrodef)
 {
 	tagEntryInfo *tag = getEntryInCorkQueue (macroCorkIndex);
-	if (!tag)
-		return;
-
-	tag->extensionFields.endLine = endLine;
-	if (macrodef)
-		attachParserFieldToCorkEntry (macroCorkIndex, Cpp.macrodefFieldIndex, macrodef);
+	if (tag)
+	{
+		setTagEndLine (tag, endLine);
+		if (macrodef)
+			attachParserField (tag, Cpp.macrodefFieldIndex, macrodef);
+	}
 }
 
+static vString * conditionMayFlush (vString* condition, bool del)
+{
+	bool standing_alone = doesCPreProRunAsStandaloneParser(CPREPRO_MACRO);
+
+	if (condition == NULL)
+		return condition;
+
+	size_t len = vStringLength(condition);
+	if (len > 0
+		&& (! (
+				(len == 7
+				 && strcmp (vStringValue (condition), "defined") == 0)
+			   )))
+	{
+		if (standing_alone)
+			pushLanguage (Cpp.lang);
+
+		makeSimpleRefTag (condition, Cpp.defineMacroKindIndex, Cpp.macroConditionRoleIndex);
+
+		if (standing_alone)
+			popLanguage ();
+	}
+
+	if (del)
+	{
+		vStringDelete (condition);
+		return NULL;
+	}
+
+	vStringClear(condition);
+	return condition;
+}
+
+static void conditionMayPut (vString *condition, int c)
+{
+	if (condition == NULL)
+		return;
+
+	if (vStringLength (condition) > 0
+		|| (!isdigit(c)))
+		vStringPut(condition, c);
+}
+
+extern void cppVStringPut (vString* string, const int c)
+{
+	if (c <= 0xff)
+		vStringPut (string, c);
+	else
+	{
+		char marker = '"';
+		switch (c)
+		{
+			case CHAR_SYMBOL:
+				marker = '\'';
+				/* Fall through */
+			case STRING_SYMBOL:
+				vStringPut (string, marker);
+				vStringCat (string, cppGetLastCharOrStringContents ());
+				vStringPut (string, marker);
+				break;
+			default:
+				AssertNotReached();
+		}
+	}
+}
 
 /*  This function returns the next character, stripping out comments,
  *  C pre-processor directives, and the contents of single and double
@@ -1328,6 +1587,7 @@ extern int cppGetc (void)
 	int c;
 	int macroCorkIndex = CORK_NIL;
 	vString *macrodef = NULL;
+	vString *condition = NULL;
 
 
 	do {
@@ -1346,6 +1606,7 @@ process:
 								  macrodef? vStringValue (macrodef): NULL);
 					macroCorkIndex = CORK_NIL;
 				}
+				condition = conditionMayFlush(condition, true);
 				break;
 
 			case TAB:
@@ -1353,9 +1614,12 @@ process:
 				if (macrodef && vStringLength (macrodef) > 0
 					&& vStringLast (macrodef) != ' ')
 					vStringPut (macrodef, ' ');
+				condition = conditionMayFlush(condition, false);
 				break;  /* ignore most white space */
 
 			case NEWLINE:
+				if (directive)
+					condition = conditionMayFlush(condition, true);
 				if (directive  &&  ! ignore)
 				{
 					directive = false;
@@ -1371,6 +1635,8 @@ process:
 				break;
 
 			case DOUBLE_QUOTE:
+				condition = conditionMayFlush(condition, false);
+
 				if (Cpp.directive.state == DRCTV_INCLUDE)
 					goto enter;
 				else
@@ -1392,6 +1658,8 @@ process:
 				break;
 
 			case '#':
+				condition = conditionMayFlush(condition, false);
+
 				if (Cpp.directive.accept)
 				{
 					directive = true;
@@ -1403,18 +1671,25 @@ process:
 				break;
 
 			case SINGLE_QUOTE:
+				condition = conditionMayFlush(condition, false);
+
 				Cpp.directive.accept = false;
 				c = skipToEndOfChar ();
 
 				/* We assume none may want to know the content of the
 				 * literal; just put ''. */
 				if (macrodef)
-					vStringCatS (macrodef, "''");
-
+				{
+					vStringPut (macrodef, '\'');
+					vStringCat (macrodef, Cpp.charOrStringContents);
+					vStringPut (macrodef, '\'');
+				}
 				break;
 
 			case '/':
 			{
+				condition = conditionMayFlush(condition, false);
+
 				const Comment comment = isComment ();
 
 				if (comment == COMMENT_C)
@@ -1438,6 +1713,8 @@ process:
 
 			case BACKSLASH:
 			{
+				condition = conditionMayFlush(condition, false);
+
 				int next = cppGetcFromUngetBufferOrFile ();
 
 				if (next == NEWLINE)
@@ -1453,6 +1730,8 @@ process:
 
 			case '?':
 			{
+				condition = conditionMayFlush(condition, false);
+
 				int next = cppGetcFromUngetBufferOrFile ();
 				if (next != '?')
 				{
@@ -1490,6 +1769,8 @@ process:
 			 */
 			case '<':
 			{
+				condition = conditionMayFlush(condition, false);
+
 				/*
 				   Quoted from http://www.open-std.org/jtc1/sc22/wg21/docs/papers/2011/n3237.html:
 				   ------
@@ -1537,6 +1818,8 @@ process:
 			}
 			case ':':
 			{
+				condition = conditionMayFlush(condition, false);
+
 				int next = cppGetcFromUngetBufferOrFile ();
 				if (next == '>')
 					c = ']';
@@ -1550,6 +1833,8 @@ process:
 			}
 			case '%':
 			{
+				condition = conditionMayFlush(condition, false);
+
 				int next = cppGetcFromUngetBufferOrFile ();
 				switch (next)
 				{
@@ -1567,6 +1852,8 @@ process:
 			default:
 				if (c == '@' && Cpp.hasAtLiteralStrings)
 				{
+					condition = conditionMayFlush(condition, false);
+
 					int next = cppGetcFromUngetBufferOrFile ();
 					if (next == DOUBLE_QUOTE)
 					{
@@ -1585,6 +1872,8 @@ process:
 				}
 				else if (c == 'R' && Cpp.hasCxxRawLiteralStrings)
 				{
+					conditionMayPut(condition, c);
+
 					/* OMG!11 HACK!!11  Get the previous character.
 					 *
 					 * We need to know whether the previous character was an identifier or not,
@@ -1644,22 +1933,34 @@ process:
 						cppUngetc(next);
 					if (macrodef)
 						vStringPut (macrodef, c);
-
+					conditionMayPut(condition, c);
 				}
 				else
 				{
 					if (macrodef)
 						vStringPut (macrodef, c);
+					if (isalnum(c) || c == '_')
+						conditionMayPut(condition, c);
+					else
+						condition = conditionMayFlush(condition, false);
 				}
 			enter:
 				Cpp.directive.accept = false;
 				if (directive)
 				{
-					ignore = handleDirective (c, &macroCorkIndex);
+					bool inspect_conidtion = false;
+					ignore = handleDirective (c, &macroCorkIndex, &inspect_conidtion);
 					if (Cpp.macrodefFieldIndex != FIELD_UNKNOWN
 						&& macroCorkIndex != CORK_NIL
 						&& macrodef == NULL)
 						macrodef = vStringNew ();
+					if (condition == NULL
+						&& inspect_conidtion)
+					{
+						condition = vStringNew ();
+						if (isalpha(c) || c == '_')
+							conditionMayPut(condition, c);
+					}
 				}
 				break;
 		}
@@ -1668,7 +1969,10 @@ process:
 	if (macrodef)
 		vStringDelete (macrodef);
 
-	DebugStatement ( debugPutc (DEBUG_CPP, c); )
+	if (condition)
+		vStringDelete (condition);
+
+	DebugStatement ( cppDebugPutc (DEBUG_CPP, c); )
 	DebugStatement ( if (c == NEWLINE)
 				debugPrintf (DEBUG_CPP, "%6ld: ", getInputLineNumber () + 1); )
 
@@ -1678,7 +1982,8 @@ process:
 static void findCppTags (void)
 {
 	cppInitCommon (Cpp.lang, 0, false, false, false,
-				   KIND_GHOST_INDEX, 0, KIND_GHOST_INDEX,
+				   KIND_GHOST_INDEX, 0, 0,
+				   KIND_GHOST_INDEX,
 				   KIND_GHOST_INDEX, 0, 0,
 				   FIELD_UNKNOWN);
 
@@ -1701,7 +2006,7 @@ static bool buildMacroInfoFromTagEntry (int corkIndex,
 {
 	cppMacroInfo **info = data;
 
-	if (entry->langType == Cpp.clientLang
+	if ((entry->langType == Cpp.clientLang || entry->langType == Cpp.lang)
 		&& entry->kindIndex == Cpp.defineMacroKindIndex
 		&& isRoleAssigned (entry, ROLE_DEFINITION_INDEX))
 	{
@@ -1722,7 +2027,7 @@ static bool buildMacroInfoFromTagEntry (int corkIndex,
 	return true;
 }
 
-extern cppMacroInfo * cppFindMacroFromSymtab (const char *const name)
+static cppMacroInfo * cppFindMacroFromSymtab (const char *const name)
 {
 	cppMacroInfo *info = NULL;
 	foreachEntriesInScope (CORK_NIL, name, buildMacroInfoFromTagEntry, &info);
@@ -1807,6 +2112,54 @@ extern vString * cppBuildMacroReplacement(
 	return ret;
 }
 
+// We stop applying macro replacements if the unget buffer gets too big
+// as it is a sign of recursive macro expansion
+#define CPP_PARSER_MAXIMUM_UNGET_BUFFER_SIZE_FOR_MACRO_REPLACEMENTS 65536
+
+extern void cppBuildMacroReplacementWithPtrArrayAndUngetResult(
+		cppMacroInfo * macro,
+		const ptrArray * args)
+{
+	vString * replacement = NULL;
+
+	// Detect other cases of nasty macro expansion that cause
+	// the unget buffer to grow fast (but the token chain to grow slowly)
+	//    -D'p=a' -D'a=p+p'
+	if ((cppUngetBufferSize() < CPP_PARSER_MAXIMUM_UNGET_BUFFER_SIZE_FOR_MACRO_REPLACEMENTS)
+		&& macro->replacements)
+	{
+		int argc = 0;
+		const char ** argv = NULL;
+
+		if (args)
+		{
+			argc = ptrArrayCount (args);
+			argv = (const char **)eMalloc (sizeof(char *) * argc);
+			for (int i = 0; i < argc; i++)
+			{
+				TRACE_PRINT("Arg[%d] for %s<%p>: %s",
+							i, macro->name, macro, ptrArrayItem (args, i));
+				argv[i] = ptrArrayItem (args, i);
+			}
+		}
+
+		replacement = cppBuildMacroReplacement(macro, argv, argc);
+
+		if (argv)
+			eFree ((void *)argv);
+	}
+
+	if (replacement)
+	{
+		cppUngetStringBuiltByMacro(vStringValue(replacement), vStringLength(replacement),
+								   macro);
+		TRACE_PRINT("Replacement for %s<%p>: %s", macro->name, macro, vStringValue (replacement));
+		vStringDelete (replacement);
+	}
+	else
+		TRACE_PRINT("Replacement for %s<%p>: ", macro->name, macro);
+
+}
 
 static void saveIgnoreToken(const char * ignoreToken)
 {
@@ -1869,8 +2222,8 @@ static void saveIgnoreToken(const char * ignoreToken)
 	}
 	info->useCount = 0;
 	info->next = NULL;
-
-	hashTablePutItem(cmdlineMacroTable,eStrndup(tokenBegin,tokenEnd - tokenBegin),info);
+	info->name = eStrndup(tokenBegin,tokenEnd - tokenBegin);
+	hashTablePutItem(cmdlineMacroTable,info->name,info);
 
 	verbose ("    ignore token: %s\n", ignoreToken);
 }
@@ -1896,7 +2249,7 @@ static cppMacroInfo * saveMacro(hashTable *table, const char * macro)
 		return NULL;
 	}
 
-	if(!(isalpha(*c) || (*c == '_' || (*c == '$') )))
+	if(!(isalpha((unsigned char) *c) || (*c == '_' || (*c == '$') )))
 	{
 		CXX_DEBUG_LEAVE_TEXT("Macro does not start with an alphanumeric character");
 		return NULL; // must be a sequence of letters and digits
@@ -1904,7 +2257,7 @@ static cppMacroInfo * saveMacro(hashTable *table, const char * macro)
 
 	const char * identifierBegin = c;
 
-	while(*c && (isalnum(*c) || (*c == '_') || (*c == '$') ))
+	while(*c && (isalnum((unsigned char) *c) || (*c == '_') || (*c == '$') ))
 		c++;
 
 	const char * identifierEnd = c;
@@ -2037,14 +2390,14 @@ static cppMacroInfo * saveMacro(hashTable *table, const char * macro)
 
 		while(*c)
 		{
-			if(isalpha(*c) || (*c == '_'))
+			if(isalpha((unsigned char) *c) || (*c == '_'))
 			{
 				if(c > begin)
 					ADD_CONSTANT_REPLACEMENT(begin,c - begin);
 
 				const char * tokenBegin = c;
 
-				while(*c && (isalnum(*c) || (*c == '_')))
+				while(*c && (isalnum((unsigned char) *c) || (*c == '_')))
 					c++;
 
 				// check if it is a parameter
@@ -2165,7 +2518,8 @@ static cppMacroInfo * saveMacro(hashTable *table, const char * macro)
 			ADD_CONSTANT_REPLACEMENT(begin,c - begin);
 	}
 
-	hashTablePutItem(table,eStrndup(identifierBegin,identifierEnd - identifierBegin),info);
+	info->name = eStrndup(identifierBegin,identifierEnd - identifierBegin);
+	hashTablePutItem(table,info->name,info);
 	CXX_DEBUG_LEAVE();
 
 	return info;
@@ -2184,6 +2538,7 @@ static void freeMacroInfo(cppMacroInfo * info)
 		pPart = pPart->next;
 		eFree(pPartToDelete);
 	}
+	eFree(info->name);
 	eFree(info);
 }
 
@@ -2193,7 +2548,7 @@ static hashTable *makeMacroTable (void)
 		1024,
 		hashCstrhash,
 		hashCstreq,
-		eFree,
+		NULL,					/* Keys refers values' name fields. */
 		(void (*)(void *))freeMacroInfo
 		);
 }
@@ -2212,13 +2567,14 @@ static void finalizeCpp (const langType language, bool initialized)
 	}
 }
 
-static void CpreProExpandMacrosInInput (const langType language CTAGS_ATTR_UNUSED, const char *name, const char *arg)
+static bool CpreProExpandMacrosInInput (const langType language CTAGS_ATTR_UNUSED, const char *name, const char *arg)
 {
 	doesExpandMacros = paramParserBool (arg, doesExpandMacros,
 										name, "parameter");
+	return true;
 }
 
-static void CpreProInstallIgnoreToken (const langType language CTAGS_ATTR_UNUSED, const char *optname CTAGS_ATTR_UNUSED, const char *arg)
+static bool CpreProInstallIgnoreToken (const langType language CTAGS_ATTR_UNUSED, const char *optname CTAGS_ATTR_UNUSED, const char *arg)
 {
 	if (arg == NULL || arg[0] == '\0')
 	{
@@ -2233,9 +2589,10 @@ static void CpreProInstallIgnoreToken (const langType language CTAGS_ATTR_UNUSED
 			cmdlineMacroTable = makeMacroTable ();
 		saveIgnoreToken(arg);
 	}
+	return true;
 }
 
-static void CpreProInstallMacroToken (const langType language CTAGS_ATTR_UNUSED, const char *optname CTAGS_ATTR_UNUSED, const char *arg)
+static bool CpreProInstallMacroToken (const langType language CTAGS_ATTR_UNUSED, const char *optname CTAGS_ATTR_UNUSED, const char *arg)
 {
 	if (arg == NULL || arg[0] == '\0')
 	{
@@ -2250,30 +2607,32 @@ static void CpreProInstallMacroToken (const langType language CTAGS_ATTR_UNUSED,
 			cmdlineMacroTable = makeMacroTable ();
 		saveMacro(cmdlineMacroTable, arg);
 	}
+	return true;
 }
 
-static void CpreProSetIf0 (const langType language CTAGS_ATTR_UNUSED, const char *name, const char *arg)
+static bool CpreProSetIf0 (const langType language CTAGS_ATTR_UNUSED, const char *name, const char *arg)
 {
 	doesExaminCodeWithInIf0Branch = paramParserBool (arg, doesExaminCodeWithInIf0Branch,
 													 name, "parameter");
+	return true;
 }
 
-static parameterHandlerTable CpreProParameterHandlerTable [] = {
+static paramDefinition CpreProParams [] = {
 	{ .name = "if0",
 	  .desc = "examine code within \"#if 0\" branch (true or [false])",
-	  .handleParameter = CpreProSetIf0,
+	  .handleParam = CpreProSetIf0,
 	},
 	{ .name = "ignore",
 	  .desc = "a token to be specially handled",
-	  .handleParameter = CpreProInstallIgnoreToken,
+	  .handleParam = CpreProInstallIgnoreToken,
 	},
 	{ .name = "define",
 	  .desc = "define replacement for an identifier (name(params,...)=definition)",
-	  .handleParameter = CpreProInstallMacroToken,
+	  .handleParam = CpreProInstallMacroToken,
 	},
 	{ .name = "_expand",
 	  .desc = "expand macros if their definitions are in the current C/C++/CUDA input file (true or [false])",
-	  .handleParameter = CpreProExpandMacrosInInput,
+	  .handleParam = CpreProExpandMacrosInInput,
 	}
 };
 
@@ -2289,9 +2648,23 @@ extern parserDefinition* CPreProParser (void)
 	def->fieldTable = CPreProFields;
 	def->fieldCount = ARRAY_SIZE (CPreProFields);
 
-	def->parameterHandlerTable = CpreProParameterHandlerTable;
-	def->parameterHandlerCount = ARRAY_SIZE(CpreProParameterHandlerTable);
+	def->paramTable = CpreProParams;
+	def->paramCount = ARRAY_SIZE(CpreProParams);
 
 	def->useCork = CORK_QUEUE | CORK_SYMTAB;
 	return def;
 }
+
+#ifdef DEBUG
+extern void cppDebugPutc (const int level, const int c)
+{
+	if (debug (level)  &&  c != EOF)
+	{
+		     if (c == STRING_SYMBOL)  printf ("\"string\"");
+		else if (c == CHAR_SYMBOL)    printf ("'c'");
+		else                          putchar (c);
+
+		fflush (stdout);
+	}
+}
+#endif
