@@ -44,8 +44,9 @@ bool g_bFirstRun = true;
 // - Clear the token chain
 // - Reset "seen" keywords
 //
-void cxxParserNewStatement(void)
+void cxxParserNewStatementFull(bool bExported)
 {
+	CXX_DEBUG_PRINT("NEW STATE: %d", bExported);
 	cxxTokenChainClear(g_cxx.pTokenChain);
 	if(g_cxx.pTemplateTokenChain)
 	{
@@ -56,13 +57,18 @@ void cxxParserNewStatement(void)
 		// we don't care about stale specializations as they
 		// are destroyed wen the base template prefix is extracted
 	}
-	g_cxx.uKeywordState = 0;
+	g_cxx.uKeywordState = bExported? CXXParserKeywordStateSeenExport: 0;
 
 	// FIXME: this cpp handling of end/statement is kind of broken:
 	//        it works only because the moon is in the correct phase.
 	cppEndStatement();
 }
 
+void cxxParserNewStatement(void)
+{
+	cxxParserNewStatementFull(false);
+}
+
 //
 // Parse a subchain of input delimited by matching pairs selected from
 // [],(),{} and <>.
@@ -423,11 +429,7 @@ void cxxParserSetEndLineForTagInCorkQueue(int iCorkQueueIndex,unsigned long lEnd
 {
 	CXX_DEBUG_ASSERT(iCorkQueueIndex > CORK_NIL,"The cork queue index is not valid");
 
-	tagEntryInfo * tag = getEntryInCorkQueue (iCorkQueueIndex);
-
-	CXX_DEBUG_ASSERT(tag,"No tag entry in the cork queue");
-
-	tag->extensionFields.endLine = lEndLine;
+	setTagEndLineToCorkEntry (iCorkQueueIndex, lEndLine);
 }
 
 //
@@ -494,10 +496,13 @@ static bool cxxParserParseEnumStructClassOrUnionFullDeclarationTrailer(
 
 	MIOPos oFilePosition = getInputFilePosition();
 	int iFileLine = getInputLineNumber();
+	int eMaybeTokenTypeOpeningBracket = (g_cxx.bConfirmedCPPLanguage
+										 ? 0
+										 : CXXTokenTypeOpeningBracket);
 
 	if(!cxxParserParseUpToOneOf(
 			CXXTokenTypeEOF | CXXTokenTypeSemicolon |
-				CXXTokenTypeOpeningBracket | CXXTokenTypeAssignment,
+				eMaybeTokenTypeOpeningBracket | CXXTokenTypeAssignment,
 			false
 		))
 	{
@@ -551,6 +556,25 @@ static bool cxxParserParseEnumStructClassOrUnionFullDeclarationTrailer(
 	if(cxxTokenTypeIs(g_cxx.pToken,CXXTokenTypeOpeningBracket))
 	{
 		CXX_DEBUG_PRINT("Found opening bracket: possibly a function declaration?");
+
+		// Revert keyword states.
+		// Here we assume the following kind of input:
+		//
+		//   static struct S {...} fn (...) { ... }
+		//
+		// To fill properties: field of fn with "static", g_cxx.uKeywordState
+		// must be set here.
+		//
+		// See cxxParserEmitFunctionTags.
+		//
+		// NOTE: C++ doesn't accept such kind of input. So we propagate
+		// the state only meaningful in C languages.
+		g_cxx.uKeywordState |= uKeywordState & (0
+			| CXXParserKeywordStateSeenStatic
+			| CXXParserKeywordStateSeenInline
+			| CXXParserKeywordStateSeenExtern
+			| CXXParserKeywordStateSeenAttributeDeprecated
+			);
 		if(!cxxParserParseBlockHandleOpeningBracket())
 		{
 			CXX_DEBUG_LEAVE_TEXT("Failed to handle the opening bracket");
@@ -575,7 +599,26 @@ static bool cxxParserParseEnumStructClassOrUnionFullDeclarationTrailer(
 	if(uKeywordState & CXXParserKeywordStateSeenTypedef)
 		cxxParserExtractTypedef(g_cxx.pTokenChain,true,false);
 	else
+	{
+		// Revert keyword states.
+		// Here we assume the following kind of input:
+		//
+		//   static struct S {...} s;
+		//
+		// To fill properties: field of s with "static", g_cxx.uKeywordState
+		// must be set here.
+		g_cxx.uKeywordState |= uKeywordState & (0
+			| CXXParserKeywordStateSeenStatic
+			| CXXParserKeywordStateSeenExtern
+			| CXXParserKeywordStateSeenMutable
+			| CXXParserKeywordStateSeenInline
+			| CXXParserKeywordStateSeenAttributeDeprecated
+			| CXXParserKeywordStateSeenConstexpr
+			| CXXParserKeywordStateSeenConstinit
+			| CXXParserKeywordStateSeenThreadLocal
+			);
 		cxxParserExtractVariableDeclarations(g_cxx.pTokenChain,0);
+	}
 
 	CXX_DEBUG_LEAVE();
 	return true;
@@ -661,8 +704,12 @@ bool cxxParserParseEnum(void)
 	{
 		if(uInitialKeywordState & CXXParserKeywordStateSeenTypedef)
 		{
+			bool bR;
+			g_cxx.uKeywordState &= ~CXXParserKeywordStateSeenTypedef;
 			CXX_DEBUG_LEAVE_TEXT("Found parenthesis after typedef: parsing as generic typedef");
-			return cxxParserParseGenericTypedef();
+			bR = cxxParserParseGenericTypedef();
+			cxxParserNewStatement();
+			return bR;
 		}
 		// probably a function declaration/prototype
 		// something like enum x func()....
@@ -758,11 +805,30 @@ bool cxxParserParseEnum(void)
 			return false;
 		}
 
-		if(cxxTokenTypeIsOneOf(g_cxx.pToken,CXXTokenTypeEOF | CXXTokenTypeSemicolon))
+		if(cxxTokenTypeIs(g_cxx.pToken,CXXTokenTypeEOF))
 		{
 			// tolerate EOF, treat as forward declaration
 			cxxParserNewStatement();
-			CXX_DEBUG_LEAVE_TEXT("EOF or semicolon before enum block: can't decode this");
+			CXX_DEBUG_LEAVE_TEXT("EOF before enum block: can't decode this");
+			return true;
+		}
+
+		if(cxxTokenTypeIs(g_cxx.pToken,CXXTokenTypeSemicolon))
+		{
+			bool bMember = (cxxScopeGetVariableKind() == CXXTagKindMEMBER);
+			if (bMember)
+			{
+				// enum type structure member with bit-width:
+				// e.g.
+				//    sturct { enum E m: 2; } v;
+				CXX_DEBUG_PRINT("Found semicolon, member definition with bit-width");
+				cxxParserExtractVariableDeclarations(g_cxx.pTokenChain, 0);
+			}
+			cxxParserNewStatement();
+			if (bMember)
+				CXX_DEBUG_LEAVE_TEXT("Semicolon before enum block: can't decode this");
+			else
+				CXX_DEBUG_LEAVE();
 			return true;
 		}
 
@@ -819,7 +885,7 @@ bool cxxParserParseEnum(void)
 		CXX_DEBUG_PRINT("Enum name is %s",vStringValue(pEnumName->pszWord));
 		cxxTokenChainTake(g_cxx.pTokenChain,pEnumName);
 	} else {
-		pEnumName = cxxTokenCreateAnonymousIdentifier(CXXTagKindENUM);
+		pEnumName = cxxTokenCreateAnonymousIdentifier(CXXTagKindENUM, NULL);
 		bAnonymous = true;
 		CXX_DEBUG_PRINT(
 				"Enum name is %s (anonymous)",
@@ -850,10 +916,18 @@ bool cxxParserParseEnum(void)
 			pTypeName = cxxTagCheckAndSetTypeField(pTypeBegin,pTypeEnd);
 		}
 
+
+		unsigned int uProperties = 0;
 		if(bIsScopedEnum)
-			pszProperties = cxxTagSetProperties(CXXTagPropertyScopedEnum);
+			uProperties |= CXXTagPropertyScopedEnum;
+		if(g_cxx.uKeywordState & CXXParserKeywordStateSeenExport)
+			uProperties |= CXXTagPropertyExport;
+
+		if(uProperties)
+			pszProperties = cxxTagSetProperties(uProperties);
 
 		iCorkQueueIndex = cxxTagCommit(&iCorkQueueIndexFQ);
+		cxxTagUseTokenAsPartOfDefTag(iCorkQueueIndex, pEnumName);
 
 		if (pszProperties)
 			vStringDelete (pszProperties);
@@ -1047,8 +1121,12 @@ static bool cxxParserParseClassStructOrUnionInternal(
 	{
 		if(uInitialKeywordState & CXXParserKeywordStateSeenTypedef)
 		{
+			bool bR;
+			g_cxx.uKeywordState &= ~CXXParserKeywordStateSeenTypedef;
 			CXX_DEBUG_LEAVE_TEXT("Found parenthesis after typedef: parsing as generic typedef");
-			return cxxParserParseGenericTypedef();
+			bR = cxxParserParseGenericTypedef();
+			cxxParserNewStatement();
+			return bR;
 		}
 
 		// probably a function declaration/prototype
@@ -1216,7 +1294,7 @@ static bool cxxParserParseClassStructOrUnionInternal(
 			);
 		cxxTokenChainTake(g_cxx.pTokenChain,pClassName);
 	} else {
-		pClassName = cxxTokenCreateAnonymousIdentifier(uTagKind);
+		pClassName = cxxTokenCreateAnonymousIdentifier(uTagKind, NULL);
 		bAnonymous = true;
 		CXX_DEBUG_PRINT(
 				"Class/struct/union name is %s (anonymous)",
@@ -1321,8 +1399,18 @@ static bool cxxParserParseClassStructOrUnionInternal(
 
 		tag->isFileScope = !isInputHeaderFile();
 
-		iCorkQueueIndex = cxxTagCommit(&iCorkQueueIndexFQ);
+		unsigned int uProperties = 0;
+		if(uInitialKeywordState & CXXParserKeywordStateSeenExport)
+			uProperties |= CXXTagPropertyExport;
+		vString * pszProperties = NULL;
 
+		if(uProperties)
+			pszProperties = cxxTagSetProperties(uProperties);
+
+		iCorkQueueIndex = cxxTagCommit(&iCorkQueueIndexFQ);
+		cxxTagUseTokenAsPartOfDefTag(iCorkQueueIndex, pClassName);
+
+		vStringDelete (pszProperties); /* NULL is acceptable. */
 	}
 
 	cxxScopePush(
@@ -1465,7 +1553,8 @@ void cxxParserAnalyzeOtherStatement(void)
 
 	CXXToken * t = cxxTokenChainFirst(g_cxx.pTokenChain);
 
-	if(!cxxTokenTypeIsOneOf(t,CXXTokenTypeIdentifier | CXXTokenTypeKeyword))
+	if(!cxxTokenTypeIsOneOf(t,CXXTokenTypeIdentifier | CXXTokenTypeKeyword
+							| CXXTokenTypeMultipleColons))
 	{
 		CXX_DEBUG_LEAVE_TEXT("Statement does not start with an identifier or keyword");
 		return;
@@ -1502,7 +1591,14 @@ void cxxParserAnalyzeOtherStatement(void)
 	const bool bPrototypeParams = cxxTagKindEnabled(CXXTagKindPROTOTYPE) && cxxTagKindEnabled(CXXTagKindPARAMETER);
 check_function_signature:
 
-	if(cxxParserLookForFunctionSignature(g_cxx.pTokenChain,&oInfo,bPrototypeParams?&oParamInfo:NULL))
+	if(
+		cxxParserLookForFunctionSignature(g_cxx.pTokenChain,&oInfo,bPrototypeParams?&oParamInfo:NULL)
+		// Even if we saw "();", we cannot say it is a function prototype; a macro expansion can be used to
+		// initialize a top-level variable like:
+		//   #define INIT() 0
+		//   int i = INIT();"
+		&& ! (oInfo.pTypeEnd && cxxTokenTypeIs(oInfo.pTypeEnd,CXXTokenTypeAssignment))
+		)
 	{
 		CXX_DEBUG_PRINT("Found function prototype");
 
@@ -1860,6 +1956,7 @@ static rescanReason cxxParserMain(const unsigned int passCount)
 	int kind_for_header = CXXTagKindINCLUDE;
 	int kind_for_macro_param = CXXTagKindMACROPARAM;
 	int role_for_macro_undef = CR_MACRO_UNDEF;
+	int role_for_macro_condition = CR_MACRO_CONDITION;
 	int role_for_header_system = CR_HEADER_SYSTEM;
 	int role_for_header_local = CR_HEADER_LOCAL;
 
@@ -1872,6 +1969,7 @@ static rescanReason cxxParserMain(const unsigned int passCount)
 			false,
 			kind_for_define,
 			role_for_macro_undef,
+			role_for_macro_condition,
 			kind_for_macro_param,
 			kind_for_header,
 			role_for_header_system,
@@ -1943,6 +2041,7 @@ rescanReason cxxCppParserMain(const unsigned int passCount)
 	cxxKeywordEnablePublicProtectedPrivate(g_cxx.bConfirmedCPPLanguage);
 
 	rescanReason r = cxxParserMain(passCount);
+	cxxParserDestroyCurrentModuleToken();
 	CXX_DEBUG_LEAVE();
 	return r;
 }
