@@ -14,6 +14,7 @@
 #include "cxx_token.h"
 #include "cxx_token_chain.h"
 #include "cxx_scope.h"
+#include "cxx_side_chain.h"
 
 #include "vstring.h"
 #include "read.h"
@@ -87,6 +88,8 @@ CXXToken * cxxParserFindFirstPossiblyNestedAndQualifiedIdentifier(
 //   type var: range declaration <-- (FIXME: this is only inside for!)
 //   very complex type with modifiers() namespace::namespace::var = ...;
 //   type<with template> namespace::var[] = {
+//   type var<T> = ...;
+//   type var<T>[] = ...;
 //   ...
 //
 // Assumptions:
@@ -162,8 +165,10 @@ bool cxxParserExtractVariableDeclarations(CXXTokenChain * pChain,unsigned int uF
 		return false;
 	}
 
-	bool bGotVariable = false;
 
+	// For tracking a specialization in a variable template
+	CXXToken * pTemplateSpecializationStart = NULL, * pTemplateSpecializationEnd = NULL;
+	bool bGotVariable = false;
 	// Loop over the whole statement.
 
 	while(t)
@@ -206,8 +211,9 @@ bool cxxParserExtractVariableDeclarations(CXXTokenChain * pChain,unsigned int uF
 				return bGotVariable;
 			}
 
-			if(t->eType == CXXTokenTypeSmallerThanSign)
+			if(cxxTokenTypeIs(t, CXXTokenTypeSmallerThanSign))
 			{
+				pTemplateSpecializationStart = t;
 				// Must be part of template type name (so properly balanced).
 				t = cxxTokenChainSkipToEndOfTemplateAngleBracket(t);
 				if(!t)
@@ -215,6 +221,7 @@ bool cxxParserExtractVariableDeclarations(CXXTokenChain * pChain,unsigned int uF
 					CXX_DEBUG_LEAVE_TEXT("Failed to skip past angle bracket chain");
 					return bGotVariable;
 				}
+				pTemplateSpecializationEnd = t;
 			}
 
 			t = t->pNext;
@@ -228,6 +235,7 @@ bool cxxParserExtractVariableDeclarations(CXXTokenChain * pChain,unsigned int uF
 			return bGotVariable;
 		}
 
+		CXXToken * pEndScanningAttrs = t;
 		CXX_DEBUG_PRINT(
 				"Found notable token '%s' of type 0x%02x(%s)",
 				vStringValue(t->pszWord),
@@ -245,6 +253,7 @@ bool cxxParserExtractVariableDeclarations(CXXTokenChain * pChain,unsigned int uF
 
 		CXXToken * pIdentifier = NULL;
 		CXXToken * pTokenBefore = NULL;
+		bool bSpecializedAsVarTemplate = false;
 
 		// If we have to continue scanning we'll remove the tokens from here
 		// so they don't end up being part of the type name.
@@ -339,7 +348,7 @@ bool cxxParserExtractVariableDeclarations(CXXTokenChain * pChain,unsigned int uF
 								cxxTokenTypeIs(t->pPrev,CXXTokenTypeKeyword) &&
 								cxxKeywordMayBePartOfTypeName(t->pPrev->eKeyword) &&
 								// but not decltype(var)!
-								(t->pPrev->eKeyword != CXXKeywordDECLTYPE)
+								!cxxKeywordIsDecltype(t->pPrev->eKeyword)
 							)
 						) &&
 						(
@@ -348,19 +357,20 @@ bool cxxParserExtractVariableDeclarations(CXXTokenChain * pChain,unsigned int uF
 						)
 					)
 				{
-					CXX_DEBUG_LEAVE_TEXT("Parenthesis seems to surround a variable definition");
+					CXX_DEBUG_PRINT("Parenthesis seems to surround a variable definition");
 					pTokenBefore = t->pPrev;
 					t = t->pNext;
 					goto got_identifier;
 				}
 
 				if(
-					cxxTokenIsKeyword(t->pPrev,CXXKeywordDECLTYPE) &&
+					cxxTokenTypeIs(t->pPrev,CXXTokenTypeKeyword) &&
+					cxxKeywordIsDecltype(t->pPrev->eKeyword) &&
 					t->pNext
 				)
 				{
 					// part of typename -> skip ahead
-					CXX_DEBUG_LEAVE_TEXT("Parenthesis follows decltype(), skipping");
+					CXX_DEBUG_PRINT("Parenthesis follows decltype(), skipping");
 					t = t->pNext;
 					continue;
 				}
@@ -423,7 +433,17 @@ bool cxxParserExtractVariableDeclarations(CXXTokenChain * pChain,unsigned int uF
 				// check for array
 				// Keep the array specifier as part of type
 
-				pIdentifier = t->pPrev;
+				//         pTemplateSpecializationStart
+				//         | pTemplateSpecializationEnd
+				// --------V-V
+				// type var<T>[] = ...
+				// -----------^------- t
+				if (t->pPrev && t->pPrev == pTemplateSpecializationEnd)
+					bSpecializedAsVarTemplate = true;
+
+				pIdentifier = bSpecializedAsVarTemplate
+					? pTemplateSpecializationStart->pPrev
+					: t->pPrev;
 				pTokenBefore = pIdentifier->pPrev;
 
 				while(t->pNext && cxxTokenTypeIs(t->pNext,CXXTokenTypeSquareParenthesisChain))
@@ -465,18 +485,68 @@ bool cxxParserExtractVariableDeclarations(CXXTokenChain * pChain,unsigned int uF
 				t = t->pNext;
 			break;
 			default:
+				//         pTemplateSpecializationStart
+				//         | pTemplateSpecializationEnd
+				// --------V-V
+				// type var<T> = ...
+				// ------------^------ t
+
+				if (t->pPrev && t->pPrev == pTemplateSpecializationEnd)
+					bSpecializedAsVarTemplate = true;
+
 				// Must be identifier
-				if(t->pPrev->eType != CXXTokenTypeIdentifier)
+				if (bSpecializedAsVarTemplate &&
+					!cxxTokenTypeIs(pTemplateSpecializationStart->pPrev, CXXTokenTypeIdentifier))
+				{
+					CXX_DEBUG_LEAVE_TEXT("No identifier before the <> specializer");
+					return bGotVariable;
+				}
+				else if (!bSpecializedAsVarTemplate &&
+						 !cxxTokenTypeIs(t->pPrev, CXXTokenTypeIdentifier))
 				{
 					CXX_DEBUG_LEAVE_TEXT("No identifier before the notable token");
 					return bGotVariable;
 				}
 
-				pIdentifier = t->pPrev;
+				pIdentifier = bSpecializedAsVarTemplate
+					? pTemplateSpecializationStart->pPrev
+					: t->pPrev;
 				pTokenBefore = pIdentifier->pPrev;
+
+				// Skip the specializer.
+				// When extracting the typeref for the variable, the specializer
+				// becomes noise.
+				if (bSpecializedAsVarTemplate)
+					t = pTemplateSpecializationEnd->pNext;
+
+				// The final states:
+				// pTokenBefore
+				// |    pIdentifier
+				// V    V
+				// type var ... =
+				// -------------^- t
 			break;
 		}
 
+		if (bSpecializedAsVarTemplate) {
+			pTemplateSpecializationEnd->bFollowedBySpace = false;
+			CXXToken * pTemplateSpecialization = cxxTokenChainExtractRange(
+				pTemplateSpecializationStart,
+				pTemplateSpecializationEnd,
+				0
+				);
+			if(g_cxx.pTemplateSpecializationTokenChain)
+				cxxTokenChainClear(g_cxx.pTemplateSpecializationTokenChain);
+			else
+				g_cxx.pTemplateSpecializationTokenChain = cxxTokenChainCreate();
+			cxxTokenChainAppend(g_cxx.pTemplateSpecializationTokenChain,
+								pTemplateSpecialization);
+			cxxTokenChainDestroyRange(pChain,
+									  pTemplateSpecializationStart,
+									  pTemplateSpecializationEnd);
+			pTemplateSpecializationStart = pTemplateSpecializationEnd = NULL;
+		}
+
 got_identifier:
 		CXX_DEBUG_ASSERT(pIdentifier,"We should have found an identifier here");
 
@@ -508,7 +578,7 @@ got_identifier:
 		CXXToken * pScopeStart = NULL;
 
 		// Skip back to the beginning of the scope, if any
-		while(pTokenBefore->eType == CXXTokenTypeMultipleColons)
+		while(cxxTokenTypeIs(pTokenBefore, CXXTokenTypeMultipleColons))
 		{
 			if(!cxxParserCurrentLanguageIsCPP())
 			{
@@ -598,14 +668,25 @@ got_identifier:
 						// Possibly one of:
 						//   MACRO(whatever) variable;
 						//   decltype(whatever) variable;
+						//   __typeof(whatever) variable;
+						//   __typeof__(whatever) variable;
+						//   typeof(whatever) variable;
 						cxxTokenTypeIs(pTokenBefore,CXXTokenTypeParenthesisChain) &&
 						pTokenBefore->pPrev &&
-						!pTokenBefore->pPrev->pPrev &&
+						(!pTokenBefore->pPrev->pPrev ||
+						 (
+							 cxxTokenTypeIs(pTokenBefore->pPrev->pPrev,CXXTokenTypeKeyword) &&
+							 cxxKeywordMayAppearInVariableDeclaration(pTokenBefore->pPrev->pPrev->eKeyword)
+						 )
+						) &&
 						(
 							// macro
 							cxxTokenTypeIs(pTokenBefore->pPrev,CXXTokenTypeIdentifier) ||
-							// decltype
-							cxxTokenIsKeyword(pTokenBefore->pPrev,CXXKeywordDECLTYPE)
+							// decltype or typeof
+							(
+								cxxTokenTypeIs(pTokenBefore->pPrev,CXXTokenTypeKeyword) &&
+								cxxKeywordIsDecltype(pTokenBefore->pPrev->eKeyword)
+							)
 						)
 					)
 				{
@@ -757,6 +838,17 @@ got_identifier:
 				// Volatile is part of the type, so we don't mark it as a property
 				//if(g_cxx.uKeywordState & CXXParserKeywordStateSeenVolatile)
 				//	uProperties |= CXXTagPropertyVolatile;
+				if(g_cxx.uKeywordState & CXXParserKeywordStateSeenConstexpr)
+					uProperties |= CXXTagPropertyConstexpr;
+				if(g_cxx.uKeywordState & CXXParserKeywordStateSeenConstinit)
+					uProperties |= CXXTagPropertyConstinit;
+				// consteval is not here; it is for functions.
+				if(g_cxx.uKeywordState & CXXParserKeywordStateSeenThreadLocal)
+					uProperties |= CXXTagPropertyThreadLocal;
+				if(g_cxx.uKeywordState & CXXParserKeywordStateSeenExport)
+					uProperties |= CXXTagPropertyExport;
+				if(bSpecializedAsVarTemplate)
+					uProperties |= CXXTagPropertyTemplateSpecialization;
 
 				pszProperties = cxxTagSetProperties(uProperties);
 			}
@@ -764,7 +856,12 @@ got_identifier:
 			if(bGotTemplate)
 				cxxTagHandleTemplateFields();
 
+			cxxSideChainCollectInRange(cxxTokenChainFirst(pChain), pEndScanningAttrs,
+									   pIdentifier);
+			cxxSideChainScan(pIdentifier->pSideChain);
+
 			iCorkIndex = cxxTagCommit(&iCorkIndexFQ);
+			cxxTagUseTokenAsPartOfDefTag(iCorkIndexFQ, pIdentifier);
 
 			if(pTypeToken)
 				cxxTokenDestroy(pTypeToken);
