@@ -128,8 +128,7 @@ static void makeJsonTag (tokenInfo *const token, const jsonKind kind)
 
 	initTagEntry (&e, vStringValue (token->string), kind);
 
-	e.lineNumber	= token->lineNumber;
-	e.filePosition	= token->filePosition;
+	updateTagLine (&e, token->lineNumber, token->filePosition);
 
 	if (vStringLength (token->scope) > 0)
 	{
@@ -244,9 +243,7 @@ static void pushScope (tokenInfo *const token,
 					   const tokenInfo *const parent,
 					   const jsonKind parentKind)
 {
-	if (vStringLength (token->scope) > 0)
-		vStringPut (token->scope, '.');
-	vStringCat (token->scope, parent->string);
+	vStringJoin(token->scope, '.', parent->string);
 	token->scopeKind = parentKind;
 }
 
