@@ -5,9 +5,14 @@
 *   regarding variable and function definitions.
 *
 *   This source code is released for free distribution under the terms of the
-*   GNU General Public License.
+*   GNU General Public License version 2 or (at your option) any later version.
 *
-*   This module contains code for generating tags for Windows PowerShell scripts.
+*   This module contains code for generating tags for Windows PowerShell scripts
+*   (https://en.wikipedia.org/wiki/PowerShell).
+*
+*   References:
+*   - https://learn.microsoft.com/en-us/powershell/scripting/developer/windows-powershell-reference
+*   - https://learn.microsoft.com/en-us/powershell/scripting/lang-spec/chapter-01
 */
 
 /*
@@ -15,7 +20,6 @@
 */
 #include "general.h"  /* must always come first */
 #include "debug.h"
-#include "main.h"
 #include "parse.h"
 #include "read.h"
 #include "vstring.h"
@@ -39,12 +43,20 @@ static const char *const accessTypes[] = {
 typedef enum {
 	K_FUNCTION,
 	K_VARIABLE,
+	K_CLASS,
+	K_FILTER,
+	K_ENUM,
+	K_ENUMLABEL,
 	COUNT_KIND
 } powerShellKind;
 
 static kindDefinition PowerShellKinds[COUNT_KIND] = {
 	{ true, 'f', "function",	"functions" },
-	{ true, 'v', "variable",	"variables" }
+	{ true, 'v', "variable",	"variables" },
+	{ true, 'c', "class",		"classes" },
+	{ true, 'i', "filter",		"filter" },
+	{ true, 'g', "enum",		"enum names" },
+	{ true, 'e', "enumlabel",	"enum labels" },
 };
 
 
@@ -69,13 +81,32 @@ typedef enum eTokenType {
 	TOKEN_VARIABLE
 } tokenType;
 
+enum {
+	KEYWORD_function,
+	KEYWORD_filter,
+	KEYWORD_class,
+	KEYWORD_enum,
+};
+
+/* We need an integer that is not an unsigned to allow KEYWORD_NONE. */
+typedef int keywordId;
+
+static const keywordTable PowerShellKeywordTable[] = {
+	{ "function",	KEYWORD_function },
+	{ "filter",		KEYWORD_filter },
+	{ "class",		KEYWORD_class },
+	{ "enum",		KEYWORD_enum },
+};
+
 typedef struct {
 	tokenType		type;
+	keywordId		keyword;
 	vString *		string;
 	vString *		scope;
 	unsigned long	lineNumber;
 	MIOPos			filePosition;
-	int 			parentKind; /* -1 if none */
+	int				parentKind; /* KIND_GHOST_INDEX if none */
+	bool            herestring;	/* Meaningful only when type is TOKEN_STRING. */
 } tokenInfo;
 
 
@@ -100,8 +131,7 @@ static void initPowerShellEntry (tagEntryInfo *const e, const tokenInfo *const t
 {
 	initTagEntry (e, vStringValue (token->string), kind);
 
-	e->lineNumber	= token->lineNumber;
-	e->filePosition	= token->filePosition;
+	updateTagLine (e, token->lineNumber, token->filePosition);
 
 	if (access != NULL)
 		e->extensionFields.access = access;
@@ -128,13 +158,13 @@ static void makeSimplePowerShellTag (const tokenInfo *const token, const powerSh
 }
 
 static void makeFunctionTag (const tokenInfo *const token, const vString *const arglist,
-							 const char *const access)
+							 const char *const access, int kind)
 {
-	if (PowerShellKinds[K_FUNCTION].enabled)
+	if (PowerShellKinds[kind].enabled)
 	{
 		tagEntryInfo e;
 
-		initPowerShellEntry (&e, token, K_FUNCTION, access);
+		initPowerShellEntry (&e, token, kind, access);
 
 		if (arglist)
 			e.extensionFields.signature = vStringValue (arglist);
@@ -143,16 +173,41 @@ static void makeFunctionTag (const tokenInfo *const token, const vString *const
 	}
 }
 
+static void makeClassTag (const tokenInfo *const token)
+{
+	if (PowerShellKinds[K_CLASS].enabled)
+	{
+		tagEntryInfo e;
+
+		initPowerShellEntry (&e, token, K_CLASS, NULL);
+
+		makeTagEntry (&e);
+	}
+}
+
+static void makeEnumTag (const tokenInfo *const token)
+{
+	if (PowerShellKinds[K_ENUM].enabled)
+	{
+		tagEntryInfo e;
+
+		initPowerShellEntry (&e, token, K_ENUM, NULL);
+
+		makeTagEntry (&e);
+	}
+}
+
 static tokenInfo *newToken (void)
 {
 	tokenInfo *const token = xMalloc (1, tokenInfo);
 
 	token->type			= TOKEN_UNDEFINED;
+	token->keyword		= KEYWORD_NONE;
 	token->string		= vStringNew ();
 	token->scope		= vStringNew ();
 	token->lineNumber   = getInputLineNumber ();
 	token->filePosition = getInputFilePosition ();
-	token->parentKind	= -1;
+	token->parentKind	= KIND_GHOST_INDEX;
 
 	return token;
 }
@@ -185,7 +240,12 @@ static void addToScope (tokenInfo *const token, const vString *const extra)
 
 static bool isIdentChar (const int c)
 {
-	return (isalnum (c) || c == ':' || c == '_' || c == '-' || c >= 0x80);
+	return (isalnum (c) || c == '_' || c == '-' || c >= 0x80);
+}
+
+static bool isScopeIdentifierChar (const int c)
+{
+	return (isIdentChar (c) || c == ':');
 }
 
 static void parseString (vString *const string, const int delimiter)
@@ -194,32 +254,83 @@ static void parseString (vString *const string, const int delimiter)
 	{
 		int c = getcFromInputFile ();
 
-		if (c == '\\' && (c = getcFromInputFile ()) != EOF)
-			vStringPut (string, (char) c);
+		if (delimiter == '"' && c == '`' && (c = getcFromInputFile ()) != EOF)
+			vStringPut (string, c);
 		else if (c == EOF || c == delimiter)
 			break;
 		else
-			vStringPut (string, (char) c);
+			vStringPut (string, c);
 	}
 }
 
+static bool parseHereString (vString *const string, const int delimiter)
+{
+	enum phs_state {
+		PHS_INIT,
+		PHS_NEWLINE0,
+		PHS_QUOTE,
+		PHS_AT,
+		PHS_NEWLINE1,
+		PHS_END = PHS_NEWLINE1
+	} s = PHS_INIT;
+
+	while (s != PHS_END)
+	{
+		int c = getcFromInputFile ();
+
+		if (c == EOF)
+			return false;
+
+		vStringPut (string, c);
+
+		if (s == PHS_INIT && c == '\n')
+			s = PHS_NEWLINE0;
+		else if (s == PHS_NEWLINE0 && c == delimiter)
+			s = PHS_QUOTE;
+		else if (s == PHS_QUOTE && c == '@')
+			s = PHS_AT;
+		else if (s == PHS_AT && c == '\n')
+			s = PHS_NEWLINE1;
+		else
+			s = PHS_INIT;
+	}
+
+	Assert (vStringLength (string) >= 4);
+	vStringTruncate (string, vStringLength (string) - 4);
+	return true;
+}
+
+/* parse a identifier that may contain scopes, such as function names and
+ * variable names.
+ *
+ * 	VariableName
+ * 	FunctionName
+ * 	local:VariableName
+ * 	private:FunctionName
+ */
+static void parseScopeIdentifier (vString *const string, const int firstChar)
+{
+	int c = firstChar;
+	do
+	{
+		vStringPut (string, c);
+		c = getcFromInputFile ();
+	} while (isScopeIdentifierChar (c));
+	ungetcToInputFile (c);
+}
+
+/* parse a identifier that do not contain scope, such as class names. */
 static void parseIdentifier (vString *const string, const int firstChar)
 {
 	int c = firstChar;
 	do
 	{
-		vStringPut (string, (char) c);
+		vStringPut (string, c);
 		c = getcFromInputFile ();
 	} while (isIdentChar (c));
 	ungetcToInputFile (c);
 }
 
-static bool isTokenFunction (vString *const name)
-{
-	return (strcasecmp (vStringValue (name), "function") == 0 ||
-			strcasecmp (vStringValue (name), "filter") == 0);
-}
-
 static bool isSpace (int c)
 {
 	return (c == '\t' || c == ' ' || c == '\v' ||
@@ -251,7 +362,7 @@ static int skipSingleComment (void)
 	return c;
 }
 
-static void readToken (tokenInfo *const token)
+static void readTokenFull (tokenInfo *const token, bool includingScope)
 {
 	int c;
 
@@ -287,8 +398,33 @@ getNextChar:
 			parseString (token->string, c);
 			token->lineNumber = getInputLineNumber ();
 			token->filePosition = getInputFilePosition ();
+			token->herestring = false;
 			break;
 
+		case '@':
+		{
+			int d = getcFromInputFile ();
+			if (d == '\'' || d == '"')
+			{
+				bool is_herestr;
+				token->type = TOKEN_STRING;
+				is_herestr = parseHereString (token->string, d);
+				token->lineNumber = getInputLineNumber ();
+				token->filePosition = getInputFilePosition ();
+				if (is_herestr)
+					token->herestring = true;
+				else
+					token->type = TOKEN_UNDEFINED;
+			}
+			else if (d == EOF)
+				token->type = TOKEN_UNDEFINED;
+			else
+			{
+				ungetcToInputFile (d);
+				token->type = TOKEN_UNDEFINED;
+			}
+			break;
+		}
 		case '<':
 		{
 			int d = getcFromInputFile ();
@@ -338,14 +474,14 @@ getNextChar:
 		case '$': /* variable start */
 		{
 			int d = getcFromInputFile ();
-			if (! isIdentChar (d))
+			if (! isScopeIdentifierChar (d))
 			{
 				ungetcToInputFile (d);
 				token->type = TOKEN_UNDEFINED;
 			}
 			else
 			{
-				parseIdentifier (token->string, d);
+				parseScopeIdentifier (token->string, d);
 				token->type = TOKEN_VARIABLE;
 			}
 			break;
@@ -356,16 +492,26 @@ getNextChar:
 				token->type = TOKEN_UNDEFINED;
 			else
 			{
-				parseIdentifier (token->string, c);
-				if (isTokenFunction (token->string))
-					token->type = TOKEN_KEYWORD;
+				if (includingScope)
+					parseScopeIdentifier (token->string, c);
 				else
+					parseIdentifier (token->string, c);
+				token->keyword = lookupCaseKeyword (
+						vStringValue (token->string), getInputLanguage ());
+				if (token->keyword == KEYWORD_NONE)
 					token->type = TOKEN_IDENTIFIER;
+				else
+					token->type = TOKEN_KEYWORD;
 			}
 			break;
 	}
 }
 
+static void readToken (tokenInfo *const token)
+{
+	readTokenFull (token, false);
+}
+
 static void enterScope (tokenInfo *const parentToken,
 						const vString *const extraScope,
 						const int parentKind);
@@ -387,9 +533,9 @@ static const char *parsePowerShellScope (tokenInfo *const token)
 		/* extract the scope */
 		vStringNCopyS (powershellScope, tokenName, powershellScopeLen);
 		/* cut the resulting scope string from the identifier */
-		memmove (token->string->buffer,
+		memmove (vStringValue (token->string),
 				 /* +1 to skip the leading colon */
-				 token->string->buffer + powershellScopeLen + 1,
+				 vStringValue (token->string) + powershellScopeLen + 1,
 				 /* +1 for the skipped leading colon and - 1 to include the trailing \0 byte */
 				 token->string->length + 1 - powershellScopeLen - 1);
 		token->string->length -= powershellScopeLen + 1;
@@ -406,13 +552,13 @@ static const char *parsePowerShellScope (tokenInfo *const token)
  *
  * 	function myfunc($foo, $bar) {}
  */
-static bool parseFunction (tokenInfo *const token)
+static bool parseFunction (tokenInfo *const token, int kind)
 {
 	bool readNext = true;
 	tokenInfo *nameFree = NULL;
 	const char *access;
 
-	readToken (token);
+	readTokenFull (token, true);
 
 	if (token->type != TOKEN_IDENTIFIER)
 		return false;
@@ -456,7 +602,6 @@ static bool parseFunction (tokenInfo *const token)
 				case TOKEN_STRING:			vStringCatS (arglist, "'...'");	break;
 
 				case TOKEN_IDENTIFIER:
-				case TOKEN_KEYWORD:
 				case TOKEN_VARIABLE:
 				{
 					switch (vStringLast (arglist))
@@ -485,23 +630,113 @@ static bool parseFunction (tokenInfo *const token)
 		}
 		while (token->type != TOKEN_EOF && depth > 0);
 
-		makeFunctionTag (nameFree, arglist, access);
+		makeFunctionTag (nameFree, arglist, access, kind);
 		vStringDelete (arglist);
 
 		readToken (token);
 	}
 	else if (token->type == TOKEN_OPEN_CURLY)
 	{	/* filters doesn't need to have an arglist */
-		makeFunctionTag (nameFree, NULL, access);
+		makeFunctionTag (nameFree, NULL, access, kind);
 	}
 
 	if (token->type == TOKEN_OPEN_CURLY)
-		enterScope (token, nameFree->string, K_FUNCTION);
+		enterScope (token, nameFree->string, kind);
 	else
 		readNext = false;
 
-	if (nameFree)
-		deleteToken (nameFree);
+	deleteToken (nameFree);
+
+	return readNext;
+}
+
+/* parse a class
+ *
+ * 	class MyClass {}
+ * 	class Derived : Base {}
+ */
+static bool parseClass (tokenInfo *const token)
+{
+	bool readNext = true;
+	vString *nameFree = NULL;
+
+	readToken (token);
+
+	if (token->type != TOKEN_IDENTIFIER)
+		return false;
+
+	makeClassTag (token);
+	nameFree = vStringNewCopy (token->string);
+	readToken (token);
+
+	while (token->type != TOKEN_OPEN_CURLY && token->type != TOKEN_EOF)
+	{
+		readToken (token);
+	}
+
+	if (token->type == TOKEN_OPEN_CURLY)
+		enterScope (token, nameFree, K_CLASS);
+	else
+		readNext = false;
+
+	vStringDelete (nameFree);
+
+	return readNext;
+}
+
+/* parse a enum
+ *
+ * 	enum EnumName {}
+ */
+static bool parseEnum (tokenInfo *const token)
+{
+	bool readNext = true;
+	vString *nameFree = NULL;
+
+	readToken (token);
+
+	if (token->type != TOKEN_IDENTIFIER)
+		return false;
+
+	makeEnumTag (token);
+	nameFree = vStringNewCopy (token->string);
+	readToken (token);
+
+	while (token->type != TOKEN_OPEN_CURLY && token->type != TOKEN_EOF)
+	{
+		readToken (token);
+	}
+
+	if (token->type == TOKEN_OPEN_CURLY)
+		enterScope (token, nameFree, K_ENUM);
+	else
+		readNext = false;
+
+	vStringDelete (nameFree);
+
+	return readNext;
+}
+
+/* parses declarations of the form
+ * 	<label> [= <int-value>]
+ * that is, contents of an enum
+ */
+static bool parseEnumLabel (tokenInfo *const token)
+{
+	bool readNext = true;
+
+	if (token->parentKind != K_ENUM)
+		return false;
+
+	if (token->type != TOKEN_IDENTIFIER)
+		return false;
+
+	makeSimplePowerShellTag (token, K_ENUMLABEL, ACCESS_UNDEFINED);
+	readToken (token);
+	if (token->type != TOKEN_EQUAL_SIGN)
+		readNext = false;
+	else /* skip int-value */
+		readToken (token);
 
 	return readNext;
 }
@@ -521,8 +756,11 @@ static bool parseVariable (tokenInfo *const token)
 	readToken (token);
 	if (token->type == TOKEN_EQUAL_SIGN)
 	{
-		if (token->parentKind != K_FUNCTION)
-		{	/* ignore local variables (i.e. within a function) */
+		if (token->parentKind != K_FUNCTION &&
+				token->parentKind != K_FILTER &&
+				token->parentKind != K_CLASS)
+		{   /* ignore local variables (i.e. within a function)
+			 * TODO: Parses class properties to make tags. */
 			access = parsePowerShellScope (name);
 			makeSimplePowerShellTag (name, K_VARIABLE, access);
 			readNext = true;
@@ -560,17 +798,41 @@ static void enterScope (tokenInfo *const parentToken,
 		switch (token->type)
 		{
 			case TOKEN_OPEN_CURLY:
-				enterScope (token, NULL, -1);
+				enterScope (token, NULL, KIND_GHOST_INDEX);
 				break;
 
 			case TOKEN_KEYWORD:
-				readNext = parseFunction (token);
+				switch (token->keyword)
+				{
+					case KEYWORD_function:
+						readNext = parseFunction (token, K_FUNCTION);
+						break;
+
+					case KEYWORD_filter:
+						readNext = parseFunction (token, K_FILTER);
+						break;
+
+					case KEYWORD_class:
+						readNext = parseClass (token);
+						break;
+
+					case KEYWORD_enum:
+						readNext = parseEnum (token);
+						break;
+
+					default: break;
+				}
 				break;
 
 			case TOKEN_VARIABLE:
 				readNext = parseVariable (token);
 				break;
 
+			case TOKEN_IDENTIFIER:
+				if (parentKind == K_ENUM)
+					readNext = parseEnumLabel (token);
+				break;
+
 			default: break;
 		}
 
@@ -589,7 +851,7 @@ static void findPowerShellTags (void)
 
 	do
 	{
-		enterScope (token, NULL, -1);
+		enterScope (token, NULL, KIND_GHOST_INDEX);
 	}
 	while (token->type != TOKEN_EOF); /* keep going even with unmatched braces */
 
@@ -604,5 +866,11 @@ extern parserDefinition* PowerShellParser (void)
 	def->kindCount  = ARRAY_SIZE (PowerShellKinds);
 	def->extensions = extensions;
 	def->parser     = findPowerShellTags;
+	def->keywordTable = PowerShellKeywordTable;
+	def->keywordCount = ARRAY_SIZE (PowerShellKeywordTable);
+
+	def->versionCurrent = 1;
+	def->versionAge = 1;
+
 	return def;
 }
