@@ -8,7 +8,7 @@
 #ifndef EDITMODEL_H
 #define EDITMODEL_H
 
-namespace Scintilla {
+namespace Scintilla::Internal {
 
 /**
 */
@@ -27,26 +27,31 @@ public:
 	int xOffset;		///< Horizontal scrolled amount in pixels
 	bool trackLineWidth;
 
-	SpecialRepresentations reprs;
+	std::unique_ptr<SpecialRepresentations> reprs;
 	Caret caret;
 	SelectionPosition posDrag;
 	Sci::Position braces[2];
 	int bracesMatchStyle;
 	int highlightGuideColumn;
+	bool hasFocus;
 	Selection sel;
 	bool primarySelection;
+	std::string copySeparator;
 
-	enum IMEInteraction { imeWindowed, imeInline } imeInteraction;
-	enum class CharacterSource { directInput, tentativeInput, imeResult };
+	Scintilla::IMEInteraction imeInteraction;
+	Scintilla::Bidirectional bidirectional;
 
-	int foldFlags;
-	int foldDisplayTextStyle;
+	Scintilla::FoldFlag foldFlags;
+	Scintilla::FoldDisplayTextStyle foldDisplayTextStyle;
 	UniqueString defaultFoldDisplayText;
 	std::unique_ptr<IContractionState> pcs;
 	// Hotspot support
 	Range hotspot;
+	bool hotspotSingleLine;
 	Sci::Position hoverIndicatorPos;
 
+	Scintilla::ChangeHistoryOption changeHistoryOption = Scintilla::ChangeHistoryOption::Disabled;
+
 	// Wrapping support
 	int wrapWidth;
 
@@ -59,13 +64,17 @@ public:
 	EditModel &operator=(const EditModel &) = delete;
 	EditModel &operator=(EditModel &&) = delete;
 	virtual ~EditModel();
-	virtual Sci::Line TopLineOfMain() const = 0;
+	virtual Sci::Line TopLineOfMain() const noexcept = 0;
 	virtual Point GetVisibleOriginInMain() const = 0;
 	virtual Sci::Line LinesOnScreen() const = 0;
-	virtual Range GetHotSpotRange() const noexcept = 0;
+	bool BidirectionalEnabled() const noexcept;
+	bool BidirectionalR2L() const noexcept;
+	SurfaceMode CurrentSurfaceMode() const noexcept;
 	void SetDefaultFoldDisplayText(const char *text);
 	const char *GetDefaultFoldDisplayText() const noexcept;
 	const char *GetFoldDisplayText(Sci::Line lineDoc) const noexcept;
+	InSelection LineEndInSelection(Sci::Line lineDoc) const;
+	[[nodiscard]] int GetMark(Sci::Line line) const;
 };
 
 }
