@@ -6,33 +6,35 @@
 // The License.txt file describes the conditions under which this software may be distributed.
 
 #include <cstddef>
+#include <cstdint>
 #include <cassert>
 #include <cstring>
 
 #include <stdexcept>
+#include <string_view>
 #include <vector>
 #include <forward_list>
+#include <optional>
 #include <algorithm>
 #include <memory>
 
+#include "ScintillaTypes.h"
+
+#include "Debugging.h"
+#include "Geometry.h"
 #include "Platform.h"
 
-#include "Scintilla.h"
 #include "Position.h"
 #include "SplitVector.h"
 #include "Partitioning.h"
 #include "CellBuffer.h"
 #include "PerLine.h"
 
-using namespace Scintilla;
+using namespace Scintilla::Internal;
 
 MarkerHandleSet::MarkerHandleSet() {
 }
 
-MarkerHandleSet::~MarkerHandleSet() {
-	mhList.clear();
-}
-
 bool MarkerHandleSet::Empty() const noexcept {
 	return mhList.empty();
 }
@@ -88,16 +90,13 @@ void MarkerHandleSet::CombineWith(MarkerHandleSet *other) noexcept {
 	mhList.splice_after(mhList.before_begin(), other->mhList);
 }
 
-LineMarkers::~LineMarkers() {
-}
-
 void LineMarkers::Init() {
 	markers.DeleteAll();
 }
 
 void LineMarkers::InsertLine(Sci::Line line) {
 	if (markers.Length()) {
-		markers.Insert(line, 0);
+		markers.Insert(line, nullptr);
 	}
 }
 
@@ -145,7 +144,7 @@ int LineMarkers::NumberFromLine(Sci::Line line, int which) const noexcept {
 void LineMarkers::MergeMarkers(Sci::Line line) {
 	if (markers[line + 1]) {
 		if (!markers[line])
-			markers[line] = Sci::make_unique<MarkerHandleSet>();
+			markers[line] = std::make_unique<MarkerHandleSet>();
 		markers[line]->CombineWith(markers[line + 1].get());
 		markers[line + 1].reset();
 	}
@@ -181,7 +180,7 @@ int LineMarkers::AddMark(Sci::Line line, int markerNum, Sci::Line lines) {
 	}
 	if (!markers[line]) {
 		// Need new structure to hold marker handle
-		markers[line] = Sci::make_unique<MarkerHandleSet>();
+		markers[line] = std::make_unique<MarkerHandleSet>();
 	}
 	markers[line]->InsertHandle(handleCurrent, markerNum);
 
@@ -214,23 +213,20 @@ void LineMarkers::DeleteMarkFromHandle(int markerHandle) {
 	}
 }
 
-LineLevels::~LineLevels() {
-}
-
 void LineLevels::Init() {
 	levels.DeleteAll();
 }
 
 void LineLevels::InsertLine(Sci::Line line) {
 	if (levels.Length()) {
-		const int level = (line < levels.Length()) ? levels[line] : SC_FOLDLEVELBASE;
+		const int level = (line < levels.Length()) ? levels[line] : static_cast<int>(Scintilla::FoldLevel::Base);
 		levels.Insert(line, level);
 	}
 }
 
 void LineLevels::InsertLines(Sci::Line line, Sci::Line lines) {
 	if (levels.Length()) {
-		const int level = (line < levels.Length()) ? levels[line] : SC_FOLDLEVELBASE;
+		const int level = (line < levels.Length()) ? levels[line] : static_cast<int>(Scintilla::FoldLevel::Base);
 		levels.InsertValue(line, lines, level);
 	}
 }
@@ -239,17 +235,17 @@ void LineLevels::RemoveLine(Sci::Line line) {
 	if (levels.Length()) {
 		// Move up following lines but merge header flag from this line
 		// to line before to avoid a temporary disappearance causing expansion.
-		int firstHeader = levels[line] & SC_FOLDLEVELHEADERFLAG;
+		int firstHeader = levels[line] & static_cast<int>(Scintilla::FoldLevel::HeaderFlag);
 		levels.Delete(line);
 		if (line == levels.Length()-1) // Last line loses the header flag
-			levels[line-1] &= ~SC_FOLDLEVELHEADERFLAG;
+			levels[line-1] &= ~static_cast<int>(Scintilla::FoldLevel::HeaderFlag);
 		else if (line > 0)
 			levels[line-1] |= firstHeader;
 	}
 }
 
 void LineLevels::ExpandLevels(Sci::Line sizeNew) {
-	levels.InsertValue(levels.Length(), sizeNew - levels.Length(), SC_FOLDLEVELBASE);
+	levels.InsertValue(levels.Length(), sizeNew - levels.Length(), static_cast<int>(Scintilla::FoldLevel::Base));
 }
 
 void LineLevels::ClearLevels() {
@@ -257,28 +253,40 @@ void LineLevels::ClearLevels() {
 }
 
 int LineLevels::SetLevel(Sci::Line line, int level, Sci::Line lines) {
-	int prev = 0;
+	int prev = level;
 	if ((line >= 0) && (line < lines)) {
 		if (!levels.Length()) {
 			ExpandLevels(lines + 1);
 		}
 		prev = levels[line];
-		if (prev != level) {
-			levels[line] = level;
-		}
+		levels[line] = level;
 	}
 	return prev;
 }
 
 int LineLevels::GetLevel(Sci::Line line) const noexcept {
-	if (levels.Length() && (line >= 0) && (line < levels.Length())) {
+	if ((line >= 0) && (line < levels.Length())) {
 		return levels[line];
-	} else {
-		return SC_FOLDLEVELBASE;
 	}
+	return static_cast<int>(Scintilla::FoldLevel::Base);
 }
 
-LineState::~LineState() {
+Scintilla::FoldLevel LineLevels::GetFoldLevel(Sci::Line line) const noexcept {
+	if ((line >= 0) && (line < levels.Length())) {
+		return static_cast<FoldLevel>(levels[line]);
+	}
+	return Scintilla::FoldLevel::Base;
+}
+
+Sci::Line LineLevels::GetFoldParent(Sci::Line line) const noexcept {
+	const FoldLevel level = LevelNumberPart(GetFoldLevel(line));
+	for (Sci::Line lineLook = line - 1; lineLook >= 0; lineLook--) {
+		const FoldLevel levelTry = GetFoldLevel(lineLook);
+		if (LevelIsHeader(levelTry) && LevelNumberPart(levelTry) < level) {
+			return lineLook;
+		}
+	}
+	return -1;
 }
 
 void LineState::Init() {
@@ -307,10 +315,13 @@ void LineState::RemoveLine(Sci::Line line) {
 	}
 }
 
-int LineState::SetLineState(Sci::Line line, int state) {
-	lineStates.EnsureLength(line + 1);
-	const int stateOld = lineStates[line];
-	lineStates[line] = state;
+int LineState::SetLineState(Sci::Line line, int state, Sci::Line lines) {
+	int stateOld = state;
+	if ((line >= 0) && (line < lines)) {
+		lineStates.EnsureLength(lines + 1);
+		stateOld = lineStates[line];
+		lineStates[line] = state;
+	}
 	return stateOld;
 }
 
@@ -338,26 +349,19 @@ namespace {
 
 constexpr int IndividualStyles = 0x100;
 
-size_t NumberLines(const char *text) noexcept {
-	int lines = 1;
-	if (text) {
-		while (*text) {
-			if (*text == '\n')
-				lines++;
-			text++;
-		}
-	}
-	return lines;
+size_t NumberLines(std::string_view sv) {
+	return std::count(sv.begin(), sv.end(), '\n') + 1;
 }
 
 std::unique_ptr<char[]>AllocateAnnotation(size_t length, int style) {
 	const size_t len = sizeof(AnnotationHeader) + length + ((style == IndividualStyles) ? length : 0);
-	return Sci::make_unique<char[]>(len);
+	return std::make_unique<char[]>(len);
 }
 
 }
 
-LineAnnotation::~LineAnnotation() {
+bool LineAnnotation::Empty() const noexcept {
+	return annotations.Length() == 0;
 }
 
 void LineAnnotation::Init() {
@@ -480,9 +484,6 @@ int LineAnnotation::Lines(Sci::Line line) const noexcept {
 		return 0;
 }
 
-LineTabstops::~LineTabstops() {
-}
-
 void LineTabstops::Init() {
 	tabstops.DeleteAll();
 }
@@ -522,7 +523,7 @@ bool LineTabstops::ClearTabstops(Sci::Line line) noexcept {
 bool LineTabstops::AddTabstop(Sci::Line line, int x) {
 	tabstops.EnsureLength(line + 1);
 	if (!tabstops[line]) {
-		tabstops[line] = Sci::make_unique<TabstopList>();
+		tabstops[line] = std::make_unique<TabstopList>();
 	}
 
 	TabstopList *tl = tabstops[line].get();
@@ -540,7 +541,7 @@ bool LineTabstops::AddTabstop(Sci::Line line, int x) {
 
 int LineTabstops::GetNextTabstop(Sci::Line line, int x) const noexcept {
 	if (line < tabstops.Length()) {
-		TabstopList *tl = tabstops[line].get();
+		const TabstopList *tl = tabstops[line].get();
 		if (tl) {
 			for (const int i : *tl) {
 				if (i > x) {
