@@ -137,7 +137,7 @@ typedef enum
 
 static gpointer last_toolbutton_action = GBO_TO_POINTER(GEANY_GBO_BUILD);
 
-static BuildMenuItems menu_items = {NULL, {NULL, NULL, NULL, NULL}};
+static BuildMenuItems build_menu_items = {NULL, {NULL, NULL, NULL, NULL}};
 
 static struct
 {
@@ -179,8 +179,8 @@ void build_finalize(void)
 	g_free(build_info.dir);
 	g_free(build_info.custom_target);
 
-	if (menu_items.menu != NULL && GTK_IS_WIDGET(menu_items.menu))
-		gtk_widget_destroy(menu_items.menu);
+	if (build_menu_items.menu != NULL && GTK_IS_WIDGET(build_menu_items.menu))
+		gtk_widget_destroy(build_menu_items.menu);
 }
 
 
@@ -708,70 +708,12 @@ static void clear_all_errors(void)
 }
 
 
-/* Replaces occurrences of %e and %p with the appropriate filenames and
- * %l with current line number. %d and %p replacements should be in UTF8 */
-static gchar *build_replace_placeholder(const GeanyDocument *doc, const gchar *src)
-{
-	GString *stack;
-	gchar *replacement;
-	gchar *executable = NULL;
-	gint line_num;
-
-	g_return_val_if_fail(doc == NULL || doc->is_valid, NULL);
-
-	stack = g_string_new(src);
-	if (doc != NULL && doc->file_name != NULL)
-	{
-		/* replace %f with the filename (including extension) */
-		replacement = g_path_get_basename(doc->file_name);
-		utils_string_replace_all(stack, "%f", replacement);
-		g_free(replacement);
-
-		/* replace %d with the absolute path of the dir of the current file */
-		replacement = g_path_get_dirname(doc->file_name);
-		utils_string_replace_all(stack, "%d", replacement);
-		g_free(replacement);
-
-		/* replace %e with the filename (excluding extension) */
-		executable = utils_remove_ext_from_filename(doc->file_name);
-		replacement = g_path_get_basename(executable);
-		utils_string_replace_all(stack, "%e", replacement);
-		g_free(replacement);
-
-		/* replace %l with the current 1-based line number */
-		line_num = sci_get_current_line(doc->editor->sci) + 1;
-		replacement = g_strdup_printf("%i", line_num);
-		utils_string_replace_all(stack, "%l", replacement);
-		g_free(replacement);
-	}
-
-	/* replace %p with the current project's (absolute) base directory */
-	replacement = NULL; /* prevent double free if no replacement found */
-	if (app->project)
-	{
-		replacement = project_get_base_path();
-	}
-	else if (strstr(stack->str, "%p"))
-	{   /* fall back to %d */
-		ui_set_statusbar(FALSE, _("failed to substitute %%p, no project active"));
-		if (doc != NULL && doc->file_name != NULL)
-			replacement = g_path_get_dirname(doc->file_name);
-	}
-
-	utils_string_replace_all(stack, "%p", replacement);
-	g_free(replacement);
-	g_free(executable);
-
-	return g_string_free(stack, FALSE); /* don't forget to free src also if needed */
-}
-
-
 /* dir is the UTF-8 working directory to run cmd in. It can be NULL to use the
  * idx document directory */
 static void build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *dir)
 {
 	GError *error = NULL;
-	gchar *argv[] = { "/bin/sh", "-c", NULL, NULL };
+	const gchar *argv[] = { "/bin/sh", "-c", NULL, NULL };
 	gchar *working_dir;
 	gchar *utf8_working_dir;
 	gchar *cmd_string;
@@ -792,6 +734,8 @@ static void build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *d
 	working_dir = utils_get_locale_from_utf8(utf8_working_dir);
 
 	gtk_list_store_clear(msgwindow.store_compiler);
+	// reset width after any long error messages
+	gtk_tree_view_columns_autosize(GTK_TREE_VIEW(msgwindow.tree_compiler));
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_COMPILER);
 	msgwin_compiler_add(COLOR_BLUE, _("%s (in directory: %s)"), cmd, utf8_working_dir);
 	g_free(utf8_working_dir);
@@ -813,7 +757,7 @@ static void build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *d
 	build_info.file_type_id = (doc == NULL) ? GEANY_FILETYPES_NONE : doc->file_type->id;
 	build_info.message_count = 0;
 
-	if (!spawn_with_callbacks(working_dir, cmd, argv, NULL, 0, NULL, NULL, build_iofunc,
+	if (!spawn_with_callbacks(working_dir, cmd, (gchar **) argv, NULL, 0, NULL, NULL, build_iofunc,
 		GINT_TO_POINTER(0), 0, build_iofunc, GINT_TO_POINTER(1), 0, build_exit_cb, NULL,
 		&build_info.pid, &error))
 	{
@@ -842,11 +786,11 @@ static gchar *prepare_run_cmd(GeanyDocument *doc, gchar **working_dir, guint cmd
 
 	cmd = get_build_cmd(doc, GEANY_GBG_EXEC, cmdindex, NULL);
 
-	cmd_string_utf8 = build_replace_placeholder(doc, cmd->command);
+	cmd_string_utf8 = utils_replace_document_placeholders(doc, cmd->command);
 	cmd_working_dir =  cmd->working_dir;
 	if (EMPTY(cmd_working_dir))
 		cmd_working_dir = "%d";
-	working_dir_utf8 = build_replace_placeholder(doc, cmd_working_dir);
+	working_dir_utf8 = utils_replace_document_placeholders(doc, cmd_working_dir);
 	*working_dir = utils_get_locale_from_utf8(working_dir_utf8);
 
 	if (EMPTY(*working_dir) || ! g_file_test(*working_dir, G_FILE_TEST_EXISTS) ||
@@ -1219,8 +1163,8 @@ static void build_command(GeanyDocument *doc, GeanyBuildGroup grp, guint cmd, gc
 	else
 		full_command = cmdstr;
 
-	dir = build_replace_placeholder(doc, buildcmd->working_dir);
-	subs_command = build_replace_placeholder(doc, full_command);
+	dir = utils_replace_document_placeholders(doc, buildcmd->working_dir);
+	subs_command = utils_replace_document_placeholders(doc, full_command);
 	build_info.grp = grp;
 	build_info.cmd = cmd;
 	build_spawn_cmd(doc, subs_command, dir);
@@ -1266,17 +1210,24 @@ static void on_build_menu_item(GtkWidget *w, gpointer user_data)
 	if (grp == GEANY_GBG_NON_FT && cmd == GBO_TO_CMD(GEANY_GBO_CUSTOM))
 	{
 		static GtkWidget *dialog = NULL; /* keep dialog for combo history */
+		gchar *str;
 
+		bc = get_build_cmd(doc, grp, cmd, NULL);
+		str = g_strdup_printf(_("Enter custom text to append to the command \"%s\":"),
+			bc->command);
 		if (! dialog)
 		{
-			dialog = dialogs_show_input_persistent(_("Custom Text"), GTK_WINDOW(main_widgets.window),
-				_("Enter custom text here, all entered text is appended to the command."),
-				build_info.custom_target, &on_make_custom_input_response, NULL);
+			dialog = dialogs_show_input_persistent(NULL, GTK_WINDOW(main_widgets.window),
+				str, build_info.custom_target, &on_make_custom_input_response, NULL);
 		}
 		else
 		{
+			gtk_label_set_label(GTK_LABEL(ui_lookup_widget(dialog, "label")), str);
 			gtk_widget_show(dialog);
 		}
+		SETPTR(str, utils_str_remove_chars(g_strdup(bc->label), "_"));
+		gtk_window_set_title(GTK_WINDOW(dialog), str);
+		g_free(str);
 		return;
 	}
 	else if (grp == GEANY_GBG_EXEC)
@@ -1385,11 +1336,11 @@ static void create_build_menu_item(GtkWidget *menu, GeanyKeyGroup *group, GtkAcc
 	{
 		g_signal_connect(item, "activate", G_CALLBACK(bs->cb), GRP_CMD_TO_POINTER(grp,cmd));
 	}
-	menu_items.menu_item[grp][cmd] = item;
+	build_menu_items.menu_item[grp][cmd] = item;
 }
 
 
-static void create_build_menu(BuildMenuItems *build_menu_items)
+static void create_build_menu(BuildMenuItems *menu_items)
 {
 	GtkWidget *menu;
 	GtkAccelGroup *accel_group = gtk_accel_group_new();
@@ -1397,10 +1348,10 @@ static void create_build_menu(BuildMenuItems *build_menu_items)
 	guint i, j;
 
 	menu = gtk_menu_new();
-	build_menu_items->menu_item[GEANY_GBG_FT] = g_new0(GtkWidget*, build_groups_count[GEANY_GBG_FT]);
-	build_menu_items->menu_item[GEANY_GBG_NON_FT] = g_new0(GtkWidget*, build_groups_count[GEANY_GBG_NON_FT]);
-	build_menu_items->menu_item[GEANY_GBG_EXEC] = g_new0(GtkWidget*, build_groups_count[GEANY_GBG_EXEC]);
-	build_menu_items->menu_item[GBG_FIXED] = g_new0(GtkWidget*, GBF_COUNT);
+	menu_items->menu_item[GEANY_GBG_FT] = g_new0(GtkWidget*, build_groups_count[GEANY_GBG_FT]);
+	menu_items->menu_item[GEANY_GBG_NON_FT] = g_new0(GtkWidget*, build_groups_count[GEANY_GBG_NON_FT]);
+	menu_items->menu_item[GEANY_GBG_EXEC] = g_new0(GtkWidget*, build_groups_count[GEANY_GBG_EXEC]);
+	menu_items->menu_item[GBG_FIXED] = g_new0(GtkWidget*, GBF_COUNT);
 
 	for (i = 0; build_menu_specs[i].build_grp != MENU_DONE; ++i)
 	{
@@ -1410,7 +1361,7 @@ static void create_build_menu(BuildMenuItems *build_menu_items)
 			GtkWidget *item = gtk_separator_menu_item_new();
 			gtk_widget_show(item);
 			gtk_container_add(GTK_CONTAINER(menu), item);
-			build_menu_items->menu_item[GBG_FIXED][bs->build_cmd] = item;
+			menu_items->menu_item[GBG_FIXED][bs->build_cmd] = item;
 		}
 		else if (bs->fix_label != NULL)
 		{
@@ -1434,7 +1385,7 @@ static void create_build_menu(BuildMenuItems *build_menu_items)
 			create_build_menu_item(menu, keygroup, accel_group, bs, lbl, bs->build_grp, bs->build_cmd);
 		}
 	}
-	build_menu_items->menu = menu;
+	menu_items->menu = menu;
 	gtk_widget_show(menu);
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(ui_lookup_widget(main_widgets.window, "menu_build1")), menu);
 }
@@ -1464,8 +1415,8 @@ void build_menu_update(GeanyDocument *doc)
 
 	g_return_if_fail(doc == NULL || doc->is_valid);
 
-	if (menu_items.menu == NULL)
-		create_build_menu(&menu_items);
+	if (build_menu_items.menu == NULL)
+		create_build_menu(&build_menu_items);
 	if (doc == NULL)
 		doc = document_get_current();
 	have_path = doc != NULL && doc->file_name != NULL;
@@ -1481,15 +1432,15 @@ void build_menu_update(GeanyDocument *doc)
 			case MENU_SEPARATOR:
 				if (vis == TRUE)
 				{
-					gtk_widget_show_all(menu_items.menu_item[GBG_FIXED][bs->build_cmd]);
+					gtk_widget_show_all(build_menu_items.menu_item[GBG_FIXED][bs->build_cmd]);
 					vis = FALSE;
 				}
 				else
-					gtk_widget_hide(menu_items.menu_item[GBG_FIXED][bs->build_cmd]);
+					gtk_widget_hide(build_menu_items.menu_item[GBG_FIXED][bs->build_cmd]);
 				break;
 			case MENU_NEXT_ERROR:
 			case MENU_PREV_ERROR:
-				gtk_widget_set_sensitive(menu_items.menu_item[GBG_FIXED][bs->build_cmd], have_errors);
+				gtk_widget_set_sensitive(build_menu_items.menu_item[GBG_FIXED][bs->build_cmd], have_errors);
 				vis |= TRUE;
 				break;
 			case MENU_COMMANDS:
@@ -1508,7 +1459,7 @@ void build_menu_update(GeanyDocument *doc)
 				}
 				for (cmd = bs->build_cmd; cmd < cmdcount; ++cmd)
 				{
-					GtkWidget *menu_item = menu_items.menu_item[grp][cmd];
+					GtkWidget *menu_item = build_menu_items.menu_item[grp][cmd];
 					const gchar *label;
 					bc = get_build_cmd(doc, grp, cmd, NULL);
 					if (bc)
@@ -2266,7 +2217,7 @@ BuildMenuItems *build_get_menu_items(gint filetype_idx)
 {
 	BuildMenuItems *items;
 
-	items = &menu_items;
+	items = &build_menu_items;
 	if (items->menu == NULL)
 		create_build_menu(items);
 	return items;
@@ -2352,7 +2303,7 @@ static void assign_cmd(GeanyBuildCommand *type, guint id,
 }
 
 /* for the specified source load new format build menu items or try to make some sense of
- * old format setings, not done perfectly but better than ignoring them */
+ * old format settings, not done perfectly but better than ignoring them */
 void build_load_menu(GKeyFile *config, GeanyBuildSource src, gpointer p)
 {
 	GeanyFiletype *ft;
