@@ -475,6 +475,37 @@ void on_entry_tagfilter_activate(GtkEntry *entry, gpointer user_data)
 }
 
 
+void on_entry_docfilter_changed(GtkAction *action, gpointer user_data)
+{
+	GeanyDocument *doc = document_get_current();
+	GtkEntry *filter_entry;
+
+	if (!doc)
+		return;
+
+	filter_entry = GTK_ENTRY(ui_lookup_widget(main_widgets.window, "entry_docfilter"));
+	sidebar_openfiles_set_filter(gtk_entry_get_text(filter_entry));
+}
+
+
+void on_entry_docfilter_icon_press(GtkEntry *entry, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer user_data)
+{
+	if (event->button.button == 1)
+		gtk_entry_set_text(entry, "");
+}
+
+
+void on_entry_docfilter_activate(GtkEntry *entry, gpointer user_data)
+{
+	GeanyDocument *doc = document_get_current();
+
+	if (!doc)
+		return;
+
+	sidebar_focus_openfiles_tab();
+}
+
+
 /* hides toolbar from toolbar popup menu */
 static void on_hide_toolbar1_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
@@ -515,18 +546,8 @@ void on_normal_size1_activate(GtkMenuItem *menuitem, gpointer user_data)
 }
 
 
-/* Changes window-title after switching tabs and lots of other things.
- * note: using 'after' makes Scintilla redraw before the UI, appearing more responsive */
-static void on_notebook1_switch_page_after(GtkNotebook *notebook, gpointer page,
-		guint page_num, gpointer user_data)
+static void handle_switch_page(GeanyDocument *doc)
 {
-	GeanyDocument *doc;
-
-	if (G_UNLIKELY(main_status.opening_session_files || main_status.closing_all))
-		return;
-
-	doc = document_get_from_notebook_child(page);
-
 	if (doc != NULL)
 	{
 		GtkEntry *filter_entry = GTK_ENTRY(ui_lookup_widget(main_widgets.window, "entry_tagfilter"));
@@ -535,7 +556,7 @@ static void on_notebook1_switch_page_after(GtkNotebook *notebook, gpointer page,
 		sidebar_select_openfiles_item(doc);
 		ui_save_buttons_toggle(doc->changed);
 		ui_set_window_title(doc);
-		ui_update_statusbar(doc, -1);
+		ui_update_statusbar(doc);
 		ui_update_popup_reundo_items(doc);
 		ui_document_show_hide(doc); /* update the document menu */
 		build_menu_update(doc);
@@ -559,6 +580,53 @@ static void on_notebook1_switch_page_after(GtkNotebook *notebook, gpointer page,
 }
 
 
+static gboolean delay_handle_switch_page(gpointer data)
+{
+	gulong *handler_id = data;
+
+	if (main_status.opening_session_files)
+		return G_SOURCE_CONTINUE;
+	/* guard against the unlikely case where we didn't run yet but are already
+	 * closing all documents */
+	else if (! main_status.closing_all)
+		handle_switch_page(document_get_current());
+
+	*handler_id = 0;
+	return G_SOURCE_REMOVE;
+}
+
+
+/* Changes window-title after switching tabs and lots of other things.
+ * note: using 'after' makes Scintilla redraw before the UI, appearing more responsive
+ *
+ * When page switch happens while opening session files, we delay handling of the
+ * event to when session opening is complete.  This is mostly to avoid repeatedly update
+ * the UI unnecessarily, which isn't entirely cheap */
+static void on_notebook1_switch_page_after(GtkNotebook *notebook, gpointer page,
+		guint page_num, gpointer user_data)
+{
+	static gulong handler_id = 0;
+
+	if (main_status.opening_session_files)
+	{
+		/* if opening session files, delay the handling to after session is fully open */
+		if (handler_id == 0)
+			handler_id = g_idle_add(delay_handle_switch_page, &handler_id);
+		return;
+	}
+	else if (main_status.closing_all)
+		return;
+
+	if (handler_id != 0)
+	{
+		g_source_remove(handler_id);
+		handler_id = 0;
+	}
+
+	handle_switch_page(document_get_from_notebook_child(page));
+}
+
+
 static void on_tv_notebook_switch_page(GtkNotebook *notebook, gpointer page,
 		guint page_num, gpointer user_data)
 {
@@ -589,7 +657,7 @@ static void convert_eol(gint mode)
 
 	sci_set_eol_mode(doc->editor->sci, mode);
 
-	ui_update_statusbar(doc, -1);
+	ui_update_statusbar(doc);
 }
 
 
@@ -801,7 +869,7 @@ static void on_set_file_readonly1_toggled(GtkCheckMenuItem *checkmenuitem, gpoin
 		doc->readonly = ! doc->readonly;
 		sci_set_readonly(doc->editor->sci, doc->readonly);
 		ui_update_tab_status(doc);
-		ui_update_statusbar(doc, -1);
+		ui_update_statusbar(doc);
 	}
 }
 
@@ -1319,7 +1387,7 @@ static void on_menu_write_unicode_bom1_toggled(GtkCheckMenuItem *checkmenuitem,
 
 		doc->has_bom = ! doc->has_bom;
 
-		ui_update_statusbar(doc, -1);
+		ui_update_statusbar(doc);
 	}
 }
 
@@ -1658,7 +1726,7 @@ static void set_indent_type(GtkCheckMenuItem *menuitem, GeanyIndentType type)
 	g_return_if_fail(doc != NULL);
 
 	editor_set_indent(doc->editor, type, doc->editor->indent_width);
-	ui_update_statusbar(doc, -1);
+	ui_update_statusbar(doc);
 }
 
 
@@ -1966,7 +2034,7 @@ static void on_reset_indentation1_activate(GtkMenuItem *menuitem, gpointer user_
 	foreach_document(i)
 		document_apply_indent_settings(documents[i]);
 
-	ui_update_statusbar(NULL, -1);
+	ui_update_statusbar(NULL);
 	ui_document_show_hide(NULL);
 }
 
@@ -1986,7 +2054,7 @@ static void on_detect_type_from_file_activate(GtkMenuItem *menuitem, gpointer us
 	{
 		editor_set_indent_type(doc->editor, type);
 		ui_document_show_hide(doc);
-		ui_update_statusbar(doc, -1);
+		ui_update_statusbar(doc);
 	}
 }
 
