@@ -84,9 +84,9 @@ typedef struct GeanyDocumentPrivate
 	/* Iter for this document within the Open Files treeview of the sidebar. */
 	GtkTreeIter		 iter;
 	/* Used by the Undo/Redo management code. */
-	GTrashStack		*undo_actions;
+	GSList			*undo_actions;
 	/* Used by the Undo/Redo management code. */
-	GTrashStack		*redo_actions;
+	GSList			*redo_actions;
 	/* Used so Undo/Redo works for encoding changes. */
 	FileEncoding	 saved_encoding;
 	gboolean		 colourise_needed;	/* use document.c:queue_colourise() instead */
@@ -108,7 +108,7 @@ typedef struct GeanyDocumentPrivate
 	/* Whether it's temporarily protected (read-only and saving needs confirmation). Does
 	 * not imply doc->readonly as writable files can be protected */
 	gint			 protected;
-	/* Save pointer to info bars allowing to cancel them programatically (to avoid multiple ones) */
+	/* Save pointer to info bars allowing to cancel them programmatically (to avoid multiple ones) */
 	GtkWidget		*info_bars[NUM_MSG_TYPES];
 	/* Keyed Data List to attach arbitrary data to the document */
 	GData			*data;
