@@ -57,6 +57,8 @@ typedef enum
 	GCB_SAVE_SETTINGS,
 	GCB_LOAD_SETTINGS,
 	GCB_KEY_PRESS_NOTIFY,
+	GCB_DOCUMENT_BEFORE_SAVE_AS,
+	GCB_GEANY_BEFORE_QUIT,
 	GCB_MAX
 }
 GeanyCallbackId;
