@@ -594,7 +594,7 @@ static void set_character_classes(ScintillaObject *sci, guint ft_id)
 	SSM(sci, SCI_SETWORDCHARS, 0, (sptr_t) word);
 
 	/* setting wordchars resets character classes, so we have to set whitespaces after
-	 * wordchars, but we want wordchars to have precenence over whitepace chars */
+	 * wordchars, but we want wordchars to have precedence over whitepace chars */
 	whitespace = g_malloc0(strlen(whitespace_chars) + 1);
 	for (i = 0, j = 0; whitespace_chars[i] != 0; i++)
 	{
@@ -1007,12 +1007,14 @@ void highlighting_init_styles(guint filetype_idx, GKeyFile *config, GKeyFile *co
 		init_styleset_case(BATCH);
 		init_styleset_case(C);
 		init_styleset_case(CAML);
+		init_styleset_case(CIL);
 		init_styleset_case(CMAKE);
 		init_styleset_case(COBOL);
 		init_styleset_case(COFFEESCRIPT);
 		init_styleset_case(CONF);
 		init_styleset_case(CSS);
 		init_styleset_case(D);
+		init_styleset_case(DART);
 		init_styleset_case(DIFF);
 		init_styleset_case(LISP);
 		init_styleset_case(ERLANG);
@@ -1034,6 +1036,8 @@ void highlighting_init_styles(guint filetype_idx, GKeyFile *config, GKeyFile *co
 		init_styleset_case(MAKE);
 		init_styleset_case(MATLAB);
 		init_styleset_case(MARKDOWN);
+		init_styleset_case(NIM);
+		init_styleset_case(NIX);
 		init_styleset_case(NSIS);
 		init_styleset_case(OBJECTIVEC);
 		init_styleset_case(PASCAL);
@@ -1041,20 +1045,24 @@ void highlighting_init_styles(guint filetype_idx, GKeyFile *config, GKeyFile *co
 		init_styleset_case(PHP);
 		init_styleset_case(PO);
 		init_styleset_case(POWERSHELL);
+		init_styleset_case(PROLOG);
 		init_styleset_case(PYTHON);
 		init_styleset_case(R);
+		init_styleset_case(RAKU);
 		init_styleset_case(RUBY);
 		init_styleset_case(RUST);
 		init_styleset_case(SH);
 		init_styleset_case(SMALLTALK);
 		init_styleset_case(SQL);
 		init_styleset_case(TCL);
+		init_styleset_case(TOML);
 		init_styleset_case(TXT2TAGS);
 		init_styleset_case(VHDL);
 		init_styleset_case(VERILOG);
 		init_styleset_case(XML);
 		init_styleset_case(YAML);
 		init_styleset_case(ZEPHIR);
+		init_styleset_case(ZIG);
 		default:
 			if (ft->lexer_filetype)
 				geany_debug("Filetype %s has a recursive lexer_filetype %s set!",
@@ -1099,12 +1107,14 @@ void highlighting_set_styles(ScintillaObject *sci, GeanyFiletype *ft)
 		styleset_case(BATCH);
 		styleset_case(C);
 		styleset_case(CAML);
+		styleset_case(CIL);
 		styleset_case(CMAKE);
 		styleset_case(COBOL);
 		styleset_case(COFFEESCRIPT);
 		styleset_case(CONF);
 		styleset_case(CSS);
 		styleset_case(D);
+		styleset_case(DART);
 		styleset_case(DIFF);
 		styleset_case(LISP);
 		styleset_case(ERLANG);
@@ -1126,6 +1136,8 @@ void highlighting_set_styles(ScintillaObject *sci, GeanyFiletype *ft)
 		styleset_case(MAKE);
 		styleset_case(MARKDOWN);
 		styleset_case(MATLAB);
+		styleset_case(NIM);
+		styleset_case(NIX);
 		styleset_case(NSIS);
 		styleset_case(OBJECTIVEC);
 		styleset_case(PASCAL);
@@ -1133,20 +1145,24 @@ void highlighting_set_styles(ScintillaObject *sci, GeanyFiletype *ft)
 		styleset_case(PHP);
 		styleset_case(PO);
 		styleset_case(POWERSHELL);
+		styleset_case(PROLOG);
 		styleset_case(PYTHON);
 		styleset_case(R);
+		styleset_case(RAKU);
 		styleset_case(RUBY);
 		styleset_case(RUST);
 		styleset_case(SH);
 		styleset_case(SMALLTALK);
 		styleset_case(SQL);
 		styleset_case(TCL);
+		styleset_case(TOML);
 		styleset_case(TXT2TAGS);
 		styleset_case(VHDL);
 		styleset_case(VERILOG);
 		styleset_case(XML);
 		styleset_case(YAML);
 		styleset_case(ZEPHIR);
+		styleset_case(ZIG);
 		case GEANY_FILETYPES_NONE:
 		default:
 			styleset_default(sci, ft->id);
@@ -1483,6 +1499,16 @@ gboolean highlighting_is_string_style(gint lexer, gint style)
 		case SCLEX_R:
 			return (style == SCE_R_STRING);
 
+		case SCLEX_RAKU:
+			return (style == SCE_RAKU_CHARACTER ||
+				style == SCE_RAKU_HEREDOC_Q ||
+				style == SCE_RAKU_HEREDOC_QQ ||
+				style == SCE_RAKU_STRING ||
+				style == SCE_RAKU_STRING_Q ||
+				style == SCE_RAKU_STRING_QQ ||
+				style == SCE_RAKU_STRING_Q_LANG ||
+				style == SCE_RAKU_REGEX);
+
 		case SCLEX_RUBY:
 			return (style == SCE_RB_CHARACTER ||
 				style == SCE_RB_STRING ||
@@ -1548,10 +1574,12 @@ gboolean highlighting_is_string_style(gint lexer, gint style)
 				style == SCE_HJA_SINGLESTRING ||
 				style == SCE_HJA_STRINGEOL ||
 				style == SCE_HJA_REGEX ||
+				style == SCE_HJA_TEMPLATELITERAL ||
 				style == SCE_HJ_DOUBLESTRING ||
 				style == SCE_HJ_SINGLESTRING ||
 				style == SCE_HJ_STRINGEOL ||
 				style == SCE_HJ_REGEX ||
+				style == SCE_HJ_TEMPLATELITERAL ||
 				style == SCE_HPA_CHARACTER ||
 				style == SCE_HPA_STRING ||
 				style == SCE_HPA_TRIPLE ||
@@ -1567,6 +1595,10 @@ gboolean highlighting_is_string_style(gint lexer, gint style)
 				style == SCE_H_SGML_SIMPLESTRING ||
 				style == SCE_H_SINGLESTRING);
 
+		case SCLEX_CIL:
+			return (style == SCE_CIL_STRING ||
+				style == SCE_CIL_STRINGEOL);
+
 		case SCLEX_CMAKE:
 			return (style == SCE_CMAKE_STRINGDQ ||
 				style == SCE_CMAKE_STRINGLQ ||
@@ -1595,6 +1627,8 @@ gboolean highlighting_is_string_style(gint lexer, gint style)
 				style == SCE_RUST_STRINGR ||
 				style == SCE_RUST_BYTESTRING ||
 				style == SCE_RUST_BYTESTRINGR ||
+				style == SCE_RUST_CSTRING ||
+				style == SCE_RUST_CSTRINGR ||
 				style == SCE_RUST_LEXERROR);
 
 		case SCLEX_COFFEESCRIPT:
@@ -1634,6 +1668,56 @@ gboolean highlighting_is_string_style(gint lexer, gint style)
 			return (style == SCE_POWERSHELL_STRING ||
 				style == SCE_POWERSHELL_CHARACTER);
 
+		case SCLEX_VISUALPROLOG:
+			return (style == SCE_VISUALPROLOG_STRING ||
+				style == SCE_VISUALPROLOG_STRING_QUOTE ||
+				style == SCE_VISUALPROLOG_STRING_ESCAPE ||
+				style == SCE_VISUALPROLOG_STRING_ESCAPE_ERROR ||
+				style == SCE_VISUALPROLOG_STRING_EOL ||
+				style == SCE_VISUALPROLOG_EMBEDDED ||
+				style == SCE_VISUALPROLOG_PLACEHOLDER);
+
+		case SCLEX_TOML:
+			return (style == SCE_TOML_STRING_SQ ||
+				style == SCE_TOML_STRING_DQ ||
+				style == SCE_TOML_TRIPLE_STRING_SQ ||
+				style == SCE_TOML_TRIPLE_STRING_DQ ||
+				style == SCE_TOML_STRINGEOL);
+
+		case SCLEX_AU3:
+			return (style == SCE_AU3_STRING);
+
+		case SCLEX_NIM:
+			return (style == SCE_NIM_STRING ||
+				style == SCE_NIM_CHARACTER ||
+				style == SCE_NIM_TRIPLE ||
+				style == SCE_NIM_TRIPLEDOUBLE ||
+				style == SCE_NIM_STRINGEOL);
+
+		case SCLEX_ZIG:
+			return (style == SCE_ZIG_STRING ||
+				style == SCE_ZIG_MULTISTRING ||
+				style == SCE_ZIG_CHARACTER ||
+				style == SCE_ZIG_ESCAPECHAR ||
+				style == SCE_ZIG_STRINGEOL);
+
+		case SCLEX_DART:
+			return (style == SCE_DART_STRING_SQ ||
+				style == SCE_DART_STRING_DQ ||
+				style == SCE_DART_TRIPLE_STRING_SQ ||
+				style == SCE_DART_TRIPLE_STRING_DQ ||
+				style == SCE_DART_RAWSTRING_SQ ||
+				style == SCE_DART_RAWSTRING_DQ ||
+				style == SCE_DART_TRIPLE_RAWSTRING_SQ ||
+				style == SCE_DART_TRIPLE_RAWSTRING_DQ ||
+				style == SCE_DART_STRINGEOL);
+
+		case SCLEX_NIX:
+			return (style == SCE_NIX_STRING ||
+				style == SCE_NIX_STRING_MULTILINE ||
+				style == SCE_NIX_ESCAPECHAR ||
+				style == SCE_NIX_STRINGEOL);
+
 		case SCLEX_BATCH:
 		case SCLEX_DIFF:
 		case SCLEX_LATEX:
@@ -1644,9 +1728,6 @@ gboolean highlighting_is_string_style(gint lexer, gint style)
 		case SCLEX_YAML:
 			/* there is no string type in those lexers, listing here just for completeness */
 			return FALSE;
-
-		case SCLEX_AU3:
-			return (style == SCE_AU3_STRING);
 	}
 	return FALSE;
 }
@@ -1701,6 +1782,11 @@ gboolean highlighting_is_comment_style(gint lexer, gint style)
 		case SCLEX_PERL:
 			return (style == SCE_PL_COMMENTLINE);
 
+		case SCLEX_RAKU:
+			return (style == SCE_RAKU_COMMENTLINE ||
+				style == SCE_RAKU_COMMENTEMBED ||
+				style == SCE_RAKU_POD);
+
 		case SCLEX_PROPERTIES:
 			return (style == SCE_PROPS_COMMENT);
 
@@ -1787,6 +1873,11 @@ gboolean highlighting_is_comment_style(gint lexer, gint style)
 				style == SCE_HPHP_COMMENTLINE ||
 				style == SCE_H_SGML_COMMENT);
 
+		case SCLEX_CIL:
+			return (
+				style == SCE_CIL_COMMENT ||
+				style == SCE_CIL_COMMENTLINE);
+
 		case SCLEX_CMAKE:
 			return (style == SCE_CMAKE_COMMENT);
 
@@ -1877,6 +1968,36 @@ gboolean highlighting_is_comment_style(gint lexer, gint style)
 		case SCLEX_AU3:
 			return (style == SCE_AU3_COMMENT ||
 				style == SCE_AU3_COMMENTBLOCK);
+
+		case SCLEX_VISUALPROLOG:
+			return (style == SCE_VISUALPROLOG_COMMENT_BLOCK ||
+				style == SCE_VISUALPROLOG_COMMENT_LINE ||
+				style == SCE_VISUALPROLOG_COMMENT_KEY ||
+				style == SCE_VISUALPROLOG_COMMENT_KEY_ERROR);
+
+		case SCLEX_NIM:
+			return (style == SCE_NIM_COMMENT ||
+				style == SCE_NIM_COMMENTDOC ||
+				style == SCE_NIM_COMMENTLINE ||
+				style == SCE_NIM_COMMENTLINEDOC);
+
+		case SCLEX_ZIG:
+			return (style == SCE_ZIG_COMMENTLINE ||
+				style == SCE_ZIG_COMMENTLINEDOC ||
+				style == SCE_ZIG_COMMENTLINETOP);
+
+		case SCLEX_DART:
+			return (style == SCE_DART_COMMENTLINE ||
+				style == SCE_DART_COMMENTLINEDOC ||
+				style == SCE_DART_COMMENTBLOCK ||
+				style == SCE_DART_COMMENTBLOCKDOC);
+
+		case SCLEX_NIX:
+			return (style == SCE_NIX_COMMENTLINE ||
+				style == SCE_NIX_COMMENTBLOCK);
+
+		case SCLEX_TOML:
+			return (style == SCE_TOML_COMMENT);
 	}
 	return FALSE;
 }
