@@ -79,24 +79,26 @@
 #define GEANY_DISK_CHECK_TIMEOUT		30
 #define GEANY_DEFAULT_TOOLS_MAKE		"make"
 #ifdef G_OS_WIN32
-#define GEANY_DEFAULT_TOOLS_TERMINAL	"cmd.exe /Q /C %c"
+# define GEANY_DEFAULT_TOOLS_TERMINAL	"cmd.exe /Q /C %c"
+# define GEANY_DEFAULT_USE_NATIVE_DLGS	TRUE
+# define GEANY_DEFAULT_FONT_SYMBOL_LIST	"Sans 9"
+# define GEANY_DEFAULT_FONT_MSG_WINDOW	"Consolas 9"
+# define GEANY_DEFAULT_FONT_EDITOR		"Consolas 10"
 #elif defined(__APPLE__)
-#define GEANY_DEFAULT_TOOLS_TERMINAL	"open -a terminal %c"
+# define GEANY_DEFAULT_TOOLS_TERMINAL	"open -a terminal %c"
+# define GEANY_DEFAULT_USE_NATIVE_DLGS	TRUE
+# define GEANY_DEFAULT_FONT_SYMBOL_LIST	"Helvetica Medium 12"
+# define GEANY_DEFAULT_FONT_MSG_WINDOW	"Menlo Medium 12"
+# define GEANY_DEFAULT_FONT_EDITOR		"Menlo Medium 12"
 #else
-#define GEANY_DEFAULT_TOOLS_TERMINAL	"xterm -e \"/bin/sh %c\""
+# define GEANY_DEFAULT_TOOLS_TERMINAL	"xterm -e \"/bin/sh %c\""
+# define GEANY_DEFAULT_USE_NATIVE_DLGS	FALSE
+# define GEANY_DEFAULT_FONT_SYMBOL_LIST	"Sans 9"
+# define GEANY_DEFAULT_FONT_MSG_WINDOW	"Monospace 9"
+# define GEANY_DEFAULT_FONT_EDITOR		"Monospace 10"
 #endif
-#ifdef __APPLE__
-#define GEANY_DEFAULT_TOOLS_BROWSER		"open -a safari"
-#define GEANY_DEFAULT_FONT_SYMBOL_LIST	"Helvetica Medium 12"
-#define GEANY_DEFAULT_FONT_MSG_WINDOW	"Menlo Medium 12"
-#define GEANY_DEFAULT_FONT_EDITOR		"Menlo Medium 12"
-#else
 /* Browser chosen by GTK */
 #define GEANY_DEFAULT_TOOLS_BROWSER		""
-#define GEANY_DEFAULT_FONT_SYMBOL_LIST	"Sans 9"
-#define GEANY_DEFAULT_FONT_MSG_WINDOW	"Monospace 9"
-#define GEANY_DEFAULT_FONT_EDITOR		"Monospace 10"
-#endif
 #define GEANY_DEFAULT_TOOLS_PRINTCMD	"lpr"
 #define GEANY_DEFAULT_TOOLS_GREP		"grep"
 #define GEANY_DEFAULT_MRU_LENGTH		10
@@ -345,6 +347,8 @@ static void init_pref_groups(void)
 		"find_selection_type", GEANY_FIND_SEL_CURRENT_WORD);
 	stash_group_add_boolean(group, &search_prefs.replace_and_find_by_default,
 		"replace_and_find_by_default", TRUE);
+	stash_group_add_boolean(group, &search_prefs.skip_confirmation_for_replace_in_session,
+		"skip_confirmation_for_replace_in_session", FALSE);
 
 	group = stash_group_new(PACKAGE);
 	configuration_add_various_pref_group(group, "socket");
@@ -545,6 +549,7 @@ static void save_dialog_prefs(GKeyFile *config)
 	g_key_file_set_integer(config, PACKAGE, "tab_pos_editor", interface_prefs.tab_pos_editor);
 	g_key_file_set_integer(config, PACKAGE, "tab_pos_msgwin", interface_prefs.tab_pos_msgwin);
 	g_key_file_set_integer(config, PACKAGE, "tab_label_length", interface_prefs.tab_label_len);
+	g_key_file_set_boolean(config, PACKAGE, "use_native_dialogs", interface_prefs.use_native_windows_dialogs);
 
 	/* display */
 	g_key_file_set_boolean(config, PACKAGE, "show_indent_guide", editor_prefs.show_indent_guide);
@@ -617,7 +622,7 @@ static void save_dialog_prefs(GKeyFile *config)
 	build_save_menu(config, NULL, GEANY_BCS_PREF);
 
 	/* printing */
-	g_key_file_set_string(config, "printing", "print_cmd", printing_prefs.external_print_cmd ? printing_prefs.external_print_cmd : "");
+	g_key_file_set_string(config, "printing", "print_cmd_common_format", printing_prefs.external_print_cmd ? printing_prefs.external_print_cmd : "");
 	g_key_file_set_boolean(config, "printing", "use_gtk_printing", printing_prefs.use_gtk_printing);
 	g_key_file_set_boolean(config, "printing", "print_line_numbers", printing_prefs.print_line_numbers);
 	g_key_file_set_boolean(config, "printing", "print_page_numbers", printing_prefs.print_page_numbers);
@@ -790,7 +795,7 @@ static void load_recent_files(GKeyFile *config, GQueue *queue, const gchar *key)
 
 
 /*
- * Load session list from the given keyfile and return an array containg the file names
+ * Load session list from the given keyfile and return an array containing the file names
  * */
 GPtrArray *configuration_load_session_files(GKeyFile *config)
 {
@@ -895,6 +900,7 @@ static void load_dialog_prefs(GKeyFile *config)
 	interface_prefs.editor_font = utils_get_setting_string(config, PACKAGE, "editor_font", GEANY_DEFAULT_FONT_EDITOR);
 	interface_prefs.tagbar_font = utils_get_setting_string(config, PACKAGE, "tagbar_font", GEANY_DEFAULT_FONT_SYMBOL_LIST);
 	interface_prefs.msgwin_font = utils_get_setting_string(config, PACKAGE, "msgwin_font", GEANY_DEFAULT_FONT_MSG_WINDOW);
+	interface_prefs.use_native_windows_dialogs = utils_get_setting_boolean(config, PACKAGE, "use_native_dialogs", GEANY_DEFAULT_USE_NATIVE_DLGS);
 
 	/* display, editor */
 	editor_prefs.long_line_enabled = utils_get_setting_boolean(config, PACKAGE, "long_line_enabled", TRUE);
@@ -1066,9 +1072,9 @@ static void load_dialog_prefs(GKeyFile *config)
 	if (!EMPTY(tmp_string2))
 	{
 	#ifdef G_OS_WIN32
-		tmp_string = g_strconcat(GEANY_DEFAULT_TOOLS_PRINTCMD, " \"%f\"", NULL);
+		tmp_string = g_strconcat(GEANY_DEFAULT_TOOLS_PRINTCMD, " \"%d\\%f\"", NULL);
 	#else
-		tmp_string = g_strconcat(GEANY_DEFAULT_TOOLS_PRINTCMD, " '%f'", NULL);
+		tmp_string = g_strconcat(GEANY_DEFAULT_TOOLS_PRINTCMD, " '%d/%f'", NULL);
 	#endif
 	}
 	else
@@ -1076,7 +1082,17 @@ static void load_dialog_prefs(GKeyFile *config)
 		tmp_string = g_strdup("");
 	}
 
-	printing_prefs.external_print_cmd = utils_get_setting_string(config, "printing", "print_cmd", tmp_string);
+	/* new value is print_cmd_common_format with build-like format, old is print_cmd with only %f for full path */
+	if (g_key_file_has_key(config, "printing", "print_cmd_common_format", NULL))
+		printing_prefs.external_print_cmd = utils_get_setting_string(config, "printing", "print_cmd_common_format", tmp_string);
+	else /* load old value */
+	{
+		printing_prefs.external_print_cmd = g_key_file_get_string(config, "printing", "print_cmd", NULL);
+		if (! printing_prefs.external_print_cmd)
+			printing_prefs.external_print_cmd = g_strdup(tmp_string);
+		else /* transform the old value */
+			utils_str_replace_all(&printing_prefs.external_print_cmd, "%f", "%d" G_DIR_SEPARATOR_S "%f");
+	}
 	g_free(tmp_string);
 	g_free(tmp_string2);
 
@@ -1140,6 +1156,7 @@ static void load_ui_prefs(GKeyFile *config)
 	scribble_pos = utils_get_setting_integer(config, PACKAGE, "scribble_pos", -1);
 }
 
+
 static void load_ui_session(GKeyFile *config)
 {
 	gint *geo;
@@ -1165,7 +1182,7 @@ static void load_ui_session(GKeyFile *config)
 		ui_prefs.geometry[3] = MAX(-1, geo[3]);
 		ui_prefs.geometry[4] = geo[4] != 0;
 	}
-	hpan_position = utils_get_setting_integer(config, PACKAGE, "treeview_position", 156);
+	hpan_position = utils_get_setting_integer(config, PACKAGE, "treeview_position", 230);
 	vpan_position = utils_get_setting_integer(config, PACKAGE, "msgwindow_position", (geo) ?
 				(GEANY_MSGWIN_HEIGHT + geo[3] - 440) :
 				(GEANY_MSGWIN_HEIGHT + GEANY_WINDOW_DEFAULT_HEIGHT - 440));
@@ -1352,8 +1369,8 @@ static gboolean open_session_file(gchar **tmp, guint len)
 
 
 /* Open session files
- * Note: notebook page switch handler and adding to recent files list is always disabled
- * for all files opened within this function */
+ * Note: notebook page switch handler is delayed, and adding to recent files list is
+ * always disabled for all files opened within this function */
 void configuration_open_files(GPtrArray *session_files)
 {
 	gboolean failure = FALSE;
@@ -1379,7 +1396,7 @@ void configuration_open_files(GPtrArray *session_files)
 	if (failure)
 		ui_set_statusbar(TRUE, _("Failed to load one or more session files."));
 	else
-		document_show_tab_idle(session_notebook_page >= 0 ? document_get_from_page(session_notebook_page) : document_get_current());
+		document_show_tab(session_notebook_page >= 0 ? document_get_from_page(session_notebook_page) : document_get_current());
 
 	session_notebook_page = -1;
 	main_status.opening_session_files--;
