@@ -462,6 +462,18 @@ gboolean main_is_realized(void)
 }
 
 
+/**
+ *  Checks whether Geany is 'closing all' documents right now.
+ *
+ *  @return @c TRUE if the Geany is 'closing all' documents right now or @c FALSE otherwise.
+ **/
+GEANY_API_SYMBOL
+gboolean geany_is_closing_all_documents(void)
+{
+	return main_status.closing_all;
+}
+
+
 /**
  *  Initialises the gettext translation system.
  *  This is a convenience function to set up gettext for internationalisation support
@@ -587,7 +599,7 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 		gchar *build_date = utils_parse_and_format_build_date(__DATE__);
 
 		printf(PACKAGE " %s (", main_get_version_string());
-		/* note for translators: library versions are printed after this */
+		/* Translators: library versions are printed after this */
 		printf(_("built on %s with "), build_date);
 		printf(geany_lib_versions,
 			GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION,
@@ -700,7 +712,7 @@ static gint create_config_dir(void)
 				else
 				{
 					dialogs_show_msgbox(GTK_MESSAGE_WARNING,
-						/* for translators: the third %s in brackets is the error message which
+						/* Translators: the third %s in brackets is the error message which
 						 * describes why moving the dir didn't work */
 						_("Your old configuration directory \"%s\" could not be moved to \"%s\" (%s). "
 						  "Please move manually the directory to the new location."),
@@ -849,7 +861,7 @@ gboolean main_handle_filename(const gchar *locale_filename)
 
 		doc = document_find_by_filename(utf8_filename);
 		if (doc)
-			document_show_tab_idle(doc);
+			document_show_tab(doc);
 		else
 			doc = document_new_file(utf8_filename, NULL, NULL);
 		g_free(utf8_filename);
@@ -882,11 +894,11 @@ static void open_cl_files(gint argc, gchar **argv)
 #endif
 		if (filename && ! main_handle_filename(filename))
 		{
-			const gchar *msg = _("Could not find file '%s'.");
+			gchar *msg = g_strdup_printf(_("Could not find file '%s'."), filename);
 
-			g_printerr(msg, filename);	/* also print to the terminal */
-			g_printerr("\n");
-			ui_set_statusbar(TRUE, msg, filename);
+			g_printerr("%s\n", msg);	/* also print to the terminal */
+			ui_set_statusbar(TRUE, "%s", msg);
+			g_free(msg);
 		}
 		g_free(filename);
 	}
@@ -1018,10 +1030,6 @@ static const gchar *get_locale(void)
 GEANY_EXPORT_SYMBOL
 void main_init_headless(void)
 {
-#if ! GLIB_CHECK_VERSION(2, 36, 0)
-	g_type_init();
-#endif
-
 	app = g_new0(GeanyApp, 1);
 	memset(&main_status, 0, sizeof(GeanyStatus));
 	memset(&prefs, 0, sizeof(GeanyPrefs));
@@ -1033,6 +1041,8 @@ void main_init_headless(void)
 	memset(&template_prefs, 0, sizeof(GeanyTemplatePrefs));
 	memset(&ui_prefs, 0, sizeof(UIPrefs));
 	memset(&ui_widgets, 0, sizeof(UIWidgets));
+
+	encodings_init_headless();
 }
 
 
@@ -1054,17 +1064,22 @@ gint main_lib(gint argc, gchar **argv)
 #ifdef ENABLE_NLS
 	main_locale_init(utils_resource_dir(RESOURCE_DIR_LOCALE), GETTEXT_PACKAGE);
 #endif
+
+	/* Magic ID used on X11 and Wayland for bringing our existing window on top;
+	 * need to read it here, since GTK will clear this from the environment in
+	 * gtk_init () (called from parse_command_line_options () below). Need to
+	 * make a copy, since the value is not guaranteed to be valid after calling
+	 * unsetenv() (which is done by GTK). */
+#ifdef HAVE_SOCKET
+	gchar *desktop_startup_id = g_strdup(getenv("DESKTOP_STARTUP_ID"));
+	if (!desktop_startup_id)
+		desktop_startup_id = g_strdup(getenv("XDG_ACTIVATION_TOKEN"));
+#endif
+
 	/* initialize TM before parsing command-line - needed for tag file generation */
 	app->tm_workspace = tm_get_workspace();
 	parse_command_line_options(&argc, &argv);
 
-#if ! GLIB_CHECK_VERSION(2, 32, 0)
-	/* Initialize GLib's thread system in case any plugins want to use it or their
-	 * dependencies (e.g. WebKit, Soup, ...). Deprecated since GLIB 2.32. */
-	if (!g_thread_supported())
-		g_thread_init(NULL);
-#endif
-
 #ifdef G_OS_UNIX
 	g_unix_signal_add(SIGTERM, signal_cb, GINT_TO_POINTER(SIGTERM));
 
@@ -1083,7 +1098,7 @@ gint main_lib(gint argc, gchar **argv)
 #endif
 		socket_info.lock_socket = -1;
 		socket_info.lock_socket_tag = 0;
-		socket_info.lock_socket = socket_init(argc, argv, socket_port);
+		socket_info.lock_socket = socket_init(argc, argv, socket_port, desktop_startup_id);
 		/* Quit if filenames were sent to first instance or the list of open
 		 * documents has been printed */
 		if ((socket_info.lock_socket == -2 /* socket exists */ && argc > 1) ||
@@ -1095,6 +1110,7 @@ gint main_lib(gint argc, gchar **argv)
 			g_free(app->datadir);
 			g_free(app->docdir);
 			g_free(app);
+			g_free(desktop_startup_id);
 			return 0;
 		}
 		/* Start a new instance if no command line strings were passed,
@@ -1105,6 +1121,7 @@ gint main_lib(gint argc, gchar **argv)
 			cl_options.new_instance = TRUE;
 		}
 	}
+	g_free(desktop_startup_id);
 #endif
 
 #ifdef G_OS_WIN32
@@ -1174,9 +1191,11 @@ gint main_lib(gint argc, gchar **argv)
 	ui_set_statusbar(TRUE, _("This is Geany %s."), main_get_version_string());
 	if (config_dir_result != 0)
 	{
-		const gchar *message = _("Configuration directory could not be created (%s).");
-		ui_set_statusbar(TRUE, message, g_strerror(config_dir_result));
-		g_warning(message, g_strerror(config_dir_result));
+		gchar *message = g_strdup_printf(_("Configuration directory could not be created (%s)."),
+				g_strerror(config_dir_result));
+		ui_set_statusbar(TRUE, "%s", message);
+		g_warning("%s", message);
+		g_free(message);
 	}
 #ifdef HAVE_SOCKET
 	if (socket_info.lock_socket == -1)
@@ -1270,6 +1289,8 @@ static void queue_free(GQueue *queue)
 
 static gboolean do_main_quit(void)
 {
+	g_signal_emit_by_name(geany_object, "geany-before-quit");
+
 	configuration_save();
 
 	if (app->project != NULL)
@@ -1283,8 +1304,6 @@ static gboolean do_main_quit(void)
 
 	geany_debug("Quitting...");
 
-	main_status.quitting = TRUE;
-
 #ifdef HAVE_SOCKET
 	socket_finalize();
 #endif
@@ -1405,8 +1424,7 @@ gboolean main_quit(void)
 		if (do_main_quit())
 			return TRUE;
 	}
-	else
-	if (! prefs.confirm_exit ||
+	else if (! prefs.confirm_exit ||
 		dialogs_show_question_full(NULL, GTK_STOCK_QUIT, GTK_STOCK_CANCEL, NULL,
 			_("Do you really want to quit?")))
 	{
