@@ -84,12 +84,15 @@ static gboolean kb_grab_key_dialog_key_press_cb(GtkWidget *dialog, GdkEventKey *
 static void kb_change_iter_shortcut(KbData *kbdata, GtkTreeIter *iter, const gchar *new_text);
 static gboolean kb_find_duplicate(GtkTreeStore *store, GtkWidget *parent, GtkTreeIter *old_iter,
 		guint key, GdkModifierType mods, const gchar *shortcut);
+static void kb_filter_entry_changed_cb(GtkEntry *entry, gpointer user_data);
+static void kb_filter_entry_icon_release_cb(GtkEntry *entry, GtkEntryIconPosition icon_pos,
+		GdkEvent *event, gpointer user_data);
+static gboolean kb_tree_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
 static void on_toolbar_show_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_show_notebook_tabs_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_enable_plugins_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_use_folding_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_check_line_end_toggled(GtkToggleButton *togglebutton, gpointer user_data);
-static void on_open_encoding_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_sidebar_visible_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_prefs_print_radio_button_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_prefs_print_page_header_toggled(GtkToggleButton *togglebutton, gpointer user_data);
@@ -142,6 +145,7 @@ enum
 	KB_TREE_INDEX,
 	KB_TREE_EDITABLE,
 	KB_TREE_WEIGHT,
+	KB_TREE_VISIBLE,
 	KB_TREE_COLUMNS
 };
 
@@ -177,7 +181,8 @@ static void kb_tree_view_change_button_clicked_cb(GtkWidget *button, KbData *kbd
 			GtkWidget *accel_label;
 			gchar *str;
 
-			dialog = gtk_dialog_new_with_buttons(_("Grab Key"), GTK_WINDOW(ui_widgets.prefs_dialog),
+			dialog = gtk_dialog_new_with_buttons(_("Assign Keybinding"),
+					GTK_WINDOW(ui_widgets.prefs_dialog),
 					GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
@@ -198,9 +203,11 @@ static void kb_tree_view_change_button_clicked_cb(GtkWidget *button, KbData *kbd
 			gtk_widget_show_all(dialog);
 			if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
 			{
+				GtkTreeIter child_iter;
 				const gchar *new_text = gtk_label_get_text(GTK_LABEL(accel_label));
 
-				kb_change_iter_shortcut(kbdata, &iter, new_text);
+				gtk_tree_model_filter_convert_iter_to_child_iter(GTK_TREE_MODEL_FILTER(model), &child_iter, &iter);
+				kb_change_iter_shortcut(kbdata, &child_iter, new_text);
 			}
 			gtk_widget_destroy(dialog);
 
@@ -262,16 +269,21 @@ static gboolean kb_tree_view_button_press_event_cb(GtkWidget *widget, GdkEventBu
 }
 
 
-static void kb_init_tree(KbData *kbdata)
+static void kb_init_tree(KbData *kbdata, GtkWidget *kb_filter_entry)
 {
 	GtkCellRenderer *renderer;
+	GtkTreeModel *filter_model;
 	GtkTreeViewColumn *column;
 
 	kbdata->tree = GTK_TREE_VIEW(ui_lookup_widget(ui_widgets.prefs_dialog, "treeview7"));
 
 	kbdata->store = gtk_tree_store_new(KB_TREE_COLUMNS,
-		G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_BOOLEAN, G_TYPE_INT);
-	gtk_tree_view_set_model(GTK_TREE_VIEW(kbdata->tree), GTK_TREE_MODEL(kbdata->store));
+		G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_BOOLEAN, G_TYPE_INT, G_TYPE_BOOLEAN);
+	filter_model = gtk_tree_model_filter_new(GTK_TREE_MODEL(kbdata->store), NULL);
+	gtk_tree_model_filter_set_visible_column(GTK_TREE_MODEL_FILTER(filter_model), KB_TREE_VISIBLE);
+	/* set model to tree view */
+	gtk_tree_view_set_model(GTK_TREE_VIEW(kbdata->tree), filter_model);
+	g_object_unref(filter_model);
 	g_object_unref(kbdata->store);
 
 	renderer = gtk_cell_renderer_text_new();
@@ -293,9 +305,13 @@ static void kb_init_tree(KbData *kbdata)
 
 	g_signal_connect(renderer, "edited", G_CALLBACK(kb_cell_edited_cb), kbdata);
 	g_signal_connect(kbdata->tree, "button-press-event", G_CALLBACK(kb_tree_view_button_press_event_cb), kbdata);
+	g_signal_connect(kbdata->tree, "key-press-event", G_CALLBACK(kb_tree_key_press_cb), kb_filter_entry);
 	g_signal_connect(kbdata->tree, "popup-menu", G_CALLBACK(kb_popup_menu_cb), kbdata);
 	g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "button2"), "clicked",
 				G_CALLBACK(kb_tree_view_change_button_clicked_cb), kbdata);
+	g_signal_connect(kb_filter_entry, "changed", G_CALLBACK(kb_filter_entry_changed_cb), NULL);
+	g_signal_connect(kb_filter_entry, "icon-release", G_CALLBACK(kb_filter_entry_icon_release_cb), NULL);
+
 }
 
 
@@ -349,7 +365,7 @@ void prefs_kb_search_name(const gchar *search)
 }
 
 
-static void kb_init(KbData *kbdata)
+static void kb_init(KbData *kbdata, GtkWidget *kb_filter_entry)
 {
 	GtkTreeIter parent, iter;
 	gsize g, i;
@@ -358,20 +374,21 @@ static void kb_init(KbData *kbdata)
 	GeanyKeyBinding *kb;
 
 	if (kbdata->store == NULL)
-		kb_init_tree(kbdata);
+		kb_init_tree(kbdata, kb_filter_entry);
 
+	gtk_entry_set_text(GTK_ENTRY(kb_filter_entry), "");
 	foreach_ptr_array(group, g, keybinding_groups)
 	{
 		gtk_tree_store_append(kbdata->store, &parent, NULL);
 		gtk_tree_store_set(kbdata->store, &parent, KB_TREE_ACTION, group->label,
-			KB_TREE_INDEX, g, -1);
+			KB_TREE_INDEX, g, KB_TREE_VISIBLE, TRUE, -1);
 
 		foreach_ptr_array(kb, i, group->key_items)
 		{
 			label = keybindings_get_label(kb);
 			gtk_tree_store_append(kbdata->store, &iter, &parent);
 			gtk_tree_store_set(kbdata->store, &iter, KB_TREE_ACTION, label,
-				KB_TREE_EDITABLE, TRUE, KB_TREE_INDEX, kb->id, -1);
+				KB_TREE_EDITABLE, TRUE, KB_TREE_INDEX, kb->id, KB_TREE_VISIBLE, TRUE, -1);
 			kb_set_shortcut(kbdata->store, &iter, kb->key, kb->mods);
 			g_free(label);
 		}
@@ -425,6 +442,10 @@ static void prefs_init_dialog(void)
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "entry_contextaction");
 	gtk_entry_set_text(GTK_ENTRY(widget), tool_prefs.context_action_cmd);
 
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_native_dialogs");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
+		interface_prefs.use_native_windows_dialogs);
+
 	project_setup_prefs();	/* project files path */
 
 
@@ -533,18 +554,11 @@ static void prefs_init_dialog(void)
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_new_encoding");
 	ui_encodings_combo_box_set_active_encoding(GTK_COMBO_BOX(widget), file_prefs.default_new_encoding);
 
-	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_open_encoding");
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
-			(file_prefs.default_open_encoding >= 0) ? TRUE : FALSE);
-	on_open_encoding_toggled(GTK_TOGGLE_BUTTON(widget), NULL);
-
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_open_encoding");
 	if (file_prefs.default_open_encoding >= 0)
-	{
 		ui_encodings_combo_box_set_active_encoding(GTK_COMBO_BOX(widget), file_prefs.default_open_encoding);
-	}
 	else
-		ui_encodings_combo_box_set_active_encoding(GTK_COMBO_BOX(widget), GEANY_ENCODING_UTF_8);
+		ui_encodings_combo_box_set_active_encoding(GTK_COMBO_BOX(widget), GEANY_ENCODINGS_MAX);
 
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_eol");
 	if (file_prefs.default_eol_character >= 0 && file_prefs.default_eol_character < 3)
@@ -578,6 +592,7 @@ static void prefs_init_dialog(void)
 
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_line_end");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), editor_prefs.show_line_endings);
+	on_check_line_end_toggled(GTK_TOGGLE_BUTTON(widget), NULL);
 
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_line_endings_only_when_differ");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
@@ -695,7 +710,8 @@ static void prefs_init_dialog(void)
 
 
 	/* Keybindings */
-	kb_init(&global_kb_data);
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "entry_keybinding_filter");
+	kb_init(&global_kb_data, widget);
 
 	/* Printing */
 	{
@@ -908,6 +924,10 @@ on_prefs_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		g_free(tool_prefs.context_action_cmd);
 		tool_prefs.context_action_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
 
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_native_dialogs");
+		interface_prefs.use_native_windows_dialogs =
+			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 		project_apply_prefs();	/* project file path */
 
 
@@ -1006,13 +1026,9 @@ on_prefs_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_new_encoding");
 		file_prefs.default_new_encoding = ui_encodings_combo_box_get_active_encoding(GTK_COMBO_BOX(widget));
 
-		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_open_encoding");
-		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-		{
-			widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_open_encoding");
-			file_prefs.default_open_encoding = ui_encodings_combo_box_get_active_encoding(GTK_COMBO_BOX(widget));
-		}
-		else
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_open_encoding");
+		file_prefs.default_open_encoding = ui_encodings_combo_box_get_active_encoding(GTK_COMBO_BOX(widget));
+		if (file_prefs.default_open_encoding >= GEANY_ENCODINGS_MAX)
 			file_prefs.default_open_encoding = -1;
 
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "combo_eol");
@@ -1293,7 +1309,7 @@ on_prefs_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		/* various preferences */
 		ui_save_buttons_toggle((doc != NULL) ? doc->changed : FALSE);
 		msgwin_show_hide_tabs();
-		ui_update_statusbar(doc, -1);
+		ui_update_statusbar(doc);
 		ui_set_window_title(doc);
 
 		/* store all settings */
@@ -1390,17 +1406,109 @@ static void kb_cell_edited_cb(GtkCellRendererText *cellrenderertext,
 {
 	if (path != NULL && new_text != NULL)
 	{
+		GtkTreeModel *filter_model = gtk_tree_view_get_model(kbdata->tree);
 		GtkTreeIter iter;
+		GtkTreeIter child_iter;
 
-		gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(kbdata->store), &iter, path);
-		if (gtk_tree_model_iter_has_child(GTK_TREE_MODEL(kbdata->store), &iter))
+		gtk_tree_model_get_iter_from_string(filter_model, &iter, path);
+		if (gtk_tree_model_iter_has_child(filter_model, &iter))
 			return;	/* ignore group items */
 
-		kb_change_iter_shortcut(kbdata, &iter, new_text);
+		gtk_tree_model_filter_convert_iter_to_child_iter(GTK_TREE_MODEL_FILTER(filter_model),
+				&child_iter, &iter);
+		kb_change_iter_shortcut(kbdata, &child_iter, new_text);
 	}
 }
 
 
+static void kb_tree_update_visibility(GtkTreeStore *store, const gchar *entry_text)
+{
+	GtkTreeModel *model = GTK_TREE_MODEL(store);
+	GtkTreeIter child, parent;
+
+	/* get first parent */
+	if (! gtk_tree_model_iter_children(model, &parent, NULL))
+		return;
+
+	/* foreach parent */
+	while (TRUE)
+	{
+		gboolean visible_parent = FALSE;
+		gboolean visible_children = FALSE;
+		gchar *group_name;
+
+		/* check if group name matches and then display *all* its children, otherwise
+		 * check each child if the action name matches */
+		gtk_tree_model_get(model, &parent, KB_TREE_ACTION, &group_name, -1);
+		if (group_name && ! EMPTY(entry_text))
+			visible_parent = utils_utf8_substring_match(entry_text, group_name);
+		else
+			visible_parent = FALSE;
+		g_free(group_name);
+
+		/* get first child */
+		if (! gtk_tree_model_iter_children(model, &child, &parent))
+			return;
+
+		/* foreach child */
+		while (TRUE)
+		{
+			gboolean visible;
+			gchar *action_name;
+
+			gtk_tree_model_get(model, &child, KB_TREE_ACTION, &action_name, -1);
+			if (!action_name || EMPTY(entry_text))
+				visible = TRUE;
+			else
+				visible = utils_utf8_substring_match(entry_text, action_name);
+			g_free(action_name);
+
+			if (visible)
+				visible_children = TRUE;
+
+			gtk_tree_store_set(store, &child, KB_TREE_VISIBLE, visible || visible_parent, -1);
+
+			if (! gtk_tree_model_iter_next(model, &child))
+				break;
+		}
+
+		gtk_tree_store_set(store, &parent, KB_TREE_VISIBLE, visible_children || visible_parent, -1);
+
+		if (! gtk_tree_model_iter_next(model, &parent))
+			return;
+	}
+}
+
+
+static void kb_filter_entry_changed_cb(GtkEntry *entry, gpointer user_data)
+{
+	const gchar *entry_text = gtk_entry_get_text(entry);
+
+	kb_tree_update_visibility(global_kb_data.store, entry_text);
+	gtk_tree_view_expand_all(global_kb_data.tree);
+}
+
+
+static void kb_filter_entry_icon_release_cb(GtkEntry *entry, GtkEntryIconPosition icon_pos,
+		GdkEvent *event, gpointer user_data)
+{
+	if (event->button.button == 1 && icon_pos == GTK_ENTRY_ICON_PRIMARY)
+		kb_filter_entry_changed_cb(entry, user_data);
+}
+
+
+static gboolean kb_tree_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
+{
+	if (event->keyval == GDK_KEY_f && (event->state & GEANY_PRIMARY_MOD_MASK))
+	{
+		GtkWidget *kb_filter_entry = user_data;
+		gtk_widget_grab_focus(kb_filter_entry);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+
 static gboolean kb_grab_key_dialog_key_press_cb(GtkWidget *dialog, GdkEventKey *event, GtkLabel *label)
 {
 	gchar *str;
@@ -1547,15 +1655,6 @@ static void on_enable_plugins_toggled(GtkToggleButton *togglebutton, gpointer us
 }
 
 
-static void on_open_encoding_toggled(GtkToggleButton *togglebutton, gpointer user_data)
-{
-	gboolean sens = gtk_toggle_button_get_active(togglebutton);
-
-	gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "eventbox3"), sens);
-	gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "label_open_encoding"), sens);
-}
-
-
 static void on_sidebar_visible_toggled(GtkToggleButton *togglebutton, gpointer user_data)
 {
 	gboolean sens = gtk_toggle_button_get_active(togglebutton);
@@ -1664,23 +1763,24 @@ void prefs_show_dialog(void)
 		{
 			struct {
 				const gchar *combo, *renderer;
+				gboolean has_detect;
 			} names[] = {
-				{ "combo_new_encoding", "combo_new_encoding_renderer" },
-				{ "combo_open_encoding", "combo_open_encoding_renderer" }
+				{ "combo_new_encoding", "combo_new_encoding_renderer", FALSE },
+				{ "combo_open_encoding", "combo_open_encoding_renderer", TRUE }
 			};
 			guint i;
-			GtkTreeStore *encoding_list = encodings_encoding_store_new(FALSE);
 
 			for (i = 0; i < G_N_ELEMENTS(names); i++)
 			{
+				GtkTreeStore *encoding_list = encodings_encoding_store_new(names[i].has_detect);
 				GtkWidget *combo = ui_lookup_widget(ui_widgets.prefs_dialog, names[i].combo);
 
 				gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(combo),
 						ui_builder_get_object(names[i].renderer),
 						encodings_encoding_store_cell_data_func, NULL, NULL);
 				gtk_combo_box_set_model(GTK_COMBO_BOX(combo), GTK_TREE_MODEL(encoding_list));
+				g_object_unref(encoding_list);
 			}
-			g_object_unref(encoding_list);
 		}
 
 		/* init the eol character combo box */
@@ -1713,6 +1813,12 @@ void prefs_show_dialog(void)
 		gtk_misc_set_padding(GTK_MISC(label), 6, 0);
 		gtk_box_pack_start(GTK_BOX(ui_lookup_widget(ui_widgets.prefs_dialog,
 			"label_project_indent_warning")), label, FALSE, TRUE, 5);
+		label = geany_wrap_label_new(_("Note: To apply these settings to all currently open documents, use <i>Project->Apply Default Indentation</i>."));
+		gtk_widget_show(label);
+		gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+		gtk_misc_set_padding(GTK_MISC(label), 6, 0);
+		gtk_box_pack_start(GTK_BOX(ui_lookup_widget(ui_widgets.prefs_dialog,
+			"label_indent_warning")), label, FALSE, TRUE, 5);
 
 		/* add the clear icon to GtkEntry widgets in the dialog */
 		{
@@ -1725,6 +1831,7 @@ void prefs_show_dialog(void)
 				"entry_com_term",
 				"entry_browser",
 				"entry_grep",
+				"entry_keybinding_filter",
 				"entry_contextaction",
 				"entry_template_developer",
 				"entry_template_initial",
@@ -1751,9 +1858,6 @@ void prefs_show_dialog(void)
 		vte_append_preferences_tab();
 #endif
 
-#ifndef G_OS_WIN32
-		gtk_widget_hide(ui_lookup_widget(ui_widgets.prefs_dialog, "check_native_windows_dialogs"));
-#endif
 		ui_setup_open_button_callback(ui_lookup_widget(ui_widgets.prefs_dialog, "startup_path_button"), NULL,
 			GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, "startup_path_entry")));
 		ui_setup_open_button_callback(ui_lookup_widget(ui_widgets.prefs_dialog, "extra_plugin_path_button"), NULL,
@@ -1781,6 +1885,13 @@ void prefs_show_dialog(void)
 			NULL,
 			GTK_FILE_CHOOSER_ACTION_OPEN,
 			GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_browser")));
+#ifdef G_OS_WIN32
+		/* Browser isn't configurable on Windows, see utils_open_browser() */
+		gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_browser"), FALSE);
+		gtk_widget_set_tooltip_text(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_browser"),
+			_("Not configurable on Windows (the system default browser is used)."));
+		gtk_widget_set_sensitive(ui_lookup_widget(ui_widgets.prefs_dialog, "button_browser"), FALSE);
+#endif
 		ui_setup_open_button_callback(ui_lookup_widget(ui_widgets.prefs_dialog, "button_grep"),
 			NULL,
 			GTK_FILE_CHOOSER_ACTION_OPEN,
@@ -1813,8 +1924,6 @@ void prefs_show_dialog(void)
 				"toggled", G_CALLBACK(on_use_folding_toggled), NULL);
 		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "check_line_end"),
 				"toggled", G_CALLBACK(on_check_line_end_toggled), NULL);
-		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "check_open_encoding"),
-				"toggled", G_CALLBACK(on_open_encoding_toggled), NULL);
 		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "check_sidebar_visible"),
 				"toggled", G_CALLBACK(on_sidebar_visible_toggled), NULL);
 
