@@ -70,10 +70,6 @@
 # include "geany.h"
 #endif
 
-#if ! GLIB_CHECK_VERSION(2, 31, 20) && ! defined(G_SPAWN_ERROR_TOO_BIG)
-# define G_SPAWN_ERROR_TOO_BIG G_SPAWN_ERROR_2BIG
-#endif
-
 #ifdef G_OS_WIN32
 /* Each 4KB under Windows seem to come in 2 portions, so 2K + 2K is more
    balanced than 4095 + 1. May be different on the latest Windows/glib? */
@@ -128,7 +124,7 @@ static gchar *spawn_get_program_name(const gchar *command_line, GError **error)
 	if (!*command_line)
 	{
 		g_set_error_literal(error, G_SHELL_ERROR, G_SHELL_ERROR_EMPTY_STRING,
-			/* TL note: from glib */
+			/* Translators: this is from glib */
 			_("Text was empty (or contained only whitespace)"));
 		return FALSE;
 	}
@@ -218,7 +214,7 @@ static gchar *spawn_get_program_name(const gchar *command_line, GError **error)
  *
  *  Unix:
  *     - the standard shell quoting and escaping rules are used, see @c g_shell_parse_argv()
- *     - as a consequence, an unqouted # at the start of an argument comments to the end of line
+ *     - as a consequence, an unquoted # at the start of an argument comments to the end of line
  *
  *  Windows:
  *     - leading carriage returns are skipped too
@@ -547,7 +543,7 @@ static gboolean spawn_async_with_pipes(const gchar *working_directory, const gch
 		}
 		else
 		{
-			/* quote the first token, to avoid Windows attemps to run two or more
+			/* quote the first token, to avoid Windows attempts to run two or more
 			   unquoted tokens as a program until an existing file name is found */
 			g_string_printf(command, "\"%s\"", program);
 		}
@@ -902,7 +898,7 @@ static gboolean spawn_timeout_read_cb(gpointer data)
 {
 	SpawnChannelData *sc = (SpawnChannelData *) data;
 
-	/* The solution for continuous empty G_IO_IN-s is to generate them outselves. :) */
+	/* The solution for continuous empty G_IO_IN-s is to generate them ourselves. :) */
 	return spawn_read_cb(sc->channel, G_IO_IN, data);
 }
 
@@ -1185,7 +1181,7 @@ gboolean spawn_with_callbacks(const gchar *working_directory, const gchar *comma
 		{
 			SpawnChannelData *sc = &sw->sc[i];
 			GIOCondition condition;
-			GSourceFunc callback;
+			GIOFunc callback;
 
 			if (pipe[i] == -1)
 				continue;
@@ -1205,7 +1201,7 @@ gboolean spawn_with_callbacks(const gchar *working_directory, const gchar *comma
 			{
 				sc->cb.write = stdin_cb;
 				condition = G_IO_OUT | SPAWN_IO_FAILURE;
-				callback = (GSourceFunc) spawn_write_cb;
+				callback = spawn_write_cb;
 			}
 			else
 			{
@@ -1213,7 +1209,7 @@ gboolean spawn_with_callbacks(const gchar *working_directory, const gchar *comma
 					((SPAWN_STDOUT_UNBUFFERED >> 1) << i));
 
 				condition = G_IO_IN | G_IO_PRI | SPAWN_IO_FAILURE;
-				callback = (GSourceFunc) spawn_read_cb;
+				callback = spawn_read_cb;
 
 				if (i == 1)
 				{
@@ -1245,7 +1241,7 @@ gboolean spawn_with_callbacks(const gchar *working_directory, const gchar *comma
 			else if (i)  /* to avoid new string on each call */
 				sc->buffer = g_string_sized_new(sc->max_length);
 
-			g_source_set_callback(source, callback, sc, spawn_destroy_cb);
+			g_source_set_callback(source, (GSourceFunc) (void(*)(void)) callback, sc, spawn_destroy_cb);
 			g_source_attach(source, sw->main_context);
 			g_source_unref(source);
 		}
@@ -1253,7 +1249,7 @@ gboolean spawn_with_callbacks(const gchar *working_directory, const gchar *comma
 		sw->exit_cb = exit_cb;
 		sw->exit_data = exit_data;
 		source = g_child_watch_source_new(pid);
-		g_source_set_callback(source, (GSourceFunc) spawn_watch_cb, sw, NULL);
+		g_source_set_callback(source, (GSourceFunc) (void(*)(void)) (GChildWatchFunc) spawn_watch_cb, sw, NULL);
 		g_source_attach(source, sw->main_context);
 		g_source_unref(source);
 
