@@ -38,6 +38,7 @@
 
 #include "app.h"
 #include "callbacks.h" /* FIXME: for ignore_callback */
+#include "dialogs.h"
 #include "documentprivate.h"
 #include "editor.h"
 #include "encodings.h"
@@ -46,6 +47,7 @@
 #include "highlighting.h"
 #include "main.h"
 #include "navqueue.h"
+#include "pluginextension.h"
 #include "sciwrappers.h"
 #include "sidebar.h"
 #include "support.h"
@@ -258,6 +260,22 @@ const gchar *symbols_get_context_separator(gint ft_id)
 }
 
 
+/** Gets the icon data corresponding to the provided TMIcon.
+ * @param icon TMIcon.
+ * @returns The GdkPixbuf corresponding to the provided TMIcon.
+ *
+ * @since 2.1
+ */
+GEANY_API_SYMBOL
+GdkPixbuf *symbols_get_icon_pixbuf(TMIcon icon)
+{
+	if (icon < TM_N_ICONS)
+		return symbols_icons[icon].pixbuf;
+
+	return NULL;
+}
+
+
 /* sort by name, then line */
 static gint compare_symbol(const TMTag *tag_a, const TMTag *tag_b)
 {
@@ -305,6 +323,56 @@ static gint compare_symbol_lines(gconstpointer a, gconstpointer b)
 }
 
 
+static gboolean is_symbol_within_parent(const TMTag *tag, const TMTag *parent)
+{
+	const gchar *scope_sep = tm_parser_scope_separator(tag->lang);
+	gsize scope_len = 0;
+
+	if (EMPTY(tag->scope))
+		return FALSE;
+
+	if (!EMPTY(parent->scope))
+	{
+		if (!g_str_has_prefix(tag->scope, parent->scope))
+			return FALSE;
+		scope_len = strlen(parent->scope);
+
+		if (!g_str_has_prefix(tag->scope + scope_len, scope_sep))
+			return FALSE;
+		scope_len += strlen(scope_sep);
+	}
+
+	if (!g_str_has_prefix(tag->scope + scope_len, parent->name))
+		return FALSE;
+	scope_len += strlen(parent->name);
+
+	if (tag->scope[scope_len] == '\0')
+		return TRUE;
+
+	if (!g_str_has_prefix(tag->scope + scope_len, scope_sep))
+		return FALSE;
+
+	return TRUE;
+}
+
+
+/* for symbol tree construction make sure that symbol parents appear before
+ * their children in the list */
+static gint compare_symbol_parent_first(gconstpointer a, gconstpointer b)
+{
+	const TMTag *tag_a = TM_TAG(a);
+	const TMTag *tag_b = TM_TAG(b);
+
+	if (is_symbol_within_parent(tag_a, tag_b))
+		return 1;
+
+	if (is_symbol_within_parent(tag_b, tag_a))
+		return -1;
+
+	return compare_symbol_lines(a, b);
+}
+
+
 static GList *get_tag_list(GeanyDocument *doc, TMTagType tag_types)
 {
 	GList *tag_names = NULL;
@@ -356,7 +424,7 @@ static GList *get_tag_list(GeanyDocument *doc, TMTagType tag_types)
 			g_free(full_tagname);
 		}
 	}
-	tag_names = g_list_sort(tag_names, compare_symbol_lines);
+	tag_names = g_list_sort(tag_names, compare_symbol_parent_first);
 
 	g_strfreev(tf_strv);
 
@@ -436,7 +504,7 @@ static void tag_list_add_groups(GtkTreeStore *tree_store, TMParserType lang)
 		GdkPixbuf *icon = NULL;
 
 		if (icon_id < TM_N_ICONS)
-			icon = symbols_icons[icon_id].pixbuf;
+			icon = symbols_get_icon_pixbuf(icon_id);
 
 		g_assert(title != NULL);
 		g_ptr_array_add(top_level_iter_names, (gchar *)title);
@@ -1263,23 +1331,29 @@ int symbols_generate_global_tags(int argc, char **argv, gboolean want_preprocess
 
 void symbols_show_load_tags_dialog(void)
 {
-	GtkWidget *dialog;
+	GtkFileChooser *dialog;
 	GtkFileFilter *filter;
 
-	dialog = gtk_file_chooser_dialog_new(_("Load Tags File"), GTK_WINDOW(main_widgets.window),
-		GTK_FILE_CHOOSER_ACTION_OPEN,
-		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_OPEN, GTK_RESPONSE_OK,
-		NULL);
-	gtk_widget_set_name(dialog, "GeanyDialog");
+	if (interface_prefs.use_native_windows_dialogs)
+		dialog = GTK_FILE_CHOOSER(gtk_file_chooser_native_new(_("Load Tags File"),
+			GTK_WINDOW(main_widgets.window), GTK_FILE_CHOOSER_ACTION_OPEN, NULL, NULL));
+	else
+	{
+		dialog = GTK_FILE_CHOOSER(gtk_file_chooser_dialog_new(_("Load Tags File"), GTK_WINDOW(main_widgets.window),
+			GTK_FILE_CHOOSER_ACTION_OPEN,
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+			NULL));
+		gtk_widget_set_name(GTK_WIDGET(dialog), "GeanyDialog");
+	}
 	filter = gtk_file_filter_new();
 	gtk_file_filter_set_name(filter, _("Geany tags file (*.*.tags)"));
 	gtk_file_filter_add_pattern(filter, "*.*.tags");
-	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
+	gtk_file_chooser_add_filter(dialog, filter);
 
-	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
+	if (dialogs_file_chooser_run(dialog) == GTK_RESPONSE_ACCEPT)
 	{
-		GSList *flist = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dialog));
+		GSList *flist = gtk_file_chooser_get_filenames(dialog);
 		GSList *item;
 
 		for (item = flist; item != NULL; item = g_slist_next(item))
@@ -1292,7 +1366,7 @@ void symbols_show_load_tags_dialog(void)
 			ft = detect_global_tags_filetype(utf8_fname);
 
 			if (ft != NULL && symbols_load_global_tags(fname, ft))
-				/* For translators: the first wildcard is the filetype, the second the filename */
+				/* Translators: the first wildcard is the filetype, the second the filename */
 				ui_set_statusbar(TRUE, _("Loaded %s tags file '%s'."),
 					filetypes_get_display_name(ft), utf8_fname);
 			else
@@ -1303,7 +1377,7 @@ void symbols_show_load_tags_dialog(void)
 		}
 		g_slist_free(flist);
 	}
-	gtk_widget_destroy(dialog);
+	dialogs_file_chooser_destroy(dialog);
 }
 
 
@@ -1469,7 +1543,7 @@ static void show_goto_popup(GeanyDocument *doc, GPtrArray *tags, gboolean have_b
 
 		tooltip = g_markup_printf_escaped("%s:%lu\n<small><tt>%s</tt></small>", fname, tmtag->line, sym);
 
-		image = gtk_image_new_from_pixbuf(symbols_icons[get_tag_class(tmtag)].pixbuf);
+		image = gtk_image_new_from_pixbuf(symbols_get_icon_pixbuf(get_tag_class(tmtag)));
 		box = g_object_new(GTK_TYPE_BOX, "orientation", GTK_ORIENTATION_HORIZONTAL, "spacing", 12, NULL);
 		label = g_object_new(GTK_TYPE_LABEL, "label", text, "use-markup", TRUE, "xalign", 0.0, NULL);
 		gtk_size_group_add_widget(group, label);
@@ -1481,7 +1555,7 @@ static void show_goto_popup(GeanyDocument *doc, GPtrArray *tags, gboolean have_b
 		item = g_object_new(GTK_TYPE_IMAGE_MENU_ITEM, "image", image, "child", box, "always-show-image", TRUE,
 					"tooltip-markup", tooltip, NULL);
 		g_signal_connect_data(item, "activate", G_CALLBACK(on_goto_popup_item_activate),
-		                      tm_tag_ref(tmtag), (GClosureNotify) tm_tag_unref, 0);
+		                      tm_tag_ref(tmtag), CLOSURE_NOTIFY(tm_tag_unref), 0);
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 
 		if (! first)
@@ -1574,7 +1648,8 @@ static GPtrArray *filter_tags(GPtrArray *tags, TMTag *current_tag, gboolean defi
 	GPtrArray *filtered_tags = g_ptr_array_new();
 	guint i;
 
-	symbols_get_current_function(doc, &current_scope);
+	if (symbols_get_current_function(doc, &current_scope) == -1)
+		current_scope = NULL;  /* current_scope == "unknown" when not found */
 
 	foreach_ptr_array(tmtag, i, tags)
 	{
@@ -1686,19 +1761,41 @@ static gboolean goto_tag(const gchar *name, gboolean definition)
 }
 
 
-gboolean symbols_goto_tag(const gchar *name, gboolean definition)
+gboolean symbols_goto_tag(GeanyDocument *doc, gint pos, gboolean definition)
 {
-	if (goto_tag(name, definition))
-		return TRUE;
+	gchar *name;
+	gboolean success;
 
-	/* if we are here, there was no match and we are beeping ;-) */
-	utils_beep();
+	if (plugin_extension_goto_provided(doc, NULL))
+		return plugin_extension_goto_perform(doc, pos, definition);
 
-	if (!definition)
-		ui_set_statusbar(FALSE, _("Forward declaration \"%s\" not found."), name);
+	/* get the current selection if any, or current word */
+	if (sci_has_selection(doc->editor->sci))
+		name = sci_get_selection_contents(doc->editor->sci);
 	else
-		ui_set_statusbar(FALSE, _("Definition of \"%s\" not found."), name);
-	return FALSE;
+	{
+		editor_find_current_word(doc->editor, pos,
+			editor_info.current_word, GEANY_MAX_WORD_LENGTH, NULL);
+		name = *editor_info.current_word ? g_strdup(editor_info.current_word) : NULL;
+	}
+
+	if (!name)
+		return FALSE;
+
+	if (! (success = goto_tag(name, definition)))
+	{
+		/* if we are here, there was no match and we are beeping ;-) */
+		utils_beep();
+
+		if (!definition)
+			ui_set_statusbar(FALSE, _("Forward declaration \"%s\" not found."), name);
+		else
+			ui_set_statusbar(FALSE, _("Definition of \"%s\" not found."), name);
+	}
+
+	g_free(name);
+
+	return success;
 }
 
 
