@@ -241,6 +241,33 @@ void tm_ctags_clear_ignore_symbols(void)
 }
 
 
+static gboolean replace_str(gchar **where, const gchar *what, guint what_len,
+	const gchar *replacement, guint replacement_len)
+{
+	if (where && *where)
+	{
+		gchar *pos = strstr(*where, what);
+
+		if (pos)
+		{
+			gsize where_len = strlen(*where);
+			gchar *str = g_malloc(where_len + replacement_len - what_len + 1);
+			gsize prefix_len = (gsize) (pos - *where);
+
+			strncpy(str, *where, prefix_len);
+			strcpy(str + prefix_len, replacement);
+			strcpy(str + prefix_len + replacement_len, pos + what_len);
+			g_free(*where);
+			*where = str;
+
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+
 /* call after all tags have been collected so we don't have to handle reparses
  * with the counter (which gets complicated when also subparsers are involved) */
 static void rename_anon_tags(TMSourceFile *source_file)
@@ -320,7 +347,7 @@ static void rename_anon_tags(TMSourceFile *source_file)
 				if (!anon_counter_table)
 					anon_counter_table = g_new0(gint, 256);
 
-				anon_counter = ++anon_counter_table[kind];
+				anon_counter = ++anon_counter_table[(guchar) kind];
 
 				sprintf(buf, "anon_%s_%u", kind_name, anon_counter);
 				tag->name = g_strdup(buf);
@@ -336,7 +363,6 @@ static void rename_anon_tags(TMSourceFile *source_file)
 			{
 				TMTag *nested_tag = TM_TAG(source_file->tags_array->pdata[j]);
 				guint nested_scope_len = nested_tag->scope ? strlen(nested_tag->scope) : 0;
-				gchar *pos;
 
 				/* Tags can be interleaved with scopeless macros - skip those */
 				if (is_c && nested_tag->type & (tm_tag_macro_t | tm_tag_macro_with_arg_t))
@@ -361,22 +387,14 @@ static void rename_anon_tags(TMSourceFile *source_file)
 				if (nested_scope_len <= scope_len)
 					break;
 
-				pos = strstr(nested_tag->scope, orig_name);
 				/* We found the parent name in the nested tag scope - replace it
 				 * with the new name. Note: anonymous tag names generated by
 				 * ctags are unique enough that we don't have to check for
 				 * scope separators here. */
-				if (pos)
-				{
-					gchar *str = g_malloc(nested_scope_len + 50);
-					guint prefix_len = pos - nested_tag->scope;
+				replace_str(&nested_tag->scope, orig_name, orig_name_len, new_name, new_name_len);
 
-					strncpy(str, nested_tag->scope, prefix_len);
-					strcpy(str + prefix_len, new_name);
-					strcpy(str + prefix_len + new_name_len, pos + orig_name_len);
-					g_free(nested_tag->scope);
-					nested_tag->scope = str;
-				}
+				/* Do the same for var_type as well */
+				replace_str(&nested_tag->var_type, orig_name, orig_name_len, new_name, new_name_len);
 			}
 
 			/* We are out of the nesting - the next tags could be variables
@@ -385,22 +403,13 @@ static void rename_anon_tags(TMSourceFile *source_file)
 			{
 				TMTag *var_tag = TM_TAG(source_file->tags_array->pdata[j]);
 				guint var_scope_len = var_tag->scope ? strlen(var_tag->scope) : 0;
-				gchar *pos;
 
 				/* Should be at the same scope level as the anon tag */
-				if (var_scope_len == scope_len &&
-					var_tag->var_type && (pos = strstr(var_tag->var_type, orig_name)))
+				if (var_scope_len != scope_len || ! var_tag->var_type ||
+					! replace_str(&var_tag->var_type, orig_name, orig_name_len, new_name, new_name_len))
 				{
-					GString *str = g_string_new(var_tag->var_type);
-					gssize p = pos - var_tag->var_type;
-					g_string_erase(str, p, strlen(orig_name));
-					g_string_insert(str, p, new_name);
-					g_free(var_tag->var_type);
-					var_tag->var_type = str->str;
-					g_string_free(str, FALSE);
-				}
-				else
 					break;
+				}
 
 				j++;
 			}
@@ -435,6 +444,9 @@ void tm_ctags_parse(guchar *buffer, gsize buffer_size,
 {
 	g_return_if_fail(buffer != NULL || file_name != NULL);
 
+	if (language == TM_PARSER_NONE)
+		return;
+
 	parseRawBuffer(file_name, buffer, buffer_size, language, source_file);
 
 	rename_anon_tags(source_file);
@@ -443,6 +455,9 @@ void tm_ctags_parse(guchar *buffer, gsize buffer_size,
 
 const gchar *tm_ctags_get_lang_name(TMParserType lang)
 {
+	if (lang == TM_PARSER_NONE)
+		return "unknown";
+
 	return getLanguageName(lang);
 }
 
@@ -455,10 +470,15 @@ TMParserType tm_ctags_get_named_lang(const gchar *name)
 
 const gchar *tm_ctags_get_lang_kinds(TMParserType lang)
 {
-	guint kind_num = countLanguageKinds(lang);
 	static gchar kinds[257];
+	guint kind_num;
 	guint i;
 
+	if (lang == TM_PARSER_NONE)
+		return "";
+
+	kind_num = countLanguageKinds(lang);
+
 	for (i = 0; i < kind_num; i++)
 		kinds[i] = getLanguageKind(lang, i)->letter;
 	kinds[i] = '\0';
@@ -469,14 +489,22 @@ const gchar *tm_ctags_get_lang_kinds(TMParserType lang)
 
 const gchar *tm_ctags_get_kind_name(gchar kind, TMParserType lang)
 {
-	kindDefinition *def = getLanguageKindForLetter(lang, kind);
+	kindDefinition *def = NULL;
+
+	if (lang != TM_PARSER_NONE)
+		def = getLanguageKindForLetter(lang, kind);
+
 	return def ? def->name : "unknown";
 }
 
 
 gchar tm_ctags_get_kind_from_name(const gchar *name, TMParserType lang)
 {
-	kindDefinition *def = getLanguageKindForName(lang, name);
+	kindDefinition *def = NULL;
+
+	if (lang != TM_PARSER_NONE)
+		def = getLanguageKindForName(lang, name);
+
 	return def ? def->letter : '-';
 }
 
