@@ -681,31 +681,6 @@ gboolean tm_workspace_is_autocomplete_tag(TMTag *tag,
 }
 
 
-#if ! GLIB_CHECK_VERSION(2, 54, 0)
-static gboolean
-g_ptr_array_find (GPtrArray     *haystack,
-                  gconstpointer  needle,
-                  guint         *index_)
-{
-  guint i;
-
-  g_return_val_if_fail (haystack != NULL, FALSE);
-
-  for (i = 0; i < haystack->len; i++)
-    {
-      if (g_direct_equal (g_ptr_array_index (haystack, i), needle))
-        {
-          if (index_ != NULL)
-            *index_ = i;
-          return TRUE;
-        }
-    }
-
-  return FALSE;
-}
-#endif
-
-
 typedef struct
 {
 	TMSourceFile *file;
@@ -906,7 +881,12 @@ static gint sort_found_tags(gconstpointer a, gconstpointer b, gpointer user_data
 	 * followed by workspace tags,
 	 * followed by global tags */
 	if (t1->type & tm_tag_local_var_t && t2->type & tm_tag_local_var_t)
-		return info->sort_by_name ? g_strcmp0(t1->name, t2->name) : t2->line - t1->line;
+	{
+		if (info->sort_by_name)
+			return g_strcmp0(t1->name, t2->name);
+		else /* just like (t2->line - t1->line), but doesn't overflow converting to int */
+			return (t2->line > t1->line) ? 1 : ((t2->line < t1->line) ? -1 : 0);
+	}
 	else if (t1->type & tm_tag_local_var_t)
 		return -1;
 	else if (t2->type & tm_tag_local_var_t)
@@ -1066,7 +1046,7 @@ find_scope_members_tags (const GPtrArray *all, TMTag *type_tag, gboolean namespa
 	guint i;
 
 	if (depth == 10)
-		return NULL;  /* simple inheritence cycle avoidance */
+		return NULL;  /* simple inheritance cycle avoidance */
 
 	tags = g_ptr_array_new();
 
