@@ -183,144 +183,140 @@ void ui_set_statusbar(gboolean log, const gchar *format, ...)
 }
 
 
-/* note: some comments below are for translators */
-static gchar *create_statusbar_statistics(GeanyDocument *doc,
-	guint line, guint vcol, guint pos)
+/* get column from position, including virtual space */
+static gint get_vcol(ScintillaObject *sci, gint pos)
 {
-	const gchar *cur_tag;
-	const gchar *fmt;
-	const gchar *expos;	/* % expansion position */
-	const gchar sp[] = "      ";
-	GString *stats_str;
-	ScintillaObject *sci = doc->editor->sci;
+	gint col;
 
-	if (!EMPTY(ui_prefs.statusbar_template))
-		fmt = ui_prefs.statusbar_template;
+	/* Add temporary fix for sci infinite loop in Document::GetColumn(int)
+	 * when current pos is beyond document end (can occur when removing
+	 * blocks of selected lines especially esp. brace sections near end of file). */
+	if (pos <= sci_get_length(sci))
+		col = sci_get_col_from_position(sci, pos);
 	else
-		fmt = _(DEFAULT_STATUSBAR_TEMPLATE);
+		col = 0;
 
-	stats_str = g_string_sized_new(120);
+	return col + sci_get_cursor_virtual_space(sci);
+}
 
-	while ((expos = strchr(fmt, '%')) != NULL)
+
+/* note: some comments below are for translators */
+static gboolean insert_statusbar_statistics(GString *stats_str, const gchar placeholder, gpointer data)
+{
+	GeanyDocument *doc = data;
+	const gchar sp[] = "      ";
+	ScintillaObject *sci = doc->editor->sci;
+	gint pos = sci_get_current_position(sci);
+
+	switch (placeholder)
 	{
-		/* append leading text before % char */
-		g_string_append_len(stats_str, fmt, expos - fmt);
-
-		switch (*++expos)
-		{
-			case 'l':
-				g_string_append_printf(stats_str, "%d", line + 1);
-				break;
-			case 'L':
-				g_string_append_printf(stats_str, "%d",
-					sci_get_line_count(doc->editor->sci));
-				break;
-			case 'c':
-				g_string_append_printf(stats_str, "%d", vcol);
-				break;
-			case 'C':
-				g_string_append_printf(stats_str, "%d", vcol + 1);
-				break;
-			case 'p':
-				g_string_append_printf(stats_str, "%u", pos);
-				break;
-			case 's':
-			{
-				gint len = sci_get_selected_text_length2(sci);
-				/* check if whole lines are selected */
-				if (!len || sci_get_col_from_position(sci,
-						sci_get_selection_start(sci)) != 0 ||
-					sci_get_col_from_position(sci,
-						sci_get_selection_end(sci)) != 0)
-					g_string_append_printf(stats_str, "%d", len);
-				else /* L = lines */
-					g_string_append_printf(stats_str, _("%dL"),
-						sci_get_lines_selected(doc->editor->sci) - 1);
-				break;
-			}
-			case 'n' :
-				g_string_append_printf(stats_str, "%d",
-					sci_get_selected_text_length2(doc->editor->sci));
-				break;
-			case 'w':
-				/* RO = read-only */
-				g_string_append(stats_str, (doc->readonly) ? _("RO ") :
-					/* OVR = overwrite/overtype, INS = insert */
-					(sci_get_overtype(doc->editor->sci) ? _("OVR") : _("INS")));
-				break;
-			case 'r':
-				if (doc->readonly)
-				{
-					g_string_append(stats_str, _("RO "));	/* RO = read-only */
-					g_string_append(stats_str, sp + 1);
-				}
-				break;
-			case 't':
-			{
-				switch (editor_get_indent_prefs(doc->editor)->type)
-				{
-					case GEANY_INDENT_TYPE_TABS:
-						g_string_append(stats_str, _("TAB"));
-						break;
-					case GEANY_INDENT_TYPE_SPACES:	/* SP = space */
-						g_string_append(stats_str, _("SP"));
-						break;
-					case GEANY_INDENT_TYPE_BOTH:	/* T/S = tabs and spaces */
-						g_string_append(stats_str, _("T/S"));
-						break;
-				}
-				break;
-			}
-			case 'm':
-				if (doc->changed)
-				{
-					g_string_append(stats_str, _("MOD"));	/* MOD = modified */
-					g_string_append(stats_str, sp);
-				}
-				break;
-			case 'M':
-				g_string_append(stats_str, utils_get_eol_short_name(sci_get_eol_mode(doc->editor->sci)));
-				break;
-			case 'e':
-				g_string_append(stats_str,
-					doc->encoding ? doc->encoding : _("unknown"));
-				if (encodings_is_unicode_charset(doc->encoding) && (doc->has_bom))
-				{
-					g_string_append_c(stats_str, ' ');
-					g_string_append(stats_str, _("(with BOM)"));	/* BOM = byte order mark */
-				}
-				break;
-			case 'f':
-				g_string_append(stats_str, filetypes_get_display_name(doc->file_type));
-				break;
-			case 'S':
-				symbols_get_current_scope(doc, &cur_tag);
-				g_string_append(stats_str, cur_tag);
-				break;
-			case 'Y':
-				g_string_append_c(stats_str, ' ');
-				g_string_append_printf(stats_str, "%d",
-					sci_get_style_at(doc->editor->sci, pos));
-				break;
-			default:
-				g_string_append_len(stats_str, expos, 1);
-		}
-
-		/* skip past %c chars */
-		if (*expos)
-			fmt = expos + 1;
-		else
+		case 'l':
+			g_string_append_printf(stats_str, "%d", sci_get_line_from_position(sci, pos) + 1);
 			break;
+		case 'L':
+			g_string_append_printf(stats_str, "%d", sci_get_line_count(sci));
+			break;
+		case 'c':
+			g_string_append_printf(stats_str, "%d", get_vcol(sci, pos));
+			break;
+		case 'C':
+			g_string_append_printf(stats_str, "%d", get_vcol(sci, pos) + 1);
+			break;
+		case 'p':
+			g_string_append_printf(stats_str, "%u", pos);
+			break;
+		case 's':
+		{
+			gint len = sci_get_selected_text_length2(sci);
+			/* check if whole lines are selected */
+			if (!len || sci_get_col_from_position(sci,
+					sci_get_selection_start(sci)) != 0 ||
+				sci_get_col_from_position(sci,
+					sci_get_selection_end(sci)) != 0)
+				g_string_append_printf(stats_str, "%d", len);
+			else /* Translators: L = lines */
+				g_string_append_printf(stats_str, _("%dL"),
+					sci_get_lines_selected(sci) - 1);
+			break;
+		}
+		case 'n' :
+			g_string_append_printf(stats_str, "%d",
+				sci_get_selected_text_length2(sci));
+			break;
+		case 'w':
+			/* Translators: RO = read-only */
+			g_string_append(stats_str, (doc->readonly) ? _("RO ") :
+				/* Translators: OVR = overwrite/overtype, INS = insert */
+				(sci_get_overtype(sci) ? _("OVR") : _("INS")));
+			break;
+		case 'r':
+			if (doc->readonly)
+			{
+				g_string_append(stats_str, _("RO "));	/* Translators: RO = read-only */
+				g_string_append(stats_str, sp + 1);
+			}
+			break;
+		case 't':
+		{
+			switch (editor_get_indent_prefs(doc->editor)->type)
+			{
+				case GEANY_INDENT_TYPE_TABS:
+					g_string_append(stats_str, _("TAB"));
+					break;
+				case GEANY_INDENT_TYPE_SPACES:	/* Translators: SP = space */
+					g_string_append(stats_str, _("SP"));
+					break;
+				case GEANY_INDENT_TYPE_BOTH:	/* Translators: T/S = tabs and spaces */
+					g_string_append(stats_str, _("T/S"));
+					break;
+			}
+			break;
+		}
+		case 'm':
+			if (doc->changed)
+			{
+				/* Translators: MOD = modified */
+				g_string_append(stats_str, _("MOD"));
+				g_string_append(stats_str, sp);
+			}
+			break;
+		case 'M':
+			g_string_append(stats_str, utils_get_eol_short_name(sci_get_eol_mode(sci)));
+			break;
+		case 'e':
+			g_string_append(stats_str,
+				doc->encoding ? doc->encoding : _("unknown"));
+			if (encodings_is_unicode_charset(doc->encoding) && (doc->has_bom))
+			{
+				g_string_append_c(stats_str, ' ');
+				/* Translators: BOM = byte order mark */
+				g_string_append(stats_str, _("(with BOM)"));
+			}
+			break;
+		case 'f':
+			g_string_append(stats_str, filetypes_get_display_name(doc->file_type));
+			break;
+		case 'S':
+		{
+			const gchar *cur_tag;
+			symbols_get_current_scope(doc, &cur_tag);
+			g_string_append(stats_str, cur_tag);
+			break;
+		}
+		case 'Y':
+			g_string_append_c(stats_str, ' ');
+			g_string_append_printf(stats_str, "%d", sci_get_style_at(sci, pos));
+			break;
+		default:
+			return FALSE;
 	}
-	/* add any remaining text */
-	g_string_append(stats_str, fmt);
 
-	return g_string_free(stats_str, FALSE);
+	return TRUE;
 }
 
 
 /* updates the status bar document statistics */
-void ui_update_statusbar(GeanyDocument *doc, gint pos)
+void ui_update_statusbar(GeanyDocument *doc)
 {
 	g_return_if_fail(doc == NULL || doc->is_valid);
 
@@ -332,23 +328,15 @@ void ui_update_statusbar(GeanyDocument *doc, gint pos)
 
 	if (doc != NULL)
 	{
-		guint line, vcol;
+		const gchar *fmt;
 		gchar *stats_str;
 
-		if (pos == -1)
-			pos = sci_get_current_position(doc->editor->sci);
-		line = sci_get_line_from_position(doc->editor->sci, pos);
-
-		/* Add temporary fix for sci infinite loop in Document::GetColumn(int)
-		 * when current pos is beyond document end (can occur when removing
-		 * blocks of selected lines especially esp. brace sections near end of file). */
-		if (pos <= sci_get_length(doc->editor->sci))
-			vcol = sci_get_col_from_position(doc->editor->sci, pos);
+		if (!EMPTY(ui_prefs.statusbar_template))
+			fmt = ui_prefs.statusbar_template;
 		else
-			vcol = 0;
-		vcol += sci_get_cursor_virtual_space(doc->editor->sci);
+			fmt = _(DEFAULT_STATUSBAR_TEMPLATE);
 
-		stats_str = create_statusbar_statistics(doc, line, vcol, pos);
+		stats_str = utils_replace_placeholders(fmt, insert_statusbar_statistics, doc);
 
 		/* can be overridden by status messages */
 		set_statusbar(stats_str, TRUE);
@@ -528,11 +516,9 @@ void ui_update_menu_copy_items(GeanyDocument *doc)
 
 	if (IS_SCINTILLA(focusw))
 		enable = (doc == NULL) ? FALSE : sci_has_selection(doc->editor->sci);
-	else
-	if (GTK_IS_EDITABLE(focusw))
+	else if (GTK_IS_EDITABLE(focusw))
 		enable = gtk_editable_get_selection_bounds(GTK_EDITABLE(focusw), NULL, NULL);
-	else
-	if (GTK_IS_TEXT_VIEW(focusw))
+	else if (GTK_IS_TEXT_VIEW(focusw))
 	{
 		GtkTextBuffer *buffer = gtk_text_view_get_buffer(
 			GTK_TEXT_VIEW(focusw));
@@ -726,8 +712,7 @@ static void insert_date(GeanyDocument *doc, gint pos, const gchar *date_style)
 	else
 	{
 		gchar *str = dialogs_show_input(_("Custom Date Format"), GTK_WINDOW(main_widgets.window),
-				_("Enter here a custom date and time format. "
-				"For a list of available conversion specifiers see https://docs.gtk.org/glib/method.DateTime.format.html."),
+				_("Custom date format (see https://docs.gtk.org/glib/method.DateTime.format.html):"),
 				ui_prefs.custom_date_format);
 		if (str)
 			SETPTR(ui_prefs.custom_date_format, str);
@@ -1101,8 +1086,13 @@ void ui_document_show_hide(GeanyDocument *doc)
 }
 
 
+/* success = FALSE indicates an error for widget */
 void ui_set_search_entry_background(GtkWidget *widget, gboolean success)
 {
+	// set the entry, not just the button
+	if (GTK_IS_COMBO_BOX(widget))
+		widget = gtk_bin_get_child(GTK_BIN(widget));
+
 	gtk_widget_set_name(widget, success ? NULL : "geany-search-entry-no-match");
 }
 
@@ -1637,33 +1627,6 @@ void ui_entry_add_activate_backward_signal(GtkEntry *entry)
 }
 
 
-static void add_to_size_group(GtkWidget *widget, gpointer size_group)
-{
-	g_return_if_fail(GTK_IS_SIZE_GROUP(size_group));
-	gtk_size_group_add_widget(GTK_SIZE_GROUP(size_group), widget);
-}
-
-
-/* Copies the spacing and layout of the master GtkHButtonBox and synchronises
- * the width of each button box's children.
- * Should be called after all child widgets have been packed. */
-void ui_hbutton_box_copy_layout(GtkButtonBox *master, GtkButtonBox *copy)
-{
-	GtkSizeGroup *size_group;
-
-	gtk_box_set_spacing(GTK_BOX(copy), 10);
-	gtk_button_box_set_layout(copy, gtk_button_box_get_layout(master));
-
-	/* now we need to put the widest widget from each button box in a size group,
-	* but we don't know the width before they are drawn, and for different label
-	* translations the widest widget can vary, so we just add all widgets. */
-	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
-	gtk_container_foreach(GTK_CONTAINER(master), add_to_size_group, size_group);
-	gtk_container_foreach(GTK_CONTAINER(copy), add_to_size_group, size_group);
-	g_object_unref(size_group);
-}
-
-
 static gboolean tree_model_find_text(GtkTreeModel *model,
 		GtkTreeIter *iter, gint column, const gchar *text)
 {
@@ -1959,43 +1922,49 @@ void ui_setup_open_button_callback(GtkWidget *open_btn, const gchar *title,
 }
 
 
-#ifndef G_OS_WIN32
 static gchar *run_file_chooser(const gchar *title, GtkFileChooserAction action,
 		const gchar *utf8_path)
 {
-	GtkWidget *dialog = gtk_file_chooser_dialog_new(title,
-		GTK_WINDOW(main_widgets.window), action,
-		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);
+	GtkFileChooser *dialog;
 	gchar *locale_path;
 	gchar *ret_path = NULL;
 
-	gtk_widget_set_name(dialog, "GeanyDialog");
+	if (interface_prefs.use_native_windows_dialogs)
+		dialog = GTK_FILE_CHOOSER(gtk_file_chooser_native_new(title,
+			GTK_WINDOW(main_widgets.window), action, NULL, NULL));
+	else
+	{
+		dialog = GTK_FILE_CHOOSER(gtk_file_chooser_dialog_new(title,
+			GTK_WINDOW(main_widgets.window), action,
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL));
+		gtk_widget_set_name(GTK_WIDGET(dialog), "GeanyDialog");
+	}
+
 	locale_path = utils_get_locale_from_utf8(utf8_path);
 	if (action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)
 	{
 		if (g_path_is_absolute(locale_path) && g_file_test(locale_path, G_FILE_TEST_IS_DIR))
-			gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), locale_path);
+			gtk_file_chooser_set_current_folder(dialog, locale_path);
 	}
 	else if (action == GTK_FILE_CHOOSER_ACTION_OPEN)
 	{
 		if (g_path_is_absolute(locale_path))
-			gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog), locale_path);
+			gtk_file_chooser_set_filename(dialog, locale_path);
 	}
 	g_free(locale_path);
 
-	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
+	if (dialogs_file_chooser_run(dialog) == GTK_RESPONSE_ACCEPT)
 	{
 		gchar *dir_locale;
 
-		dir_locale = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+		dir_locale = gtk_file_chooser_get_filename(dialog);
 		ret_path = utils_get_utf8_from_locale(dir_locale);
 		g_free(dir_locale);
 	}
-	gtk_widget_destroy(dialog);
+	dialogs_file_chooser_destroy(dialog);
 	return ret_path;
 }
-#endif
 
 
 gchar *ui_get_project_directory(const gchar *path)
@@ -2003,11 +1972,7 @@ gchar *ui_get_project_directory(const gchar *path)
 	gchar *utf8_path;
 	const gchar *title = _("Select Project Base Path");
 
-#ifdef G_OS_WIN32
-	utf8_path = win32_show_folder_dialog(ui_widgets.prefs_dialog, title, path);
-#else
 	utf8_path = run_file_chooser(title, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, path);
-#endif
 
 	return utf8_path;
 }
@@ -2030,22 +1995,12 @@ static void ui_path_box_open_clicked(GtkButton *button, gpointer user_data)
 
 	if (action == GTK_FILE_CHOOSER_ACTION_OPEN)
 	{
-#ifdef G_OS_WIN32
-		utf8_path = win32_show_file_dialog(GTK_WINDOW(ui_widgets.prefs_dialog), title,
-						gtk_entry_get_text(GTK_ENTRY(entry)));
-#else
 		utf8_path = run_file_chooser(title, action, gtk_entry_get_text(GTK_ENTRY(entry)));
-#endif
 	}
 	else if (action == GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)
 	{
 		gchar *path = g_path_get_dirname(gtk_entry_get_text(GTK_ENTRY(entry)));
-#ifdef G_OS_WIN32
-		utf8_path = win32_show_folder_dialog(ui_widgets.prefs_dialog, title,
-						gtk_entry_get_text(GTK_ENTRY(entry)));
-#else
 		utf8_path = run_file_chooser(title, action, path);
-#endif
 		g_free(path);
 	}
 
@@ -2063,7 +2018,7 @@ void ui_statusbar_showhide(gboolean state)
 	if (state)
 	{
 		gtk_widget_show(ui_widgets.statusbar);
-		ui_update_statusbar(NULL, -1);
+		ui_update_statusbar(NULL);
 	}
 	else
 		gtk_widget_hide(ui_widgets.statusbar);
@@ -2256,11 +2211,11 @@ static void add_stock_icons(const GtkStockItem *items, gsize count)
 
 void ui_init_stock_items(void)
 {
-	GtkStockItem items[] =
+	const GtkStockItem items[] =
 	{
-		{ GEANY_STOCK_SAVE_ALL, N_("Save All"), 0, 0, GETTEXT_PACKAGE },
-		{ GEANY_STOCK_CLOSE_ALL, N_("Close All"), 0, 0, GETTEXT_PACKAGE },
-		{ GEANY_STOCK_BUILD, N_("Build"), 0, 0, GETTEXT_PACKAGE }
+		{ (gchar *) GEANY_STOCK_SAVE_ALL, (gchar *) N_("Save All"), 0, 0, (gchar *) GETTEXT_PACKAGE },
+		{ (gchar *) GEANY_STOCK_CLOSE_ALL, (gchar *) N_("Close All"), 0, 0, (gchar *) GETTEXT_PACKAGE },
+		{ (gchar *) GEANY_STOCK_BUILD, (gchar *) N_("Build"), 0, 0, (gchar *) GETTEXT_PACKAGE }
 	};
 
 	gtk_stock_add(items, G_N_ELEMENTS(items));
@@ -2527,6 +2482,11 @@ void ui_init_builder(void)
 
 		widget = GTK_WIDGET(iter->data);
 
+#ifdef G_OS_WIN32
+		if (GTK_IS_WINDOW(widget))
+			win32_update_titlebar_theme(widget);
+#endif
+
 		name = ui_guess_object_name(G_OBJECT(widget));
 		if (! name)
 		{
@@ -2576,33 +2536,6 @@ static void init_css_styles(void)
 	load_css_theme(theme_fn, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 	g_free(theme_fn);
 
-	// load themes to handle breakage between various GTK+ versions
-	const struct
-	{
-		guint min_version;
-		guint max_version;
-		const gchar *file;
-	}
-	css_files[] =
-	{
-		/* Unused now but can be used to load css for different GTK versions, such as
-		 * { 20, G_MAXUINT, "geany-3.20.css" },
-		 * { 0, 19, "geany-3.0.css" },
-		 */
-	};
-
-	guint gtk_version = gtk_get_minor_version();
-	for (guint i = 0; i < G_N_ELEMENTS(css_files); i++)
-	{
-		if (gtk_version >= css_files[i].min_version &&
-			gtk_version <= css_files[i].max_version)
-		{
-			theme_fn = g_build_filename(app->datadir, css_files[i].file, NULL);
-			load_css_theme(theme_fn, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-			g_free(theme_fn);
-		}
-	}
-
 	// if the user provided a geany.css file in their config dir, try and load that
 	theme_fn = g_build_filename(app->configdir, "geany.css", NULL);
 	if (g_file_test(theme_fn, G_FILE_TEST_EXISTS))
@@ -2893,6 +2826,10 @@ void ui_progress_bar_stop(void)
 		g_source_remove(progress_bar_timer_id);
 		progress_bar_timer_id = 0;
 	}
+
+	/* hack to remove tick callback which is created for "activity mode" progress
+	 * bars - without this it is called forever and causes elevated CPU usage */
+	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(main_widgets.progressbar), 0.0);
 }
 
 
