@@ -1,19 +1,18 @@
 #![cfg(not(test))] fn not_hashbang() {}
 
-pub fn foo_bar_test_func(apples:fruit::SomeStruct,(oranges,lemon):(int,int))->int{
-	use std::io::stdio::println;
+pub fn foo_bar_test_func(apples:fruit::SomeStruct,(oranges,lemon):(isize,isize))->isize{
 	let some_var_name=2*oranges;
 	let a=SomeLongStructName{v:0};
-	println("a");println("b");	println("c");
+	println!("{}", "a");
 	veg::another_function(apples.red_value,oranges,lemon);
 	some_var_name-apples.red_value+lemon+a.v
 }
 
 pub mod fruit {
 	pub struct SomeStruct{
-		pub red_value: int,
-		pub green_value: int,
-		pub blue_value: int
+		pub red_value: isize,
+		pub green_value: isize,
+		pub blue_value: isize
 	}
 }
 
@@ -27,10 +26,10 @@ impl SomeLongStructName {
 	}
 }
 
-pub struct SomeLongStructName {v:int}
+pub struct SomeLongStructName {v:isize}
 
 mod veg{
-	pub fn another_function(a:int,b:int,c:int)->int {
+	pub fn another_function(a:isize,b:isize,c:isize)->isize {
 		a+b+c
 	}
 }
@@ -43,3 +42,5 @@ mod mineral {
 	fn chalk() {
 	}
 }
+
+fn main() {}
